forward_function ch3a_active, vpp_to_noaa19

pro composite_lst_vpp,flns,year,jday,satnum
;---------------------------------------------------------------------------
; This program composites NOAA-16 AVHRR GAC data collected throughout a day 
; into two images per day at local solar time lst for each polar region.  
; The input images are from the CCAR 'navigate' program.
;
; Input:
;   flns   : An array of file names of the GAC data.  Each name is the 
;            root of the file name without the ".1", ".SZA", etc. extension
;            but with the hemisphere label (".N1", ".N2", and ".S").
;   year   : 4-digit year
;   jday   : julian day
;   lst    : local solar time when the composite is 
;   satnum  : satellite number
;
; Output files:
;   Two images with the names ""north.lst.##.img" and "south.lst.##.img" for the northern 
;   and southern polar regions at lcoal solar time lst.
;
;  Calls:
;    read_merge
;    drop_in
;    ch3a_active
;
;  Modified by Chia Moeller:
;    change file name from app_ to vpp_
;    composite_lst_vpp_int_cal3.pro
;---------------------------------------------------------------------------

; Output file names
    lst_ar1 = 4
    lst_ar2 = 14
    lst_an1 = 2
    lst_an2 = 14
    Nfile1 = 'vpp_n005_'+string(year,format='(i04)')+string(jday,format='(i03)')+'_'+string(lst_ar1,format='(i02)')+'00.img'
    Nfile2 = 'vpp_n005_'+string(year,format='(i04)')+string(jday,format='(i03)')+'_'+string(lst_ar2,format='(i02)')+'00.img'
    Sfile1 = 'vpp_s005_'+string(year,format='(i04)')+string(jday,format='(i03)')+'_'+string(lst_an1,format='(i02)')+'00.img'
    Sfile2 = 'vpp_s005_'+string(year,format='(i04)')+string(jday,format='(i03)')+'_'+string(lst_an2,format='(i02)')+'00.img'
    
    str_pos1 = 15
    ;str_pos1 = 22
    str_len1 = 3
    str_pos2 = 20
    ;str_pos2 = 27
    str_len2 = 2
    str_pos3 = 26
    ;str_pos3 = 33
    str_len3 = 2


; Define the output images
; Arctic
;************ data file used to convert GMT (UTC) to mlocal solar time ********************
time_corr_fn = '/home/chial/IDL/t5n_corr.dat' ;'t5n_corr.dat'          ;;arctic
ncols = 1805 & nrows = 1805  & time_corr = fltarr(ncols,nrows)
openr,ur,time_corr_fn,/get_lun
      readu,ur,time_corr
close,ur
;************ convert GMT (UTC) to mlocal solar time ********************

dimxyN = 1805
nfields = 9
img1 = intarr(dimxyN,dimxyN,nfields)
img2 = intarr(dimxyN,dimxyN,nfields)
score1 = fltarr(dimxyN,dimxyN) + 100.   ; Scan-time score. 0=best
score2 = fltarr(dimxyN,dimxyN) + 100.   ; Scan-time score. 0=best

; READ the images one at a time and process them, first the northern 
; hemisphere (two sets of images; see 'extract') then the southern.

; Determine if AVHRR channel 3A is used in the sunlit portions

;*************************************************************************
satnum = satnum   
caldat,julday(1,1,year)+jday-1,mo,dy,yy
use_ch3a = ch3a_active(satnum,year,mo,dy)
;***************************need address this ****************************

; Northern  hemisphere
satjday = 0
hour_start = 0
hour_end   = 0

iwhere = where(strmatch(flns,'*.N1') eq 1,count)  ; set 1
if count gt 0 then flns2 = flns[iwhere]
nflns = n_elements(flns2)

for i=0,nflns-1 do begin 
  fln = flns2[i]
  print,'Processing file: ', fln
  
  reads,strmid(fln,str_pos1,str_len1),satjday
  
  hourdiff = (satjday-jday)*24.
  
  ;--------------------------------------------------------------------------
  if jday eq 1 and (satjday eq 365 or satjday eq 366) then hourdiff = -24.
  if (jday eq 365 or jday eq 366) and satjday eq 1 then hourdiff = 24.
  ;--------------------------------------------------------------------------
  
  reads,strmid(fln,str_pos2,str_len2),hour_start
  reads,strmid(fln,str_pos3,str_len3),hour_end

  if hour_end gt hour_start then flag=0 else flag=1  
  
  ncols = dimxyN
  nrows = ncols
  read_merge,fln,ncols,nrows,img,use_ch3a
  drop_in,img,img1,score1,lst_ar1,hourdiff,flag,time_corr
  drop_in,img,img2,score2,lst_ar2,hourdiff,flag,time_corr

  print, ' ... N1 ...'
  print, ' hourdiff = ', hourdiff
  print, ' hour_start = ', hour_start
  print, ' hour_end = ', hour_end
endfor

iwhere = where(strmatch(flns,'*.N2') eq 1,count)  ; set 2
if count gt 0 then flns2 = flns[iwhere]
nflns = n_elements(flns2)

for i=0,nflns-1 do begin 
  fln = flns2[i]
  print,'Processing file: ', fln
  
  reads,strmid(fln,str_pos1,str_len1),satjday
  hourdiff = (satjday-jday)*24.
  
  ;--------------------------------------------------------------------------
  if jday eq 1 and (satjday eq 365 or satjday eq 366) then hourdiff = -24.
  if (jday eq 365 or jday eq 366) and satjday eq 1 then hourdiff = 24.
  ;--------------------------------------------------------------------------
  
  reads,strmid(fln,str_pos2,str_len2),hour_start
  reads,strmid(fln,str_pos3,str_len3),hour_end

  if hour_end gt hour_start then flag=0 else flag=1
  
  ncols = dimxyN
  nrows = ncols
  read_merge,fln,ncols,nrows,img,use_ch3a
  drop_in,img,img1,score1,lst_ar1,hourdiff,flag,time_corr
  drop_in,img,img2,score2,lst_ar2,hourdiff,flag,time_corr

  print, ' ... N2 ...'
  print, ' hourdiff = ', hourdiff
  print, ' hour_start = ', hour_start
  print, ' hour_end = ', hour_end
endfor

;  Intercalibration done below
vpp_to_noaa19, img1, lst_ar1, 0

;  WRITE the composites.
openw,u,Nfile1,/get_lun
writeu,u,img1
free_lun,u

;  Intercalibration done below
vpp_to_noaa19, img2, lst_ar2, 0

;  WRITE the composites.
openw,u,Nfile2,/get_lun
writeu,u,img2
free_lun,u


; Southern  hemisphere
;Antarctica
;************ data file used to convert GMT (UTC) to mlocal solar time ********************
;time_corr_fn = '/home/rdworak/APP_code_update/t5s_corr.dat' ;'t5s_corr.dat'          ;;antarctic
time_corr_fn = '/home/chial/IDL/t5s_corr.dat' ;
ncols = 1605 & nrows = 1605  & time_corr = fltarr(ncols,nrows)
openr,ur,time_corr_fn,/get_lun
      readu,ur,time_corr
close,ur
;************ convert GMT (UTC) to mlocal solar time ********************
dimxyS = 1605
img1 = intarr(dimxyS,dimxyS,nfields)
img2 = intarr(dimxyS,dimxyS,nfields)
score1 = fltarr(dimxyS,dimxyS) + 100.   ; Scan-time score. 0=best
score2 = fltarr(dimxyS,dimxyS) + 100.   ; Scan-time score. 0=best
satjday = 0
hour_start = 0
hour_end   = 0

iwhere = where(strmatch(flns,'*.S1') eq 1,count)  ; set 1
if count gt 0 then flns2 = flns[iwhere]
nflns = n_elements(flns2)

for i=0,nflns-1 do begin 
  fln = flns2[i]
  print,'Processing file: ', fln
  
  reads,strmid(fln,str_pos1,str_len1),satjday
  hourdiff = (satjday-jday)*24.
  
  ;--------------------------------------------------------------------------
  if jday eq 1 and (satjday eq 365 or satjday eq 366) then hourdiff = -24.
  if (jday eq 365 or jday eq 366) and satjday eq 1 then hourdiff = 24.
  ;--------------------------------------------------------------------------
  
  reads,strmid(fln,str_pos2,str_len2),hour_start
  reads,strmid(fln,str_pos3,str_len3),hour_end

  if hour_end gt hour_start then flag=0 else flag=1  
  
  ncols = dimxyS
  nrows = ncols
  read_merge,fln,ncols,nrows,img,use_ch3a
  drop_in,img,img1,score1,lst_an1,hourdiff,flag,time_corr
  drop_in,img,img2,score2,lst_an2,hourdiff,flag,time_corr

  print, ' ... S1 ...'
  print, ' hourdiff = ', hourdiff
  print, ' hour_start = ', hour_start
  print, ' hour_end = ', hour_end
endfor

iwhere = where(strmatch(flns,'*.S2') eq 1,count)  ; set 2
if count gt 0 then flns2 = flns[iwhere]
nflns = n_elements(flns2)

for i=0,nflns-1 do begin 
  fln = flns2[i]
  print,'Processing file: ', fln
  
  reads,strmid(fln,str_pos1,str_len1),satjday
  hourdiff = (satjday-jday)*24.
  
  ;--------------------------------------------------------------------------
  if jday eq 1 and (satjday eq 365 or satjday eq 366) then hourdiff = -24.
  if (jday eq 365 or jday eq 366) and satjday eq 1 then hourdiff = 24.
  ;--------------------------------------------------------------------------
  
  reads,strmid(fln,str_pos2,str_len2),hour_start
  reads,strmid(fln,str_pos3,str_len3),hour_end

  if hour_end gt hour_start then flag=0 else flag=1
  
  ncols = dimxyS
  nrows = ncols
  read_merge,fln,ncols,nrows,img,use_ch3a
  drop_in,img,img1,score1,lst_an1,hourdiff,flag,time_corr
  drop_in,img,img2,score2,lst_an2,hourdiff,flag,time_corr

  print, ' ... S1 ...'
  print, ' hourdiff = ', hourdiff
  print, ' hour_start = ', hour_start
  print, ' hour_end = ', hour_end
endfor
print,'  Writing the composite...'

;  Intercalibration done below
vpp_to_noaa19, img1, lst_an1, 1

;  WRITE the composites.
openw,u,Sfile1,/get_lun
writeu,u,img1
free_lun,u

;  Intercalibration done below
vpp_to_noaa19, img2, lst_an2, 1

;  WRITE the composites.
openw,u,Sfile2,/get_lun
writeu,u,img2
free_lun,u

print,'Done compositing.'

end


;************************** READ AND MERGE THE AVHRR DATA *******************

pro read_merge,fln,ncols,nrows,imageall,use_ch3a
;---------------------------------------------------------------------------------------------------------------
;  Reads and merges the CCAR AVHRR GAC data into a single array, and does
;  some conversions.
;
;  The input data type is UNSIGNED short (2-byte) integer with units
;  as described below for the output format.
;
;  The new format is:
;     CH1 % ALBEDO * 1000, NORMALIZED BY THE COSINE OF THE SOLAR ZEN. ANG.
;     CH2 % ALBEDO * 1000, NORMALIZED BY THE COSINE OF THE SOLAR ZEN. ANG.
;     CH3 TB * 10 and/or % ALBEDO *10.  For NOAA-16 and higher, the sunlit portion
;       of channel 3A will be normalized by the cosine of the solar zenith angle;
;       the dark portion will remain as Tbs*10..
;     CH4 TB * 10
;     CH5 TB * 10
;     SCAN ANGLE (DEGREES) * 10
;     SOLAR ZENITH ANGLE (DEGREES) * 10
;     RELATIVE AZIMUTH ANGLE (DEGREES) * 10
;     TIME (GMT) * 10
;  All output values are SIGNED short (2-byte) integer.
;
;  New format 4-2022 Chia Moeller
;   APP Albedo is 0-100, therefore scale by 10 to 1000 signed integer.
;   VPP Albedo is 0-1, therefore scale by 1000 to 1000 signed integer.
;     The scaling should be done in remapping, ie read_hdf_gac_noaa_VGAC.pro, to avoid missing value messing up.
;---------------------------------------------------------------------------------------------------------------

;print,'Working on ', fln

nfields = 10
field=strarr(nfields)
field(0)=fln+'.1'
field(1)=fln+'.2'
field(2)=fln+'.3'
field(3)=fln+'.4'
field(4)=fln+'.5'
; converted satellite zenith angle to satellite elevation angle in remap: elevation angle = 90. - sensor_zenith_angle, confirmed
field(5)=fln+'.ELEV' ; sat_elev in remap
field(6)=fln+'.SZA'  ; szaa in remap ; Solar Zenith Angle  Chia Moeller 5-2022
field(7)=fln+'.AZIM' ; solaz in rempap; Solar Azimuth Angle. 
field(8)=fln+'.SAZA' ; sataz in rempap; Satellite Azimuth Angle
field(9)=fln+'.TIME' ; utc_app in remap

;  READ the data for fields 0-6. 

imageall = intarr(ncols,nrows,nfields-1)
labels = ['ch1','ch2','ch3','ch4','ch5','Scan angle']

for i=0,5 do begin
  image = uintarr(ncols,nrows)
  openr,u,field[i],/get_lun
  readu,u,image
  close,u
  free_lun,u

; x86-64 is little endian. read_VGAC.pro swap_endian from l->b for composite use. 
  if (!version.arch ne 'ppc') then begin
    image = swap_endian(image)
    print, 'Swapped Endian!'
  endif
  convertint,image,labels[i]

; DETERMINE SCAN ANGLE
  if (i eq 5) then begin
;    print,'  Converting to scan angle...'
    elev2scan,image
    image = fix(image)
    print,"Min/max in Scan Angle : ",min(image), max(image)
  endif else begin
    print,"Min/max in image ",i,": ",min(image), max(image)
  endelse

  imageall(*,*,i) = image(*,*)

endfor

; Read the Solar Zenith Angle image
image = uintarr(ncols,nrows)
openr,u,field[6],/get_lun
readu,u,image
free_lun,u
if (!version.arch ne 'ppc') then begin
  image = swap_endian(image)
  print, 'Swapped Endian!'
endif
convertint,image,'Solar zenith angle'
imageall(*,*,6) = image(*,*)
print,"Min/max in Solar Zenith Angle : ",min(image), max(image)

; DETERMINE RELATIVE AZIMUTH
relaz,field[7],field[8],ncols,nrows,image
imageall(*,*,7) = image(*,*)
print,"Min/max in Relative Azimuth Angle : ",min(image), max(image)

; Read the TIME image
image = uintarr(ncols,nrows)
openr,u,field[9],/get_lun
readu,u,image
free_lun,u
if (!version.arch ne 'ppc') then begin
  image = swap_endian(image)
  print, 'Swapped Endian!'
endif
convertint,image,'Time'
imageall(*,*,8) = image(*,*)

return
end

;************************* DROP AN IMAGE INTO THE COMPOSITE ****************

pro drop_in,img,img12z,oldscore,lst,hourdiff,flag,time_corr,utc=utc
;---------------------------------------------------------------------------
; Adds an image to the local solar time lst composite, depending on the scan angles
; and time of acquisition.
;
; Input                                                   
;   img     : The image to be added (ncols x nrows x 9; see 'read_merge')
;   img12z  : The 12Z composite image (ncols x nrows x 8; no time item)
;   oldscore: The previously computed "score" for each pixel based on the
;             scan angle and time (ncols x nrows) [fltarr]
;   hourdiff: added to account the hour difference from previous and next day
;   time_corr: data files used to convert UTC to Local Solar Time, 
;              this file is generated based on longitude of grid
;   UTC:      keyword, if set, the composite is done at set UTC,
;             otherwise at set Local Solar Time                                                 
; Output                                             
;   img12z  : The local solar time lst composite image, same size as 'img'.  
;              (ncols x nrows x 8) [intarr]
;   oldscore: The new scores for those pixels that had lower scores than
;             before.  Otherwise the pixel scores are unchanged.
;             (ncols x nrows) [fltarr], here it is satellite scan angle
;---------------------------------------------------------------------------
TARGETTIME = lst        ; Target local solar time, or UTC is keyword UTC set
MAXTDIFF = 3.0           ; Maximum time difference from target time allowed
;MAXTDIFF = 4.0           ; Maximum time difference from target time allowed Chia Moeller
;SCANWGT = 0.4           ; Weight for scan angle in score function
;TIMEWGT = 0.6           ; Weight for time in score function
;---------------------------------------------------------------------------

; Determine which part of the input image is not filled.  Check channel 4.
; The nav/cal programs fill the rest of the image with the minimum value
; within the valid part (?).  Looking for a time of 0 probably gets it
; all anyway.  'badpix' will have values of 0 for good pixels  and 1 for
; bad pixels.

;badpix = bytarr(ncols,nrows)*byte(0)
;ibad = where(img(*,*,3) eq min(img(*,*,3)),count)
;if count gt 0 then badpix(ibad) = 1
;ibad = where(img(*,*,3) lt 1500 or img(*,*,3) gt 4000 or img(*,*,8) eq 0,count)
badpix = byte(oldscore)*byte(0)
ibad = where(img(*,*,3) lt 1500 or img(*,*,3) gt 4000, count)
if count gt 0 then badpix(ibad) = 1

igood = where(badpix eq 0,count)
if count eq 0 then begin
  print,'DROP_IN: NO GOOD PIXELS.  THE IMAGE WILL BE SKIPPED.'
  return
end

; Get the average time and determine if it is close enough in time to the
; composite to be used. Don't use the image if its time is more than MAXTDIFF
; hours before or after the target time.  Compute a scoring function based
; on the scan angle and time from target time (lower score is better), then
; replace the previous image values for those pixels with lower scores
; than before.

time = img(*,*,8)          ; time is input as hhmms, where s=seconds/10.
hhmm = fix(time/10.)
hh = fix(hhmm/100.)
mm = hhmm - hh*100
ss = (time/10. - hhmm) * 100.
dhr = hh + mm/60. + ss/3600.

;;; some granules have values larger than 24 for the observations 
;;; from next day, e.g. starting time is current day, but ending time is 
;;; the day after. these values are subtracted by 24
xxx = where(dhr gt 24., cxx)
if cxx gt 0 then dhr(xxx) = dhr(xxx)-24. 
;;; adjustment for that starting and ending day are different
if flag eq 1 then begin
   xxx = where(dhr gt 0. and dhr lt 12., cxx)
   if cxx gt 0 then dhr(xxx) = dhr(xxx) + 24.
endif
;;; adjustment for day changes
dhr = dhr + hourdiff

;************ convert GMT (UTC) to mlocal solar time ********************
if keyword_set(utc) eq 0 then  dhr = dhr + time_corr
;************ convert GMT (UTC) to mlocal solar time ********************

  scan = reform(img(*,*,5)/10.)
   
  ;x = where(scan(igood) gt 60., cx)
  ;tdiff1 = abs( dhr - TARGETTIME + 24)
  ;tdiff2 = abs( dhr - TARGETTIME + 0)
  ;tdiff3 = abs( dhr - TARGETTIME - 24)
  ;tdiff  = tdiff1<tdiff2<tdiff3    
  ;ibetter = where(tdiff lt MAXTDIFF and scan lt oldscore and badpix eq 0,count)

  tdiff = abs( dhr - TARGETTIME + 0)
  ibetter = where(tdiff lt MAXTDIFF and scan lt oldscore and badpix eq 0,count)
  
  if count gt 0 then begin
    print,'  Dropping image into composite...'

    for i=0,8 do begin
      tmpcomp = img12z(*,*,i)
      tmpimg = img(*,*,i)
      tmpcomp(ibetter) = tmpimg(ibetter)
      img12z(*,*,i) = tmpcomp(*,*)
    endfor
    oldscore(ibetter) = scan(ibetter)
  endif else begin
    print,'  This image will not be used (not any better than current composite).'
  endelse

end


;*************************** CONVERT TO SCAN ANGLE *************************

pro elev2scan,image
;---------------------------------------------------------------------------
; CONVERTS SATELLITE ELEVATION ANGLE TO SCAN ANGLE
;
; Input                                                   
;   eang   :  The satellite elevation angle (degrees * 10)
;                                                    
; Output                                             
;   scan   :  The satellite scan angle (degrees * 10)
;
; Re is the radius of the earth, Hsat is the nominal height of the
; satellite, and ReHsat = Re + Hsat
;---------------------------------------------------------------------------

eang = image/10.
degrad = 0.017453292 
Rearth = 6357. 
Hsat = 850. 
ReHsat = 7207.0 
zenang = (90. - eang) * degrad 
scan = asin(Rearth*sin(zenang)/(ReHsat)) / degrad 
image = fix(scan*10.)

return
end 

pro zen2scan,image
;---------------------------------------------------------------------------
; CONVERTS SATELLITE ZENITH ANGLE TO SCAN ANGLE
;
; Input
;   zeng   :  The satellite zenith angle (degrees * 10)
;
; Output
;   scan   :  The satellite scan angle (degrees * 10)
;
; Re is the radius of the earth, Hsat is the nominal height of the
; satellite, and ReHsat = Re + Hsat
;---------------------------------------------------------------------------

zeng = image/10.
degrad = 0.017453292
Rearth = 6357.
Hsat = 850.
ReHsat = 7207.0
zenang = (zeng) * degrad
scan = asin(Rearth*sin(zenang)/(ReHsat)) / degrad
image = fix(scan*10.)

return
end


;************************* RELATIVE AZIMUTH ANGLE **************************

;pro relaz,satazfln,solazfln,ncols,nrows,image
pro relaz,solazfln,satazfln,ncols,nrows,image
;---------------------------------------------------------------------------
; DETERMINE THE RELATIVE AZIMUTH BY SIMPLY SUBTRACTING THE SOLAR
; AZIMUTH FROM THE SATELLITE AZIMUTH.
;
; INPUT: 
;   solazfln   : File names for the satellite and solar azimuth angles
;   satazfln, 
;   ncols,
;   nrows      : The size of the image
; 
; OUTPUT: 
;   image      : Integer array of relative azimuth values
;---------------------------------------------------------------------------

imagea = uintarr(ncols,nrows)
openr,u,solazfln,/get_lun
readu,u,imagea
free_lun,u
if (!version.arch ne 'ppc') then begin
  imagea = swap_endian(imagea)
  print, 'Swapped Endian!'
endif
convertint,imagea,'Solar Azimuth angle'
print,"Min/max in Solar Azimuth Angle : ",min(imagea), max(imagea)

imageb = uintarr(ncols,nrows)
openr,u,satazfln,/get_lun
readu,u,imageb
free_lun,u
if (!version.arch ne 'ppc') then begin
  imageb = swap_endian(imageb)
  print, 'Swapped Endian!'
endif
convertint,imageb,'Satellite Azimuth angle'
print,"Min/max in Satellite Azimuth Angle : ",min(imageb), max(imageb)

image = abs(imagea - imageb)
ihi = where(image gt 1800,count)
if count gt 0 then image(ihi) = 3600 - image(ihi)

;image = imagea - imageb
;image = imageb - imagea
;ihi = where(image ge 3600, count1)
;if count1 gt 0 then image(ihi) = image(ihi) - 3600
;ilo = where(image lt 0, count2)
;if count2 gt 0 then image(ilo) = image(ilo) + 3600
;image = abs(image-1800)

return

end


;******************************* NORMALIZE *******************************

pro normalize,imgall,bandnum,use_ch3a
;---------------------------------------------------------------------------
; Normalize a shortwave channel by the cosine of solar zen angle.
; The values in the original array are replaced.  It is assumed
; that the solar zenith angle is in imgall(*,*,6).
;---------------------------------------------------------------------------

img = imgall(*,*,bandnum)
zen = imgall(*,*,6)/10.

; If this is channel 3A only normalize the sunlit portions of the image.
; Otherwise normalize the whole image.

if bandnum eq 2 then begin
  if use_ch3a then begin
    ilit = where(zen lt 90,count)
    if count gt 0 then begin
      img(ilit) = fix(((img(ilit)/10.) / cos(zen(ilit)*0.017453)) * 10.)
    endif
  endif else begin
    return                      ;  If channel 3B was passed in, return.
  endelse
endif else begin
  img = fix(((img/10.) / cos(zen*0.017453)) * 10.)
endelse

imgall(*,*,bandnum) = img

end


;*************************** INTEGER SIZE CHECK ****************************

pro convertint,image,label
;----------------------------------------------------------------------------
; Convert unsigned integer to signed integer (both 2 bytes), check for 
; values too large for signed integer.  If more than 1/10 of 1% are
; bad then the procedure is aborted.  Otherwise, the bad pixels are
; reset to 0 and processing continues.
;
; Input:
;   image  : The image to be converted
;   label  : A string identifying what's being converted.  Used if there's
;            an error.
;----------------------------------------------------------------------------

ibig = where(image gt 32767,count)
;if count gt 0 then begin
;  if n_elements(label) eq 0 then label = ''
;  print,'>>> CONVERTINT: VALUES ARE TOO LARGE FOR SIGNED INTEGER CONVERSION <<<'
;  print,'    Type: ',label,', number of pixels: ',count,'
;  print,'    Min and max values in image: ',min(image),max(image)
;  nmaxbad = n_elements(image) * 0.001       ; 1/10 of 1% are allowed to be bad
;  if count gt nmaxbad then begin
;    print,'    Too many bad pixels.  Cannot continue.'
;    exit
;  endif else begin
;    print,'    Bad pixels will be set to 0.'
;    image(ibig) = 0
;  endelse
;endif

if count gt 0 then begin
   image(ibig) = 0
endif

image = fix(image)

end


;************************* SUBSAMPLE AN IMAGE ******************************
;[ch3a_active]

function ch3a_active, satnum,year,month,day
;---------------------------------------------------------------------------
;  Determines if a satellite was using AVHRR channel 3A (1.6 um) during the
;  sunlit portion of the day based on  the satellite number and date. 
;  For example, NOAA-16 switched between 3A and 3B until May 1, 2003. 
;
;  Input:
;    satnum       : The NOAA satellite number (e.g., 7, 9, 11, 12, 14, 15, 16).
;    year, month,
;    day          : 2- or 4-digit integer for the year, 2-digit for month, and day.
;                   If year is 2-digit, values >=50 are 19xx, values <50 are 20xx.
;    hour         : The hour on a 24-hour clock, GMT (floating point)
;
;  Output:
;    TRUE if channel 3A was used; FALSE otherwise.
;
;  Notes:
;  1. Channel 3A on NOAA-15 was only used briefly, so we assume that 
;     it's always the 3.7 um channel.  This could/should be changed here.
;  2. As of May 1, 2003, NOAA-16 uses channel 3b (3.7 um) at all times.

;  Called by: ingest_image
;
;  Calls: None
;
;[enddoc]
;---------------------------------------------------------------------------
TRUE = 1
FALSE = 0
;---------------------------------------------------------------------------

if (satnum eq 16 and year eq 2003 and month lt 5) or $
   (satnum eq 16 and year le 2002) or $
   (satnum eq 18 and year eq 2005 and month eq 7) or $
   (satnum eq 18 and year eq 2005 and month eq 8 and day le 5) then begin
  ch3a = TRUE
  print,'  Assuming that channel 3 is 1.6 um for sunlit areas.'
endif else begin
  ch3a = FALSE
  print,'  Assuming that channel 3 is 3.7 um at all times.'
endelse

return,ch3a
end

;************************* INTERCALIBRATION BETWEEN VPP AND AVHRR ******************************
;[vpp_to_noaa19]

pro vpp_to_noaa19, vppimg3d, lst, pid
                        
; Purpose:
;    Caliberate VPP 5 channels digital values from VGAC I and M bands (I1, I2, M7, M11, M12) 
;    to match NOAA-19 5 AVHRR channels (channels 1-5) to mimic NOAA-19 5 AVHRR channels to 
;    be used by CASPR with the same algorithms for APP-x CDR generation.
; 
; Input/Output : 
;    vppimg3d : 3 dimensional array (ncols,nrow,9) storing VPP original 9 variables that are
;               ch1,ch2,ch3,ch4,ch5,scan angle,solar zenith angle, sensor-sun relative azimuth angle, 
;               and UTC time corresponding to the thrid dimension subscript indices from 0 to 8.
;               Integer type.
;         lst : local solar time, could be 14, 04, or 02.
;         pid : The id for hemisphere, 0=north, 1= south

; --- Scale factors for all channels, angles, time variables for real values ---
sfac = [1000.0,1000.0,10.0,10.0,10.0,10.0,10.0,10.0,10.0]

; --- Channel data value ranges for the calibration ---
vmin = [0.001, 0.001, 100.0, 100.0, 100.0,   0.0,   0.0,   0.0]
vmax = [0.999, 0.999, 400.0, 400.0, 400.0, 180.0, 180.0, 180.0]
			 	
; --- Regression coefficients for 5 channel, F=acoe(0) + acoe(1)*ch+acoe(2)*scn+acoe(3)*szn+acoe(4)*raz
if pid eq 0 then begin                                                           ; Northern Hemisphere 
 if lst eq 14 then begin 
   acoe = [[0.0435595, 0.930500, -7.75501e-05, -0.000525893, -4.77789e-06],$     ; AVHRR CH1
           [0.0103388, 0.911835, -0.000217053, -6.38642e-05, -1.62112e-05],$     ; AVHRR CH2
           [6.73663,   0.985943,    0.0128870,   -0.0497426,  -0.00489516],$     ; AVHRR CH3
           [4.27713,   0.981744,    0.0300169,   -0.0123865,   0.00771164],$     ; AVHRR CH4
           [7.25717,   0.970006,    0.0299595,   -0.0158270,   0.00797950]]      ; AVHRR CH5
 endif else begin
    acoe =[[0.0960655, 0.916712, -0.000218493, -0.000568735, -0.000622227],$     ; AVHRR CH1
           [ 0.116249, 0.844425,  1.80391e-07, -0.000555183, -0.000916527],$     ; AVHRR CH2
           [ 1.53406,  0.996171,    0.0355058,   -0.0267726,    0.0144087],$     ; AVHRR CH3
           [-4.50717,   1.00153,    -0.0233947,   0.0267718,    0.0335817],$     ; AVHRR CH4
           [-4.22725,  0.999418,    -0.0290731,   0.0306086,    0.0289854]]      ; AVHRR CH5
 endelse
endif

if pid eq 1 then begin                                                           ; Southern Hemisphere 
 if lst eq 14 then begin 
   acoe = [[0.0866695, 0.889131, 0.00180543, -0.00129548, 4.73859e-05],$     ; AVHRR CH1
           [-0.0306988, 0.901164,0.00250430, -0.000786054, 0.00108355],$     ; AVHRR CH2
           [-35.8105,   1.16163,    -0.232446,   0.162059,  -0.278580],$     ; AVHRR CH3
           [-3.12878,   1.01476, 0.00872472,   0.0190210,  -0.0288462],$     ; AVHRR CH4
           [-5.01120,   1.02192,    0.0204615,   0.0169925,-0.0256697]]      ; AVHRR CH5
 endif else begin
    acoe =[[0.236379, 0.757802, 2.56999e-05,-0.00272291, -3.36418e-06],$     ; AVHRR CH1
           [0.185096, 0.764386, 6.09533e-05, -0.00209245, 2.67030e-06],$     ; AVHRR CH2
           [-10.3159, 1.04152,  0.0160411,   -0.0100813,  -0.00249262],$     ; AVHRR CH3
           [3.72417,  0.986215, 0.00927672,  -0.00235766, 0.000241227],$     ; AVHRR CH4
           [4.38794,  0.984430, 0.0101097,   -0.00708749, 0.000180746]]      ; AVHRR CH5
 endelse
endif
; --- Viewing and illumination angles ---
fscn = float(vppimg3d(*,*,5))/sfac(5)
fszn = float(vppimg3d(*,*,6))/sfac(6)
fraz = float(vppimg3d(*,*,7))/sfac(7)

; ------- Channel loop  -------
FOR ich = 0, 4 DO BEGIN

    fsel = float(vppimg3d(*,*,ich))/sfac(ich)                            ; << ==== change here
    
    isel = where(fsel ge vmin(ich) and fsel le vmax(ich) and $
                 fscn ge vmin(5) and fscn le vmax(5) and $
                 fszn ge vmin(6) and fszn le vmax(6) and $
                 fraz ge vmin(7) and fraz le vmax(7),icnt1)
    
    if icnt1 gt 0 then begin
       ssch = fsel(isel) & sscn = fscn(isel) & sszn = fszn(isel) & sraz = fraz(isel)
   
       cuse = acoe(0,ich) + acoe(1,ich)*ssch + acoe(2,ich)*sscn + acoe(3,ich)*sszn + acoe(4,ich)*sraz
   
       iless = where(cuse le vmin(ich), icnt2)
       if icnt2 gt 0 then cuse(iless) = vmin(ich)
       
       igreat = where(cuse ge vmax(ich), icnt3)
       if icnt3 gt 0 then cuse(igreat) = vmax(ich)
   
       fsel(isel) = cuse(*)     ; << ==== change here
       
       vppimg3d(*,*,ich)=fix(fsel(*,*)*sfac(ich))
     endif
     
ENDFOR

; ------- -------


end

