   ;-------------------------------------------------------------------------------------- 
   ;AVHRR_Calibration_APP.pro    2015-08-14  Yinghui Liu
   ;-------------------------------------------------------------------------------------- 
   ; AVHRR Polar Pathfinder (APP) PROCESSING SOFTWARE Version 1.1
   ; NAME: AVHRR_Calibration_APP.pro
   ; PURPOSE: extract, calibrate, and navigate AVHRR GAC data
   ; DESCRIPTION: read AVHRR Global Area Coverage (GAC) data, calibrate the data using
   ;              Andrew Heidinger (Andrew.Heidinger@noaa.gov) calibration and navigation
   ;              procedure,remap the calibrated data with the MODIS Swath-to-Grid (MS2GT).
   ;              The calibration approach is developed by Andy Heidinger. It was in Fortran
   ;              language, and converted to IDL language by the authors. 
   ;
   ; AUTHORS:
   ;  Yinghui Liu, yinghuil@ssec.wisc.edu
   ;  Richard Dworak, rdworak@ssec.wisc.edu
   ;  Jeffrey Key, jeff.key@noaa.gov
 
   ; COPYRIGHT  
   ; THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
   ; DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
   ; FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
   ; INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
   ; EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
   ; DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
   ; THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
   ; SUPPORT TO USERS.
   
   ; Software required
   ; IDL: Interactive Data Language
   ; MS2GT: the MODIS Swath-to-Grid
   
   ; Routines in this script and their purpose:
   ; IDL Subtoutine:
   ;        mreadf.pro  
   ;        viewing_geometry.pro  
   ;        Plank_AVHRR.pro  
   ;        reposnx.pro  
   ;        avhrr_caligration_subs_tool.pro  
   ;        avhrr_caligration_subs_0.pro  
   ;        avhrr_caligration_subs_5.pro  
   ;        avhrr_caligration_subs_1.pro  
   ;        avhrr_caligration_subs_2.pro  
   ;        avhrr_caligration_subs_3.pro  
   ;        avhrr_caligration_subs_4.pro  
   ;   
   ;-------------------------------------------------------------------------------------- 



pro AVHRR_Calibration_APP,file_1b_full,avhrr_data, lat_data, lon_data, szaa_data, satz_data, relaz_data, utc_data
   ;;--------------------------------------------------------------
   ;;--- define the constants used in this program
   ;;--------------------------------------------------------------   
   @commonblock_1.pro  
   
   def_const 
   
   if keyword_set(verbose) then print,'processing ',file_1b_full    
   
   time0=systime(/seconds)
   
   AVHRR_Flag = !sym.yes           ;;; AVHRR level 1b data
   Nav_Flag   = 0  ;;2                  ;;; !0=level1b,1=clevernav,2=reposnx
   
   ;;--------------------------------------------------------------
   ;;--- determine some parameters used in this program
   ;;--------------------------------------------------------------   
   AVHRR_GAC_Flag  = 0
   AVHRR_KLM_Flag  = 0
   AVHRR_AAPP_Flag = 0
   Ver_1b          = 0
   data_type       = 0
   Byte_Swap_1b    = 0
   Nrec_Header     = 0     ;;;number of headers in the header file
   Num_Pix         = 0     ;;; or 0L ;;;the number of pixels per scanline
   l1b_rec_length  = 0     ;;; or 0L ;;;the number of bytes per record of the 1b file
   Nword_Clavr_Start=0     ;;;byte number of first clavr byte on 1b
   Nword_Clavr     = 0     ;;;number of bytes used for clavr cloud mask on 1b
   Num_Anchors     = 0     ;;; or 0L 
        
   ;; set_file_dimensions
   DETERMINE_AVHRR_FILE_TYPE,File_1b_Full,AVHRR_GAC_FLAG,AVHRR_KLM_Flag,AVHRR_AAPP_Flag, $
                             Ver_1b,Data_Type,Byte_Swap_1b
   
			     
   DEFINE_1B_DATA,AVHRR_GAC_Flag,AVHRR_KLM_Flag,AVHRR_AAPP_Flag,Nrec_Header, $
                   Nword_Clavr_Start,Nword_Clavr, $
                   num_pix,l1b_rec_length,Num_Anchors
   		   
   ;; NOte: Nrec_Header = Nrec_Avhrr_Header		   
   Sc_Id=0
   Start_Year=0 & Start_day=0 & Start_Time=0L
   End_Year  =0 & End_day=0   & End_Time=0L
   tip_parity=0 & aux_sync=0  & ramp_auto_Cal=0   
   ;Start_Time_Temp=0L & End_Time_Temp=0L    
   Num_Scans=0L   
   proc_block_Id = strarr(7)  ;proc_block_Id = ''
   READ_AVHRR_LEVEL1B_HEADER,File_1b_Full,AVHRR_GAC_Flag,AVHRR_KLM_Flag, $
                   l1b_rec_length,Sc_Id,data_type,Start_Year, $
                   Start_day,Start_time,Num_Scans,End_Year,End_day,End_time, $
                   tip_parity,aux_sync,ramp_auto_Cal,proc_block_Id,ver_1b,Byte_Swap_1b
      
   Num_Scans_Per_Segment = 240  ;;;240 
   Line_Idx_Min_Segment = 1                         ;; 
   Line_Idx_Max_Segment = Num_Scans_Per_Segment     ;;
   
   ;; SET_SENSOR_CONSTANTS 
   avhrr_1_Flag = !sym.NO
   DETERMINE_AVHRR_1,Start_Year,Sc_Id,AVHRR_KLM_Flag,Avhrr_1_Flag
   
   
   ASSIGN_AVHRR_SAT_ID_NUM_INTERNAL,AVHRR_KLM_Flag,Sc_Id,avhrr_1_Flag, $
            Avhrr_Number,Sc_Id_WMO, Sc_Id_Char,Platform_Name_Attribute,Sensor_Name_Attribute   
      
   ;--------- not sure what Orbit_Number is used for, it is used in subroutine ... (sure, need find it out)
   Orbit_Number = 0L
   for Idx=1,7 do begin ;; len(Proc_Block_Id)
      ;Orbit_Number = Orbit_Number + (ichar(Proc_Block_Id(Idx:Idx))-48)* $
      ;   (10**(len(Proc_Block_Id) - Idx))
      Orbit_Number = Orbit_Number + (byte(Proc_Block_Id(Idx-1:Idx-1))-48)* (10^(7 - Idx))
   endfor   
      
   ;;--------------------------------------------------------------
   ;;--- based on spacecraft id, set up constants
   ;;--------------------------------------------------------------
   Sc_Id_Internal = Avhrr_Number
   if (Sc_Id_Internal lt 10) then begin         ;;;Sc_Id_Internal = AVHRR_Number
    Instr_Const_File = '../Ancil_Data_Dir/'+'avhrr_'+string(byte(Sc_Id_Internal+48b))+'_instr.dat'
    Algo_Const_File  = '../Ancil_Data_Dir/'+'avhrr_'+string(byte(Sc_Id_Internal+48b))+'_algo.dat'
   endif else begin
    Instr_Const_File = '../Ancil_Data_Dir/'+'avhrr_1'+string(byte(Sc_Id_Internal-10b+48b))+'_instr.dat'
    Algo_Const_File  = '../Ancil_Data_Dir/'+'avhrr_1'+string(byte(Sc_Id_Internal-10b+48b))+'_algo.dat'
   endelse

   ;;TURN_OFF_CHANNELS_BASED_ON_SENSOR 
   ;;basically, ch 1,2,6 31 is on, and ch32 is on if not AVHRR 1
      Chan_On_Flag_Default = bytarr(42)+1B
      Chan_On_Flag_Default(3-1:5-1) = !sym.NO
      Chan_On_Flag_Default(7-1:19-1) = !sym.NO
      Chan_On_Flag_Default(21-1:30-1) = !sym.NO
      Chan_On_Flag_Default(33-1:36-1) = !sym.NO
      if (Avhrr_1_Flag eq !sym.YES) then begin
             Chan_On_Flag_Default(32-1) = !sym.NO
      endif
      Chan_On_Flag_Default(37-1:42-1) = !sym.NO
   
   
      dim1=Num_Pix & dim2=Num_Scans_Per_Segment & dim3=L1b_Rec_Length
      ch_data = {ch_struct,ref_toa:fltarr(dim1,dim2),rad_toa:fltarr(dim1,dim2), $
                           bt_toa:fltarr(dim1,dim2)}
         
   ;;--- read in Instrument Constants from appropriate file
   ;; READ_INSTR_CONSTANTS
      Prt_Coef_temp = fltarr(5)
      Prt_Coef = fltarr(5,4)    ;;;(0:4,4) in Fortran 90
      Prt_Weight = fltarr(4)
      sat_name=''
   
      openr,ur,Instr_Const_file,/get_lun,ERROR=ios0
      if (ios0 ne 0) then begin
         erstat = 19
         print,"Error opening AVHRR constants file, ios0 = ", ios0
         print, 'file : ',Instr_Const_file
         stop,erstat
      endif
         readf,ur, sat_name,format='(a3)'
         readf,ur, Solar_Ch20
         readf,ur, Ew_Ch20
         readf,ur, Space_Rad_3b, b0_3b, b1_3b, b2_3b
         readf,ur, Space_Rad_4, b0_4, b1_4, b2_4
         readf,ur, Space_Rad_5, b0_5, b1_5, b2_5
         readf,ur, nu_20
         readf,ur, a1_20
         readf,ur, a2_20
         readf,ur, nu_31
         readf,ur, a1_31
         readf,ur, a2_31
         readf,ur, nu_32
         readf,ur, a1_32
         readf,ur, a2_32
         readf,ur, Ch1_Dark_Count
         readf,ur, Ch2_Dark_Count
         readf,ur, Ch3a_Dark_Count
         readf,ur, Ch1_Gain_Low_0,Ch1_Degrad_Low_1, Ch1_Degrad_Low_2
         readf,ur, Ch1_Gain_High_0,Ch1_Degrad_High_1, Ch1_Degrad_High_2
         readf,ur, Ch2_Gain_Low_0,Ch2_Degrad_Low_1, Ch2_Degrad_Low_2
         readf,ur, Ch2_Gain_High_0,Ch2_Degrad_High_1, Ch2_Degrad_High_2
         readf,ur, Ch3a_Gain_Low_0,Ch3a_Degrad_Low_1, Ch3a_Degrad_Low_2
         readf,ur, Ch3a_Gain_High_0,Ch3a_Degrad_High_1, Ch3a_Degrad_High_2
         readf,ur, launch_date
         readf,ur, Ch1_Switch_Count
         readf,ur, Ch2_Switch_Count
         readf,ur, Ch3a_Switch_Count
         readf,ur, Prt_Coef_temp
	 Prt_Coef(*,1-1) = Prt_Coef_temp
         readf,ur, Prt_Coef_temp
	 Prt_Coef(*,2-1) = Prt_Coef_temp
         readf,ur, Prt_Coef_temp
	 Prt_Coef(*,3-1) = Prt_Coef_temp
         readf,ur, Prt_Coef_temp
	 Prt_Coef(*,4-1) = Prt_Coef_temp
         readf,ur, prt_weight
         readf,ur, b1_day_mask,b2_day_mask,b3_day_mask,b4_day_mask
      free_lun,ur
      ;;-- convert solar flux in channel 3b to mean with units mW/m^2/cm^-1
      Solar_Ch20_Nu = 1000.0 * Solar_Ch20 / Ew_Ch20
      
      
   POPULATE_PLANCK_TABLES,a1_20,a2_20,nu_20, $
                                   a1_31,a2_31,nu_31, $
                                   a1_32,a2_32,nu_32, $
				   b20,b31,b32
   ;; line 808
   ;;--- compute Sun-Earth distance\
   Sun_Earth_Distance = 1.0 - 0.016729*cos(0.9856*(Start_Day-4.0)*!dtor)   
   ;; line 811
   ;;--- compute time since launch needed for reflectance calibration
   Time_Since_Launch = Start_Year + Start_Day / 365.25  - launch_Date
 
  
   ;-------------------------------------------------------------------------
   ;if (Nav_Flag eq 2) then begin
   ;   SETUP_CLOCK_CORRECTIONS
   ;endif
   ;; line 817
   ;;--------------------------------------------------------------       
   ;;-- Marker: Interpolate a clock correction for this orbit
   ;;--------------------------------------------------------------
   if (Avhrr_Flag eq !sym.yes and Nav_Flag eq 2) then begin  ;;!0=level1b,1=clevernav,2=reposnx
      INTERPOLATE_CLOCK_ERROR,Start_Year, Start_Day,Start_Time,  $
                                   End_Year, End_Day,End_Time, $
                                   AVHRR_Number,timerr_seconds   ;;;Sc_Id_WMO to AVHRR_Number
   endif
       
   ;;--- compute number of segments in this orbit 
   if Num_Scans mod Num_Scans_Per_Segment eq 0 then begin
      Num_Segments = Num_Scans / Num_Scans_Per_Segment
      last_num_scan = Num_Scans_Per_Segment
   endif else begin
      Num_Segments = Num_Scans / Num_Scans_Per_Segment + 1
      last_num_scan = Num_Scans mod Num_Scans_Per_Segment
   endelse      
   
   avhrr_data = fltarr(5,Num_Pix,Num_Scans)+ !sym.Missing_Value_Real4
   lat_data   = fltarr(Num_Pix,Num_Scans)+ !sym.Missing_Value_Real4         
   lon_data   = fltarr(Num_Pix,Num_Scans) + !sym.Missing_Value_Real4
   szaa_data  = fltarr(Num_Pix,Num_Scans)+ !sym.Missing_Value_Real4
   satz_data  = fltarr(Num_Pix,Num_Scans)+ !sym.Missing_Value_Real4
   relaz_data = fltarr(Num_Pix,Num_Scans)+ !sym.Missing_Value_Real4
   utc_data   = fltarr(Num_Pix,Num_Scans)+ !sym.Missing_Value_Real4

      Ncal = 10L
      Space_Count_Temp_1   = fltarr(Ncal)
      Space_Count_Temp_2   = fltarr(Ncal)
      Space_Count_Temp_3   = fltarr(Ncal)
      Space_Count_Temp_4   = fltarr(Ncal)
      Space_Count_Temp_5   = fltarr(Ncal)
      BB_count_3=fltarr(Ncal)  & BB_count_4=fltarr(Ncal) & BB_count_5=fltarr(Ncal)
      T_Prt = fltarr(Ncal)
   
   for Segment_Number = 1, Num_Segments do begin
         
      ;;-----------------------------------------------------------------
      ;;---- Marker: Read level-1b data
      ;;-----------------------------------------------------------------
         ;READ_AVHRR_LEVEL1B_DATA,File_1b_Full,AVHRR_KLM_Flag,AVHRR_AAPP_Flag,Therm_Cal_1b,$


      ;;--- create interal arrays needed to process this segment
      ;;CREATE_AVHRR_ARRAYS,Num_Pix, Num_Scans_Per_Segment, L1b_Rec_Length
      dim1=Num_Pix & dim2=Num_Scans_Per_Segment & dim3=L1b_Rec_Length
      Buffer_AVHRR            = bytarr(dim3)
      dim23                   = long(dim2)*long(dim3)
      Segment_Buffer_AVHRR    = bytarr(dim23)
      Fatal_AVHRR             = bytarr(dim2)
      Resync_AVHRR            = bytarr(dim2)
      Clavr_Status_1b_AVHRR   = bytarr(dim2)+!sym.Missing_Value_Int1 
      Bad_Scan_Flag           = bytarr(dim2)+!sym.no
   
      Chan_Counts_Avhrr       = intarr(5,dim1,dim2)+!sym.Missing_Value_Int2
      Scan_Space_Counts_Avhrr = intarr(5,dim2)+!sym.Missing_Value_Int2 
      Scan_BB_Counts_Avhrr    = intarr(5,dim2)+!sym.Missing_Value_Int2
      Chan_Counts_Avhrr_Sg    = intarr(5,dim1,dim2)+!sym.Missing_Value_Int2
      IR_Coef_1_1b            = fltarr(5,dim2)+ !sym.Missing_Value_Real4  ;;;(3:5,dim2)
      IR_Coef_2_1b            = fltarr(5,dim2)+ !sym.Missing_Value_Real4  ;;;(3:5,dim2)
      IR_Coef_3_1b            = fltarr(5,dim2)+ !sym.Missing_Value_Real4  ;;;(3:5,dim2)
      IR_Linear_Slope_1b      = fltarr(5,dim2)+ !sym.Missing_Value_Real4  ;;;(3:5,dim2)
      IR_Linear_Intercept_1b  = fltarr(5,dim2)+ !sym.Missing_Value_Real4  ;;;(3:5,dim2)
      IR_Linear_Slope_New     = fltarr(5,dim2)+ !sym.Missing_Value_Real4  ;;;(3:5,dim2)
      IR_Linear_Intercept_New = fltarr(5,dim2)+ !sym.Missing_Value_Real4  ;;;(3:5,dim2)
      Vis_Slope_1             = fltarr(3,dim2)+ !sym.Missing_Value_Real4  ;;;(1:3,dim2))
      Vis_Slope_2             = fltarr(3,dim2)+ !sym.Missing_Value_Real4
      Vis_Intercept_1         = fltarr(3,dim2)+ !sym.Missing_Value_Real4
      Vis_Intercept_2         = fltarr(3,dim2)+ !sym.Missing_Value_Real4
      Vis_Intersection        = fltarr(3,dim2)+ !sym.Missing_Value_Real4
      
      Scan_Number             = lonarr(dim2)+!sym.Missing_Value_Int4
      Scan_Year               = intarr(dim2)+!sym.Missing_Value_Int2
      Scan_Day                = intarr(dim2)+!sym.Missing_Value_Int2
      Scan_Time               = fltarr(dim2)+!sym.Missing_Value_Real4
      utc                     = fltarr(dim2)+!sym.Missing_Value_Real4
      Ascend                  = bytarr(dim2)+!sym.Missing_Value_Int1 
      Ch3a_On_AVHRR           = bytarr(dim2)+!sym.Missing_Value_Int1 
      Ch6_On_Pixel_Mask       = bytarr(dim1,dim2)+!sym.Missing_Value_Int1 
      Space_Count_Temp        = intarr(5,10)
      Solzen_Anchor           = fltarr(Num_Anchors,dim2)+ !sym.Missing_Value_Real4
      Satzen_Anchor           = fltarr(Num_Anchors,dim2)+ !sym.Missing_Value_Real4
      Relaz_Anchor            = fltarr(Num_Anchors,dim2)+ !sym.Missing_Value_Real4
      
      Solaz_Anchor            = fltarr(Num_Anchors,dim2)+ !sym.Missing_Value_Real4
      Sataz_Anchor            = fltarr(Num_Anchors,dim2)+ !sym.Missing_Value_Real4
      Glintzen_Anchor         = fltarr(Num_Anchors,dim2)+ !sym.Missing_Value_Real4
      Scatangle_Anchor        = fltarr(Num_Anchors,dim2)+ !sym.Missing_Value_Real4

      Lat_Anchor_1b           = fltarr(Num_Anchors,dim2)+ !sym.Missing_Value_Real4
      Lon_Anchor_1b           = fltarr(Num_Anchors,dim2)+ !sym.Missing_Value_Real4
      Cld_Mask_Aux            = bytarr(dim1,dim2)+!sym.Missing_Value_Int1 
      Cloud_Mask_Aux_Read_Flag = !sym.no
      ch  =  replicate(ch_data,6) ;;1: vis1, 2:vis2, 3: vis3a, 4:bt3a, 5:11um, 6:12um
      Lon_1b = fltarr(dim1,dim2)+ !sym.Missing_Value_Real4
      Lat_1b = fltarr(dim1,dim2)+ !sym.Missing_Value_Real4
      Satzen = fltarr(dim1,dim2)+ !sym.Missing_Value_Real4
      Solzen = fltarr(dim1,dim2)+ !sym.Missing_Value_Real4
      Sataz = fltarr(dim1,dim2)+ !sym.Missing_Value_Real4
      Solaz = fltarr(dim1,dim2)+ !sym.Missing_Value_Real4
      Relaz = fltarr(dim1,dim2)+ !sym.Missing_Value_Real4
      Glintzen = fltarr(dim1,dim2)+ !sym.Missing_Value_Real4
      Scatangle = fltarr(dim1,dim2)+ !sym.Missing_Value_Real4
      
      Seg_Idx = Segment_Number
      therm_cal_1b = !sym.no
      ref_cal_1b   = !sym.no
            
      ;;--- initialize Calc_Asc_Des flag
      Calc_Asc_Des = !sym.NO

      Line_Idx   = 0
      Irec_start = 0
      Irec_end   = 0
      Irec       = 0
      Error_Code = 0
    
      word_start           = 0L
      word_end             = 0L
      Number_Of_Words      = 0L
      Number_Of_Words_Read = 0L
      bytes_per_word       = 0L            
      
      ;;--- based on segment number, compute starting and ending record in level-1b
      ;;--- for this segment
      Irec_start = Nrec_Header + (Seg_Idx-1)*Num_Scans_per_Segment + 1
      Irec_end =   Irec_start + Num_Scans_per_Segment - 1
      
      ;;--- initialize number of scans-read index
      Num_Scans_Read = 0

      Irec = Irec_start
      ;Line_Idx = Line_Idx_Min_Segment     ;;;1
      
      bytes_per_word = 1L
      ;;word_start = l1b_rec_length* ( 1 + (Seg_Idx-1)*Num_Scans_per_Segment) ;; bug here??
      ;; is the length of header same as the length of a record??
      word_start = long(l1b_rec_length)* ( Nrec_Header + (Seg_Idx-1)*long(Num_Scans_per_Segment))
      Number_Of_Words = long(l1b_rec_length)*Num_Scans_per_Segment
      
      if Segment_Number eq  Num_Segments and last_num_scan ne Num_Scans_Per_Segment then begin
         Number_Of_Words = long(l1b_rec_length)*last_num_scan
      endif
      
      mreadf,file_1b_full,word_start,bytes_per_word, $
               Number_Of_Words,Number_Of_Words_Read,Segment_Buffer_AVHRR
      ;;--- update number of scans read
      Num_Scans_Read = Number_Of_Words_Read / l1b_rec_length
                  
      ;;----------------------------------------------------------------------
      ;;  reset thermal calibration stats for this orbit
      ;;----------------------------------------------------------------------
      if (Seg_Idx eq 1) then begin
      ;if (Seg_Idx eq 8) then begin              ;;;debug
          Valid_Ref_cal = 0         ; 1: "TRUE", 0:"FALSE"
          Valid_therm_cal = 0       ; 1: "TRUE", 0:"FALSE" 
          Spinup_New_therm_cal = 1  ; 1: "TRUE", 0:"FALSE" 
          Prt_Idx = 0
          Cal_Idx = 0
          Mean_T_Prt = 0.0
          Mean_Space_Count_1 = 0.0
          Mean_Space_Count_2 = 0.0
          Mean_Space_Count_3 = 0.0
          Mean_Space_Count_4 = 0.0
          Mean_Space_Count_5 = 0.0
          Mean_BB_Count_3 = 0.0
          Mean_BB_Count_4 = 0.0
          Mean_BB_Count_5 = 0.0
          Ch1_Gain_Low = 0.0
          Ch1_Gain_High = 0.0
          Ch2_Gain_Low = 0.0
          Ch2_Gain_High = 0.0
          Ch3a_Gain_Low = 0.0
          Ch3a_Gain_High = 0.0
      endif
      
      
      ;;;
      do_therm_cal = !sym.NO
      latitude_count = 0
      ;;;
      
      
      ;;------------------------------------------------------------------------------------------------------------
      ;;--- UNPACK Level-1b Data
      ;;------------------------------------------------------------------------------------------------------------
      for Line_Idx = long(Line_Idx_Min_Segment), long(Line_Idx_Min_Segment) + Num_Scans_Read - 1 do begin
         ;;extract level-1b information for this scanline
         word_start = (Line_Idx-1)*l1b_rec_length + 1
         word_end = word_start +  l1b_rec_length - 1	 	 	 
         
         Buffer_AVHRR = Segment_Buffer_AVHRR(word_start-1:word_end-1)

         ;;--- unpack the level-1b data
	 if keyword_set(verbose) then print,'unpack...'
         if (AVHRR_AAPP_Flag eq !sym.YES) then begin
            UNPACK_AVHRR_DATA_RECORD_AAPP,Line_Idx   ;;;need common block
         endif else begin
            if (AVHRR_KLM_Flag eq !sym.YES) then begin
               UNPACK_AVHRR_DATA_RECORD_KLM,Line_Idx   ;;;need common block
            endif else begin
               UNPACK_AVHRR_DATA_RECORD,Line_Idx   ;;;need common block
               Ch3a_On_AVHRR(Line_Idx-1) = 0   ;;set for pre-AVHRR_KLM_Flag data
            endelse
         endelse
	 if keyword_set(verbose) then print,'done unpack...'
         
	 
         ;;----------------------------------------------------------------------
         ;; generate new calibration coefficients
         ;;----------------------------------------------------------------------
	 
	 if keyword_set(verbose) then print,'compute_new...'
         if (Therm_Cal_1b eq !sym.NO) then begin

            COMPUTE_NEW_THERM_CAL_COEF,Line_Idx
	    
	    

            ;;--- if sufficient scans lines have not been processed, this is fatal
            if (Spinup_New_Therm_Cal eq !sym.TRUE) then begin   
               Fatal_AVHRR(Line_Idx-1) = !sym.YES
            endif
         endif else begin
            Valid_Therm_Cal = !sym.true
         endelse
	 if keyword_set(verbose) then print,'done compute_new...'
         
	 
	 if abs(Lat_Anchor_1b(25,Line_Idx-1)) gt 50. then begin	 
	    latitude_count = latitude_count +1
	 
	    	 
            ;;--- compute reflectance calibration
	    if keyword_set(verbose) then print,'ref_cal...'
            REF_CAL,Time_Since_Launch,Line_Idx   ;; need common block
	    if keyword_set(verbose) then print,'done ref_cal...'
	 
	    if keyword_set(verbose) then print,'other...'

            ;;---- determine if Valid coefficients are made
            if (Valid_Therm_Cal eq !sym.false) then begin
               Fatal_AVHRR(Line_Idx-1) = !sym.YES
            endif
            if (Valid_Ref_Cal eq !sym.false) then begin
               Fatal_AVHRR(Line_Idx-1) = !sym.YES
            endif

            ;;------------------------------------------------------------------------------
            ;; Compute latitude and longitude
            ;;------------------------------------------------------------------------------
	    if keyword_set(verbose) then print,'compute latitude and longitude ...'
            if (max(abs(Lat_Anchor_1b(*,Line_Idx-1))) gt 85.0) then begin
               ;GNOMIC_ANCHOR_INTERP,Lon_Anchor_1b(*,Line_Idx-1),Lat_Anchor_1b(*,Line_Idx-1), $
               ;                       Lon_1b(*,Line_Idx-1),Lat_1b(*,Line_Idx-1)
	       lon_anchor_temp = reform(Lon_Anchor_1b(*,Line_Idx-1))
	       lat_anchor_temp = reform(Lat_Anchor_1b(*,Line_Idx-1))
	       ;; common block not needed
	       if keyword_set(verbose) then print,'gnomic'
	       GNOMIC_ANCHOR_INTERP,Lon_Anchor_temp,Lat_Anchor_temp,lon_1b_temp,lat_1b_temp,num_pix
	       Lon_1b(*,Line_Idx-1) = lon_1b_temp & Lat_1b(*,Line_Idx-1)=lat_1b_temp	    
            endif else begin
	       lon_anchor_temp = reform(Lon_Anchor_1b(*,Line_Idx-1))
	       lat_anchor_temp = reform(Lat_Anchor_1b(*,Line_Idx-1))	 
	       ;; common block not needed
               LAGRANGIAN_ANCHOR_INTERP,Lat_Anchor_temp,lat_1b_temp,num_pix
               LAGRANGIAN_ANCHOR_INTERP,Lon_Anchor_temp,lon_1b_temp,num_pix
	       Lon_1b(*,Line_Idx-1) = lon_1b_temp & Lat_1b(*,Line_Idx-1)=lat_1b_temp	    
            endelse
         
	    if keyword_set(verbose) then print,'INTERPOLATION'
	 
            ;;--- constrain lat and lon to physical values
            if ((min(Lat_1b(*,Line_Idx-1)) lt -90.0) or (max(Lat_1b(*,Line_Idx-1)) gt 90.0)) then begin
	       Lat_1b(*,Line_Idx-1) = 90.0<((-90.0)>(max(Lat_1b(*,Line_Idx-1))))
            endif
            if ((min(Lon_1b(*,Line_Idx-1)) lt -180.0) or (max(Lon_1b(*,Line_Idx-1)) gt 180.0)) then begin
               Lon_1b(*,Line_Idx-1) = 180.0<((-180.0)>(max(Lon_1b(*,Line_Idx-1))))
            endif 

         
            ;;----------------------------------------------------------------
            ;;--- Compute viewing geometry
            ;;----------------------------------------------------------------
            if keyword_set(verbose) then print,'compute viewing geometry ...'
            ;;--- compute Anchor points for pre-AVHRR_KLM_Flag series (not included in level-1b)
	    lon_anchor_temp = reform(Lon_Anchor_1b(*,Line_Idx-1))
	    lat_anchor_temp = reform(Lat_Anchor_1b(*,Line_Idx-1))
	    Satzen_Anchor_temp = reform(Satzen_Anchor(*,Line_Idx-1))
	    Solzen_Anchor_temp = reform(Solzen_Anchor(*,Line_Idx-1))
	    Relaz_Anchor_temp = reform(Relaz_Anchor(*,Line_Idx-1))
	    Solaz_Anchor_temp = reform(Solaz_Anchor(*,Line_Idx-1))
	    Sataz_Anchor_temp = reform(Sataz_Anchor(*,Line_Idx-1))
	    Glintzen_Anchor_temp = reform(Glintzen_Anchor(*,Line_Idx-1))
	    Scatangle_Anchor_temp = reform(Scatangle_Anchor(*,Line_Idx-1))
	 	 
	    ;; common block not needed	 
            COMPUTE_ANGLE_ANCHORS,Scan_Day(Line_Idx-1),        $
                                   Scan_Time(Line_Idx-1),       $
                                   Lon_Anchor_temp, $
                                   Lat_Anchor_temp, $
                                   Satzen_Anchor_temp, $
                                   Solzen_Anchor_temp, $
                                   Relaz_Anchor_temp,  $
                                   Solaz_Anchor_temp,  $
                                   Sataz_Anchor_temp,  $
                                   Glintzen_Anchor_temp,  $
                                   Scatangle_Anchor_temp
         				   
            Satzen_Anchor(*,Line_Idx-1) = Satzen_Anchor_temp
	    Solzen_Anchor(*,Line_Idx-1) = Solzen_Anchor_temp
	    Relaz_Anchor(*,Line_Idx-1) =Relaz_Anchor_temp
	    Solaz_Anchor(*,Line_Idx-1) =Solaz_Anchor_temp
	    Sataz_Anchor(*,Line_Idx-1) =Sataz_Anchor_temp
	    Glintzen_Anchor(*,Line_Idx-1) =Glintzen_Anchor_temp
	    Scatangle_Anchor(*,Line_Idx-1) =Scatangle_Anchor_temp
	 
	    if keyword_set(verbose) then print,'COMPUTE_ANGLE_ANCHORS'
	 
	 
            ;;--- interpolate pixel values from Anchor values
	    Satzen_temp = reform(Satzen(*,Line_Idx-1))
	    Solzen_temp = reform(Solzen(*,Line_Idx-1))
	    Relaz_temp = reform(Relaz(*,Line_Idx-1))
	    Solaz_temp = reform(Solaz(*,Line_Idx-1))
	    Sataz_temp = reform(Sataz(*,Line_Idx-1))
	    Glintzen_temp = reform(Glintzen(*,Line_Idx-1))
	    Scatangle_temp = reform(Scatangle(*,Line_Idx-1))
            LINEAR_ANCHOR_INTERP,Satzen_Anchor_temp,Satzen_temp,num_pix
            LINEAR_ANCHOR_INTERP,Solzen_Anchor_temp,Solzen_temp,num_pix
            LINEAR_ANCHOR_INTERP,Relaz_Anchor_temp,Relaz_temp,num_pix
            LINEAR_ANCHOR_INTERP,Solaz_Anchor_temp,Solaz_temp,num_pix
            LINEAR_ANCHOR_INTERP,Sataz_Anchor_temp,Sataz_temp,num_pix
            LINEAR_ANCHOR_INTERP,Glintzen_Anchor_temp,Glintzen_temp,num_pix
            LINEAR_ANCHOR_INTERP,Scatangle_Anchor_temp,Scatangle_temp,num_pix
            ;;--- constrain sensor zenith
            if ((min(Satzen(*,Line_Idx-1)) < 0.0) or (max(Satzen(*,Line_Idx-1)) > 89.9)) then begin
               Satzen(*,Line_Idx-1) = 89.9<(0.0>(max(Satzen(*,Line_Idx-1))))	    
            endif
            Satzen(*,Line_Idx-1) = Satzen_temp
	    Solzen(*,Line_Idx-1) = Solzen_temp
	    Relaz(*,Line_Idx-1) =Relaz_temp
	    Solaz(*,Line_Idx-1) =Solaz_temp
	    Sataz(*,Line_Idx-1) =Sataz_temp
	    Glintzen(*,Line_Idx-1) =Glintzen_temp
	    Scatangle(*,Line_Idx-1) =Scatangle_temp
	 
	 
            ;;------------------------------------------------------------------------------
            ;; check for bad scan lines here
            ;;------------------------------------------------------------------------------
            Bad_Scan_Flag(Line_Idx-1) = !sym.NO
            Error_Code = 0

            ;;--- any scan with a fatal error is considered bad
            if (Fatal_AVHRR(Line_Idx-1) eq !sym.YES) then begin
               Bad_Scan_Flag(Line_Idx-1) = !sym.YES
               Error_Code = 1
            endif

            ;;--- if in spinup mode, set all scans as bad 
            ;;--- note, these are also set to have a fatal code, but this overwrites
            ;;--- it to limit printing to screen of excessive reports
            if ((Therm_Cal_1b eq !sym.NO) and (Spinup_New_Therm_Cal eq !sym.true)) then begin
               Bad_Scan_Flag(Line_Idx-1) = !sym.YES
               Error_Code = 2
            endif

            ;;---- check for missing navigation
            if ( (total(Lat_Anchor_1b(*,Line_Idx-1)) eq 0.00) or (total(Lon_Anchor_1b(*,Line_Idx-1)) eq 0.00)) then begin
               Bad_Scan_Flag(Line_Idx-1) = !sym.YES
               Error_Code = 3
            endif

            ;;--- check erroneous asc/des flag 
            ;;--- erroneous values are common on AVHRR/1 data
            ;;--- so this is no longer considered a fatal error
            if ((Ascend(Line_Idx-1) < 0) or (Ascend(Line_Idx-1)  > 1)) then begin
               Calc_Asc_Des = !sym.YES
            endif

            ;;---- check angle Anchors on AVHRR_KLM_Flag data
            if (AVHRR_KLM_Flag eq !sym.YES) then begin
               if ((min(Satzen_Anchor(*,Line_Idx-1)) lt 0.0) or (max(Satzen_Anchor(*,Line_Idx-1)) gt 90.0)) then begin
                  Bad_Scan_Flag(Line_Idx-1) = !sym.YES
                  Error_Code  = 5
               endif
               if ((min(Solzen_Anchor(*,Line_Idx-1)) lt 0.0) or (max(Solzen_Anchor(*,Line_Idx-1)) gt 180.0)) then begin
                  Bad_Scan_Flag(Line_Idx-1) = !sym.YES
                  Error_Code = 5
               endif
               if ((min(Relaz_Anchor(*,Line_Idx-1)) lt -180.0) or (max(Relaz_Anchor(*,Line_Idx-1)) gt 180.0)) then begin
                  Bad_Scan_Flag(Line_Idx-1) = !sym.YES
                  Error_Code = 5
               endif
            endif

            ;;----- report bad scan code to standard output (but not bad scans during
            ;;----- spin-up of thermal cal
            if (Bad_Scan_Flag(Line_Idx-1) eq !sym.YES and Error_Code ne 2) then begin
	       print, $
                  'BAD SCAN: Numbers = ', Scan_Number(Line_Idx-1), ' error code = ', Error_Code, $
	          format='(a,i6,a,i2)'
            endif 

            ;;---- set geolocation to missing for bad-scans to avoid ancil-data interp
            if (Bad_Scan_Flag(Line_Idx-1) eq !sym.YES) then begin
                  Lat_1b(*,Line_Idx-1) = !sym.Missing_Value_Real4
                  Lon_1b(*,Line_Idx-1) = !sym.Missing_Value_Real4
            endif
	 
	    if keyword_set(verbose) then print,'done other...'
	 endif else begin
               Bad_Scan_Flag(Line_Idx-1) = !sym.YES
	 endelse   
	 
      endfor ;; do segment_loop  
            	 
      ;;------------------------------------------------------------------------------
      ;; Calculate ascending/descending flag if needed for this segment
      ;; not necessary here
      ;;------------------------------------------------------------------------------
      
      if latitude_count eq Num_Scans_Read then do_therm_cal = !sym.YES
      
      if  do_therm_cal eq !sym.YES then begin

      
         if (Calc_Asc_Des eq !sym.YES) then begin
            ;;common block needed
            ;CALCULATE_ASC_DES,Line_Idx_Min_Segment,Num_Scans_Read
         endif
 
         ;;------------------------------------------------------------------------------
         ;; Calibrate segment
         ;;------------------------------------------------------------------------------
         ;;--- RADIANCE CALIBRATION (3b,4,5)
         ;;common block needed
         if keyword_set(verbose) then print,'therm_cal ...'
         THERM_CAL,Line_Idx_Min_Segment,Num_Scans_Read  ;;debug
         if keyword_set(verbose) then print,'done therm_cal'
         print,systime(/seconds)-time0,format='(f20.5)'
      

         ;;--- convert AVHRR counts to single gain and store in global count arrays
         ;;--- this subroutine is not necessary in this package
         ;;common block needed
         ;; not necessary here
         ;CONVERT_AVHRR_COUNTS_SINGLE_GAIN,Line_Idx_Min_Segment,Num_Scans_Read

 
         ;;*******************************************************************
         ;; Marker: Recompute geolocation
         ;; need make sure if this is applied in PATMOS-x
         ;;*******************************************************************
         if (Nav_Flag eq 2  and  Avhrr_Flag eq !sym.yes) then begin
            if ((timerr_seconds ne !sym.Missing_Value_Real4) and $
                (timerr_seconds ne 0.0)) then begin
	        ;;common block needed
                REPOSITION_FOR_CLOCK_ERROR,AVHRR_GAC_FLAG,Line_Idx_Min_Segment,Num_Scans_Read, $
                                  Timerr_Seconds,Err_Reposnx_Flag,           $
				  Num_Pix,start_year,start_day,end_year,end_day, $
				  lat_1b,lon_1b,Bad_Scan_Flag,Scan_Time,start_time, $
				  scan_number
            endif
         endif
      
      
         ;;;;;;
         ;;consider output the results through files or through arrays
         ;;prefer array if the memory is big enough
         ;;ohterwise output to multiple files, then read them
         ;;;;;;
         xxx = where(Scan_Time ne !sym.Missing_Value_Real4, cxx)
         if cxx gt 0 then Scan_Time(xxx) = Scan_Time(xxx)/1000. 
         xxx = where(Relaz ne !sym.Missing_Value_Real4, cxx) 
         if cxx gt 0 then Relaz(xxx) = 180.- Relaz(xxx)       ;;;APP relaz = 180.- PATMOS-x relaz
            
         if Segment_Number ne Num_Segments then begin
            ind1 = (Segment_Number-1)*Num_Scans_Per_Segment & ind2 = ind1+Num_Scans_Per_Segment-1
	    ind11 = 0 & ind12 = ind2-ind1
         endif else begin      
            ind1 = (Segment_Number-1)*Num_Scans_Per_Segment & ind2 = Num_Scans-1
	    ind11 = 0 & ind12 = ind2-ind1
         endelse	 
         avhrr_data(0,*,ind1:ind2) = ch(0).Ref_Toa(*,ind11:ind12) 
         avhrr_data(1,*,ind1:ind2) = ch(1).Ref_Toa(*,ind11:ind12) 
         avhrr_data(2,*,ind1:ind2) = ch(3).Bt_Toa(*,ind11:ind12) 
         avhrr_data(3,*,ind1:ind2) = ch(4).Bt_Toa(*,ind11:ind12) 
         avhrr_data(4,*,ind1:ind2) = ch(5).Bt_Toa(*,ind11:ind12) 
         lat_data(* ,ind1:ind2)    = lat_1b(*,ind11:ind12) 
         lon_data(* ,ind1:ind2)    = lon_1b(*,ind11:ind12) 
         szaa_data(* ,ind1:ind2)   = Solzen(*,ind11:ind12) 
         satz_data(* ,ind1:ind2)   = Satzen(*,ind11:ind12) 
         relaz_data(* ,ind1:ind2)  = Relaz(*,ind11:ind12)  
         for indz=ind1,ind2 do begin
            utc_data(*,indz) = Scan_Time(indz-ind1)
         endfor
	 
      endif 
        	       
   endfor
   
end
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
