   ;-------------------------------------------------------------------------------------- 
   ;avhrr_caligration_subs_0.pro    2015-08-14  Yinghui Liu
   ;-------------------------------------------------------------------------------------- 
   ; AVHRR Polar Pathfinder (APP) PROCESSING SOFTWARE Version 1.1
   ; NAME: avhrr_caligration_subs_0.pro
   ; PURPOSE: subroutine to extract, and calibrate AVHRR GAC data
   ; DESCRIPTION: subroutine to calibrate AVHRR Global Area Coverage (GAC).
   ;              The calibration approach is developed by Andy Heidinger (Andrew.Heidinger@noaa.gov). 
   ;              It was in Fortran language, and converted to IDL language by the authors. 
   ;
   ; AUTHORS:
   ;  Yinghui Liu, yinghuil@ssec.wisc.edu
   ;  Richard Dworak, rdworak@ssec.wisc.edu
   ;  Jeffrey Key, jeff.key@noaa.gov
 
   ; COPYRIGHT  
   ; THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
   ; DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
   ; FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
   ; INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
   ; EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
   ; DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
   ; THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
   ; SUPPORT TO USERS.
   
   ; Software required
   ; IDL: Interactive Data Language
   
   ; Routines in this script and their purpose:
   ; IDL Subtoutines in this file
   ;   
   ;-------------------------------------------------------------------------------------- 





pro def_const
   cal_system = { sym_template1,     $
           NO:0,             $
	   YES:1,            $
	   FALSE:0,          $
	   TRUE:1,           $
	   UNSIGNED:0,       $
           SIGNED:1,         $
	   NOSWAP:0,         $
	   SWAP:1,           $           
	   ;Plank_c1:double(1.191062e-5), $
	   Plank_c1:1.191062d-5, $
	   Plank_c2:1.4387863d,          $
	   Missing_Value_Real4:-999.0,   $
	   Missing_Value_Real8:-999.0,   $
	   Missing_Value_Int1:-128,      $
	   Missing_Value_Int2:-32768,    $
           Biggest_I4:2147483647L,       $
           Biggest_I2:32767,             $
	   Missing_Value_Int4:-999,      $
	   No_Attribute_Missing_Value:-888.0 }
   defsysv,'!sym',cal_system
       
   return
end






pro DETERMINE_AVHRR_FILE_TYPE,file_1b_local,AVHRR_GAC_Flag,AVHRR_KLM_Flag,AVHRR_AAPP_Flag, $
                                        ver_1b,data_type,Byte_Swap_1b

    ;;; in avhrr_module.f90
    
    ;;--- use IDL-routine
    Header_Buffer_Temp = bytarr(100)        
    bytes_per_word = 1
    word_start = 0
    Number_Of_Words = 100
    Number_Of_Words_Read = 0
    
    mreadf,file_1b_local,word_start,bytes_per_word,                    $
           Number_Of_Words,Number_Of_Words_Read,Header_Buffer_Temp

    if (Header_Buffer_Temp(1-1) le 10) then begin
        AVHRR_KLM_Flag = !sym.NO
    endif else begin
        if (Header_Buffer_Temp(1-1) ge 65) then begin
           AVHRR_KLM_Flag = !sym.YES
        endif else begin
           print,"Error diagnosing AVHRR_KLM_Flag from first byte, assuming AVHRR_KLM_Flag = 0"
           AVHRR_KLM_Flag = !sym.NO
	endelse   
    endelse

;;---assuming AVHRR_KLM_Flag, unpack ver1b, Valid values are (1-5)
    ver_1b = MAKE_I2WORD(Header_Buffer_Temp(5-1:6-1),!sym.UNSIGNED,!sym.NOSWAP)

;;--- if version number unrealistic, assume AVHRR_AAPP_Flag
     AVHRR_AAPP_Flag = !sym.NO
     Byte_Swap_1b = !sym.NO
     if (AVHRR_KLM_Flag eq !sym.YES) then begin
        if ((ver_1b gt 10) or (ver_1b lt 1)) then begin
          AVHRR_AAPP_Flag = !sym.YES
          Byte_Swap_1b = !sym.YES
          ver_1b = MAKE_I2WORD(Header_Buffer_Temp(5-1:6-1),!sym.UNSIGNED,Byte_Swap_1b)
        endif
     endif

;;--- read spacedraft id
     Sc_Id = MAKE_I2WORD(Header_Buffer_Temp(73-1:74-1),!sym.UNSIGNED,Byte_Swap_1b)

;;--- fix noaa-15 bug
     if ((ver_1b eq 1) and (Sc_Id eq 4)) then begin
       ver_1b = 2
     endif

;;--- assign pre-AVHRR_KLM_Flag to version 1
     if (AVHRR_KLM_Flag eq !sym.NO) then begin
       ver_1b = 1
     endif

;;--- based on AVHRR_KLM_Flag flag, choose bytes to construct data_type
     if (AVHRR_KLM_Flag eq !sym.YES) then begin
         data_type = MAKE_I2WORD(Header_Buffer_Temp(77-1:78-1),!sym.UNSIGNED,Byte_Swap_1b)
     endif else begin
         data_type = ishft(Header_Buffer_Temp(2-1),-4)
     endelse

;;--- based on data_type, set AVHRR_GAC_Flag
     AVHRR_GAC_Flag = !sym.NO
     if (data_type eq 2) then begin
       AVHRR_GAC_Flag = !sym.YES
     endif

end 



;;----------------------------------------------------------------------
;; DEFINE_1B_DATA - based on the AVHRR_GAC_Flag and AVHRR_KLM_Flag flags, this routine defines
;;                  the parameters necessary to read in the header and
;;                  data records from the various 1B data types. This allows
;;                  defines quantities to allow the clavr cloud mask to be
;;                  written into the AVHRR_KLM_Flag 1b files.
;;
;; input
;;      AVHRR_GAC_Flag - T if AVHRR_GAC_Flag, F if hrpt or lac
;;      AVHRR_KLM_Flag - T if NOAA-AVHRR_KLM_Flag, F if NOAA-J or earlier
;;      AVHRR_AAPP_Flag - T if AAPP, F if NESDIS
;; output
;;      Num_Pix - the number of pixels per scanline
;;      l1b_rec_length - the number of bytes per record of the 1b file
;;      Nrec_Header - the number of records used for the header
;;      Nword_Clavr_Start - byte number of first clavr byte on 1b
;;      Nword_Clavr - number of bytes used for clavr cloud mask on 1b
;;-------------------------------------------------------------------------------
pro DEFINE_1B_DATA,AVHRR_GAC_Flag,AVHRR_KLM_Flag,AVHRR_AAPP_Flag,Nrec_Header,Nword_Clavr_Start,Nword_Clavr, $
                   num_pix,l1b_rec_length,Num_Anchors

  ;;;Nword_Clavr = 0L & Nword_Clavr_Start = 0L & Nrec_Header = 0L

  if (AVHRR_GAC_Flag eq !sym.YES) then begin
      Num_Pix = 409              ;;number of pixels per scan line
      if (AVHRR_KLM_Flag eq !sym.YES) then begin
       l1b_rec_length  = 4608
       Nrec_Header = 1
       Nword_Clavr_Start = 4049
       Nword_Clavr = 2*52
      endif else begin
       l1b_rec_length  = 3220
       Nrec_Header = 2
       Nword_Clavr_Start = 0
       Nword_Clavr = 0
      endelse
  endif else begin
     if (AVHRR_KLM_Flag eq !sym.YES) then begin
       Num_Pix = 2048
       Nrec_Header = 1
       l1b_rec_length = 15872
       Nword_Clavr_Start = 14977
       Nword_Clavr = 2*256
     endif else begin
       Num_Pix = 2048
       Nrec_Header = 1
       l1b_rec_length = 14800
       Nword_Clavr_Start = 0
       Nword_Clavr = 0
     endelse
  endelse

;;---- AVHRR_AAPP_Flag - assumed to be 1 km  and no clavr bits
  if (AVHRR_AAPP_Flag eq !sym.YES) then begin
       Num_Pix = 2048
       Nrec_Header = 1
       l1b_rec_length = 22016
       Nword_Clavr_Start = 0
       Nword_Clavr = 0
  endif

;;--- number of geolocation Anchors
  Num_Anchors = 51

 end



;;eqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeq
;; Read in the AVHRR Level1b header
;;
;; input and output passed through avhrr_Pixel_common
;;
;; handles AVHRR_GAC_Flag,lac, hrpt for pre and post AVHRR_KLM_Flag data - and AVHRR_AAPP_Flag
;;eqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeq

pro READ_AVHRR_LEVEL1B_HEADER,file_1b_local,AVHRR_GAC_Flag,AVHRR_KLM_Flag, $
                   l1b_rec_length,Sc_Id,data_type,Start_Year, $
                   Start_day,Start_time,Num_Scans,End_Year,End_day,End_time, $
                   tip_parity,aux_sync,ramp_auto_Cal,proc_block_Id,ver_1b,Byte_Swap_1b

    word_start = 0L
    word_end   = 0L
    Number_Of_Words = 0L
    Number_Of_Words_Read = 0L
    bytes_per_word = 0L            

    ;;--- allocate header Buffer - taken from pixel_common
    Header_Buffer_AVHRR = bytarr(l1b_rec_length)   
 
    ;;--- use external c
    bytes_per_word = 1L
    word_start = 0L
    Number_Of_Words = l1b_rec_length
    mreadf,file_1b_local,word_start,bytes_per_word, $
                    Number_Of_Words,Number_Of_Words_Read,Header_Buffer_AVHRR		    

    if (AVHRR_GAC_Flag eq !sym.YES) then begin

     if (AVHRR_KLM_Flag eq !sym.YES) then begin

        UNPACK_AVHRR_HEADER_RECORD_KLM,Sc_Id,data_type,Start_Year, $
                   Start_day,Start_time,Num_Scans,End_Year,End_day,End_time, $
                   tip_parity,aux_sync,ramp_auto_Cal,proc_block_Id,ver_1b,   $
		   Byte_Swap_1b,Header_Buffer_AVHRR

     endif else begin

        UNPACK_AVHRR_HEADER_RECORD,Sc_Id,data_type,Start_Year, $
                   Start_day,Start_time,Num_Scans,End_Year,End_day,End_time, $
                   tip_parity,aux_sync,ramp_auto_Cal,proc_block_Id,ver_1b,   $
		   Byte_Swap_1b,Header_Buffer_AVHRR

      ;;--- pre AVHRR_KLM_Flag used a 2 digit year
      if (End_Year gt 50) then begin
         End_Year = End_Year + 1900
      endif else begin
          End_Year = End_Year + 2000
      endelse
      if (Start_Year gt 50) then begin
          Start_Year = Start_Year + 1900
      endif else begin
          Start_Year = Start_Year + 2000
      endelse
                                                                                                                                          
     endelse

   endif else begin     ;;if not AVHRR_GAC_Flag

     if (AVHRR_KLM_Flag eq !sym.YES) then begin

        UNPACK_AVHRR_HEADER_RECORD_KLM,Sc_Id,data_type,Start_Year, $
                  Start_day,Start_time,Num_Scans,End_Year,End_day,End_time, $
                  tip_parity,aux_sync,ramp_auto_Cal,proc_block_Id,ver_1b,   $
		  Byte_Swap_1b,Header_Buffer_AVHRR

     endif else begin

        UNPACK_AVHRR_HEADER_RECORD,Sc_Id,data_type,Start_Year, $
                Start_day,Start_time,Num_Scans,End_Year,End_day,End_time, $
                tip_parity,aux_sync,ramp_auto_Cal,proc_block_Id,ver_1b,   $
		Byte_Swap_1b,Header_Buffer_AVHRR

;;--- pre AVHRR_KLM_Flag used a 2 digit year
      if (End_Year gt 50) then begin
         End_Year = End_Year + 1900
      endif else begin
         End_Year = End_Year + 2000
      endelse
      if (Start_Year gt 50) then begin
         Start_Year = Start_Year + 1900
      endif else begin
         Start_Year = Start_Year + 2000
      endelse
     endelse
   endelse

end 




;;eqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeq
;; unpack avhrr header record - routine should work for lac and AVHRR_GAC_Flag on
;; all pre-AVHRR_KLM_Flag sensors
;;
;; Modified December 2008 for segment processing
;;eqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeq
pro UNPACK_AVHRR_HEADER_RECORD,Sc_Id,data_type,Start_Year_Temp,$
              Start_day_Temp,Start_Time_Temp,Num_Scans,End_Year_Temp,End_day_Temp,End_Time_Temp, $
              tip_parity,aux_sync,ramp_auto_Cal,proc_block_Id,ver_1b,   $
		Byte_Swap_1b,Header_Buffer_AVHRR

   
   Start_Year_Temp=0 & Start_day_Temp=0   
   End_Year_Temp=0 & End_day_Temp=0 
   tip_parity=0 & aux_sync=0   
   ramp_auto_Cal=0
   
   Start_Time_Temp=0L & End_Time_Temp=0L & Num_Scans=0L
   
   proc_block_Id = strarr(7)  ;;'' length eq 7
   
   i4word = 0L
   
   i = 0
   
   data_byte =  intarr(2)
   
     
   ;;-----------------------------------------------------------------------
   ;; begin executable code
   ;;-----------------------------------------------------------------------

   Sc_Id = Header_Buffer_AVHRR(1-1)
   data_type = ishft(Header_Buffer_AVHRR(2-1),-4)

   tip_parity = 0
   aux_sync = 0
   ramp_auto_Cal = 0
   ver_1b = 1

   ;;----------------------------------------------------------------------
   ;; unpack starting Time_Temp
   ;;----------------------------------------------------------------------
   data_byte(1-1) = Header_Buffer_AVHRR(3-1)
   data_byte(2-1) = Header_Buffer_AVHRR(4-1)
   xxx = where(data_byte lt 0, cxx)
   if cxx gt 0 then data_byte(xxx) = data_byte(xxx)+256
   
   i4word =  data_byte(1-1)*256L + data_byte(2-1)
   Start_Year_Temp = i4word / (long(2)^9)
   Start_day_Temp = i4word - Start_Year_Temp * (long(2)^9)

   i4word = MAKE_I4WORD(Header_Buffer_AVHRR(5-1:8-1),!sym.UNSIGNED,Byte_Swap_1b)
   Start_Time_Temp = i4word - ((i4word/long(2)^27)*(long(2)^27))

   ;;---------------------------------------------------------------------
   ;; unpack processing block id
   ;;---------------------------------------------------------------------
   for i = 1,7 do begin  ;;1,len(proc_block_Id)
    ;proc_block_Id(i-1:i-1) =  char(Header_Buffer_AVHRR(17+(i-1)-1))
    proc_block_Id(i-1:i-1) =  string(byte(Header_Buffer_AVHRR(17+(i-1)-1)))
    ;;; byte is actually not needed to be added here since Header_Buffer_AVHRR is byte type
   endfor 
       
   ;;----------------------------------------------------------------------
   ;; unpack  number of scan lines
   ;;----------------------------------------------------------------------
   Num_Scans = MAKE_I4WORD(Header_Buffer_AVHRR(9-1:10-1),!sym.UNSIGNED,Byte_Swap_1b)

   ;;----------------------------------------------------------------------
   ;; unpack end time
   ;;----------------------------------------------------------------------
   data_byte(1-1) = Header_Buffer_AVHRR(11-1)
   data_byte(2-1) = Header_Buffer_AVHRR(12-1)
   xxx = where(data_byte lt 0, cxx)
   if cxx gt 0 then data_byte(xxx) = data_byte(xxx)+256
   i4word =  data_byte(1-1)*256L + data_byte(2-1)

   End_Year_Temp = i4word / (long(2)^9)
   End_day_Temp = i4word - Start_Year_Temp * (long(2)^9)

   i4word = MAKE_I4WORD(Header_Buffer_AVHRR(13-1:16-1),!sym.UNSIGNED,Byte_Swap_1b)
   End_Time_Temp = i4word - ((i4word/long(2)^27)*(long(2)^27))

   ;;----------------------------------------------------------------------
   ;; unpack quality indicators
   ;;-----------------------------------------------------------------------
;;  tip_parity = MAKE_I2WORD(Header_Buffer_AVHRR(139-1:140-1),!sym.UNSIGNED,Byte_Swap_1b)
;;  aux_sync = MAKE_I2WORD(Header_Buffer_AVHRR(141-1:142-1),!sym.UNSIGNED,Byte_Swap_1b)
;;  ramp_auto_Cal =  MAKE_I2WORD(Header_Buffer_AVHRR(187-1:188-1),!sym.UNSIGNED,Byte_Swap_1b)

 end 
 
 
 
 
 
 
 
 
 
;;eqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeq
;; NOAA 15+ unpack avhrr header record - routine should work for lac and AVHRR_GAC_Flag
;;
;; Modified January 2005 for NOAA-N
;;
;; Modified December 2008 for segment processing
;;eqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeqeq
pro UNPACK_AVHRR_HEADER_RECORD_KLM,Sc_Id,data_type,Start_Year_Temp,$
              Start_day_Temp,Start_Time_Temp,Num_Scans,End_Year_Temp,End_day_Temp,End_Time_Temp, $
              tip_parity,aux_sync,ramp_auto_Cal,proc_block_Id,ver_1b,   $
		Byte_Swap_1b,Header_Buffer_AVHRR

   Start_Year_Temp=0 & Start_day_Temp=0   
   End_Year_Temp=0 & End_day_Temp=0 
   tip_parity=0 & aux_sync=0   
   ramp_auto_Cal=0
   
   Start_Time_Temp=0L & End_Time_Temp=0L & Num_Scans=0L
   
   proc_block_Id = strarr(7) ;''   ;;;length = 7
      
   i = 0
   
;;-----------------------------------------------------------------------
;; begin executable code
;;-----------------------------------------------------------------------
      Sc_Id = MAKE_I2WORD(Header_Buffer_AVHRR(73-1:74-1),!sym.UNSIGNED,Byte_Swap_1b)
      data_type = MAKE_I2WORD(Header_Buffer_AVHRR(77-1:78-1),!sym.UNSIGNED,Byte_Swap_1b)

      for i = 1,7 do begin  ;;1,len(proc_block_Id) 
       proc_block_Id(i-1:i-1) =  string(Header_Buffer_AVHRR(66+(i-1)-1))
      endfor 

;;----------------------------------------------------------------------
;; unpack 1b version number
;;----------------------------------------------------------------------
      ver_1b = MAKE_I2WORD(Header_Buffer_AVHRR(5-1:6-1),!sym.UNSIGNED,Byte_Swap_1b)

     ;;--- correct for noaa-15 bug in version number
     if ((ver_1b eq 1) and (Sc_Id eq 4)) then begin
       ver_1b = 2
     endif  

     ;;----------------------------------------------------------------------
     ;; unpack starting Time_Temp
     ;;----------------------------------------------------------------------
     Start_Year_Temp = MAKE_I2WORD(Header_Buffer_AVHRR(85-1:86-1),!sym.UNSIGNED,Byte_Swap_1b)
     Start_day_Temp = MAKE_I2WORD(Header_Buffer_AVHRR(87-1:88-1),!sym.UNSIGNED,Byte_Swap_1b)
     Start_Time_Temp = MAKE_I4WORD(Header_Buffer_AVHRR(89-1:92-1),!sym.UNSIGNED,Byte_Swap_1b)
   
    ;;----------------------------------------------------------------------
    ;; unpack  number of scan lines
    ;; note, the POD stores this number as a two byte unsigned integer
    ;;----------------------------------------------------------------------
    Num_Scans = MAKE_I4WORD(Header_Buffer_AVHRR(129-1:130-1),!sym.UNSIGNED,Byte_Swap_1b)

    ;;----------------------------------------------------------------------
    ;; unpack end Time_Temp
    ;;----------------------------------------------------------------------
    End_Year_Temp = MAKE_I2WORD(Header_Buffer_AVHRR(97-1:98-1),!sym.UNSIGNED,Byte_Swap_1b)
    End_day_Temp = MAKE_I2WORD(Header_Buffer_AVHRR(99-1:100-1),!sym.UNSIGNED,Byte_Swap_1b)
    End_Time_Temp = MAKE_I4WORD(Header_Buffer_AVHRR(101-1:104-1),!sym.UNSIGNED,Byte_Swap_1b)

    ;;----------------------------------------------------------------------
    ;; unpack quality indicators - check this
    ;;-----------------------------------------------------------------------
    tip_parity = MAKE_I2WORD(Header_Buffer_AVHRR(139-1:140-1),!sym.UNSIGNED,Byte_Swap_1b)
    aux_sync = MAKE_I2WORD(Header_Buffer_AVHRR(141-1:142-1),!sym.UNSIGNED,Byte_Swap_1b)
    ramp_auto_Cal =  MAKE_I2WORD(Header_Buffer_AVHRR(187-1:188-1),!sym.UNSIGNED,Byte_Swap_1b)

 end 







pro DETERMINE_AVHRR_1,year, Sc_Id,AVHRR_KLM_Flag_Flag, avhrr_1_Flag    
    ;integer(kind=int2), intent(in):: year       ;;year of this data-set   
    ;integer(kind=int4), intent(in):: AVHRR_KLM_Flag_Flag   ;;AVHRR_KLM_Flag flag (yes/no)  ??? data type
    ;integer(kind=int4), intent(out):: avhrr_1_Flag  ;;AVHRR/1 flag (yes/no)  ??? data type

     ;;--- initialize to no
     avhrr_1_Flag = !sym.NO

     ;;--- check Sc_Id for pre-AVHRR_KLM_Flag data
     if (AVHRR_KLM_Flag_Flag eq !sym.NO) then begin
      if (Sc_Id eq 1 and year lt 1985) then avhrr_1_Flag = !sym.YES   ;;TIROS-N (NOAA-11 repeat) 
      if (Sc_Id eq 2) then avhrr_1_Flag = !sym.YES   ;;NOAA-6
      if (Sc_Id eq 6) then avhrr_1_Flag = !sym.YES   ;;NOAA-8
      if (Sc_Id eq 8) then avhrr_1_Flag = !sym.YES   ;;NOAA-10
     endif
end



;;-------------------------------------------------------------------
;;-- define a Sc_Id that is unique value for each satellite
;;-------------------------------------------------------------------

 pro ASSIGN_AVHRR_SAT_ID_NUM_INTERNAL,AVHRR_KLM_Flag,Sc_Id,avhrr_1_Flag, $
            Avhrr_Number,Sc_Id_WMO, Sc_Id_Char,Platform_Name_Attribute,Sensor_Name_Attribute
  ;; Avhrr_Number ;;same as Sc_Id_Internal
  
  Avhrr_Number = 0L
  if (AVHRR_KLM_Flag eq !sym.NO) then begin
    if(Sc_Id eq 1 and avhrr_1_Flag eq !sym.YES) then begin  ;;TIROS-N
       Avhrr_Number = 5   ;;;same as Sc_Id_Internal 
       Sc_Id_WMO = 708
       Sc_Id_Char = 'tirosn'
       Platform_Name_Attribute = 'TIROS-N'
       Sensor_Name_Attribute = 'AVHRR-1'
    endif
    if(Sc_Id eq 2 and avhrr_1_Flag eq !sym.YES) then begin  ;;NOAA-6
       Avhrr_Number = 6
       Sc_Id_WMO = 706
       Sc_Id_Char = 'noaa06'
       Platform_Name_Attribute = 'NOAA-6'
       Sensor_Name_Attribute = 'AVHRR-1'
    endif
    if(Sc_Id eq 4 and AVHRR_KLM_Flag eq !sym.NO) then begin  ;;NOAA-7
       Avhrr_Number = 7
       Sc_Id_WMO = 707
       Sc_Id_Char = 'noaa07'
       Platform_Name_Attribute = 'NOAA-7'
       Sensor_Name_Attribute = 'AVHRR-2'
    endif
    if(Sc_Id eq 6 and avhrr_1_Flag eq !sym.YES) then begin  ;;NOAA-8
       Avhrr_Number = 8
       Sc_Id_WMO = 200
       Sc_Id_Char = 'noaa08'
       Platform_Name_Attribute = 'NOAA-8'
       Sensor_Name_Attribute = 'AVHRR-1'
    endif
    if(Sc_Id eq 7 and avhrr_1_Flag eq !sym.NO) then begin  ;;NOAA-9
       Avhrr_Number = 9
       Sc_Id_WMO = 201
       Sc_Id_Char = 'noaa09'
       Platform_Name_Attribute = 'NOAA-9'
       Sensor_Name_Attribute = 'AVHRR-2'
    endif
    if(Sc_Id eq 8 and avhrr_1_Flag eq !sym.YES) then begin  ;;NOAA-10
       Avhrr_Number = 10
       Sc_Id_WMO = 202
       Sc_Id_Char = 'noaa10'
       Platform_Name_Attribute = 'NOAA-10'
       Sensor_Name_Attribute = 'AVHRR-1'
    endif
    if(Sc_Id eq 1 and avhrr_1_Flag eq !sym.NO) then begin  ;;NOAA-11
       Avhrr_Number = 11
       Sc_Id_WMO = 203
       Sc_Id_Char = 'noaa11'
       Platform_Name_Attribute = 'NOAA-11'
       Sensor_Name_Attribute = 'AVHRR-2'
    endif
    if(Sc_Id eq 5 and AVHRR_KLM_Flag eq!sym.NO) then begin             ;;NOAA-12
       Avhrr_Number = 12
       Sc_Id_WMO = 204
       Sc_Id_Char = 'noaa12'
       Platform_Name_Attribute = 'NOAA-12'
       Sensor_Name_Attribute = 'AVHRR-2'
    endif
    if(Sc_Id eq 3 and AVHRR_KLM_Flag eq !sym.NO) then begin           ;;NOAA-14
       Avhrr_Number = 14
       Sc_Id_WMO = 205
       Sc_Id_Char = 'noaa14'
       Platform_Name_Attribute = 'NOAA-14'
       Sensor_Name_Attribute = 'AVHRR-2'
    endif
  endif else begin
    if(Sc_Id eq 4 and AVHRR_KLM_Flag eq !sym.YES) then begin          ;;NOAA-15
       Avhrr_Number = 15
       Sc_Id_WMO = 206
       Sc_Id_Char = 'noaa15'
       Platform_Name_Attribute = 'NOAA-15'
       Sensor_Name_Attribute = 'AVHRR-3'
    endif
    if(Sc_Id eq 2 and AVHRR_KLM_Flag eq !sym.YES) then begin          ;;NOAA-16
       Avhrr_Number = 16
       Sc_Id_WMO = 207
       Sc_Id_Char = 'noaa16'
       Platform_Name_Attribute = 'NOAA-16'
       Sensor_Name_Attribute = 'AVHRR-3'
    endif
    if(Sc_Id eq 6 and AVHRR_KLM_Flag eq !sym.YES) then begin          ;;NOAA-17
       Avhrr_Number = 17
       Sc_Id_WMO = 208
       Sc_Id_Char = 'noaa17'
       Platform_Name_Attribute = 'NOAA-17'
       Sensor_Name_Attribute = 'AVHRR-3'
    endif
    if(Sc_Id eq 7 and AVHRR_KLM_Flag eq !sym.YES) then begin          ;;NOAA-18
       Avhrr_Number = 18
       Sc_Id_WMO = 209
       Sc_Id_Char = 'noaa18'
       Platform_Name_Attribute = 'NOAA-18'
       Sensor_Name_Attribute = 'AVHRR-3'
    endif
    if(Sc_Id eq 8 and AVHRR_KLM_Flag eq !sym.YES) then begin         ;;NOAA-19
       Avhrr_Number = 19
       Sc_Id_WMO = 223
       Sc_Id_Char = 'noaa19'
       Platform_Name_Attribute = 'NOAA-19'
       Sensor_Name_Attribute = 'AVHRR-3'
    endif
    if(Sc_Id eq 12 and AVHRR_KLM_Flag eq !sym.YES) then begin         ;;Metop-A
       Avhrr_Number =  2
       Sc_Id_WMO = 4
       Sc_Id_Char = 'metopa'
       Platform_Name_Attribute = 'METOP-A'
       Sensor_Name_Attribute = 'AVHRR-3'
    endif

    if(Sc_Id eq 11 and AVHRR_KLM_Flag eq !sym.YES) then begin         ;;Metop-B
       Avhrr_Number =  1
       Sc_Id_WMO = 3
       Sc_Id_Char = 'metopb'
       Platform_Name_Attribute = 'METOP-B'
       Sensor_Name_Attribute = 'AVHRR-3'
    endif
    ;;------- Metop-C Sc_Id numbers are not unknown at this time
    if(Sc_Id eq 13 and AVHRR_KLM_Flag eq !sym.YES) then begin         ;;Metop-C
       Avhrr_Number =  4
       Sc_Id_WMO = 5
       Sc_Id_Char = 'metopc'
       Platform_Name_Attribute = 'METOP-C'
       Sensor_Name_Attribute = 'AVHRR-3'
    endif

  endelse

end 


