   ;-------------------------------------------------------------------------------------- 
   ;avhrr_caligration_subs_1.pro    2015-08-14  Yinghui Liu
   ;-------------------------------------------------------------------------------------- 
   ; AVHRR Polar Pathfinder (APP) PROCESSING SOFTWARE Version 1.1
   ; NAME: avhrr_caligration_subs_1.pro
   ; PURPOSE: subroutine to extract, and calibrate AVHRR GAC data
   ; DESCRIPTION: subroutine to calibrate AVHRR Global Area Coverage (GAC).
   ;              The calibration approach is developed by Andy Heidinger (Andrew.Heidinger@noaa.gov). 
   ;              It was in Fortran language, and converted to IDL language by the authors. 
   ;
   ; AUTHORS:
   ;  Yinghui Liu, yinghuil@ssec.wisc.edu
   ;  Richard Dworak, rdworak@ssec.wisc.edu
   ;  Jeffrey Key, jeff.key@noaa.gov
 
   ; COPYRIGHT  
   ; THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
   ; DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
   ; FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
   ; INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
   ; EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
   ; DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
   ; THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
   ; SUPPORT TO USERS.
   
   ; Software required
   ; IDL: Interactive Data Language
   
   ; Routines in this script and their purpose:
   ; IDL Subtoutines in this file
   ;   
   ;-------------------------------------------------------------------------------------- 





;;-------------------------------------------------------------------------------
;; to use for AVHRR_GAC_Flag data:
;;  1. set AVHRR_GAC_Flag to true (1)
;;  2. read the AVHRR 1B data records dIrectly into Buffer
;;
;; to use for lac data:
;;  1. set AVHRR_GAC_Flag to false (0)
;;  2. combine the two  data-records from the AVHRR 1B file  into Buffer
;;
;;
;; Modifications
;;  January 2005 - Modified for NOAA-N
;;
;;  Modified for one scan-line at a time
;;-------------------------------------------------------------------------------
 pro UNPACK_AVHRR_DATA_RECORD_KLM,Line_Idx

   @commonblock_1.pro  
   
   i4word = 0L
   i2word = 0
   Num_data_bytes=0L
   Num_Chan=0L
   Num_clavr_bytes=0L
   Num_digtel_bytes=0L
   Num_house_bytes=0L
   Start_Pixel=0L
   End_Pixel=0L
   onebyte = 0b
   
;;-------------------------------------------------------------------------------
;; set parameters for AVHRR_GAC_Flag or lac processing
;;-------------------------------------------------------------------------------
   if (AVHRR_GAC_Flag eq !sym.YES) then begin
      Num_data_bytes = 4000 - 1265 + 1
      Num_digtel_bytes =  4016 - 4001 + 1
      Num_house_bytes =  4048 - 4017 + 1
      Num_clavr_bytes = 4160 - 4049 + 1
   endif else begin
      Num_data_bytes = 14928 - 1265
      Num_digtel_bytes = 14944 - 14929
      Num_house_bytes = 14976 - 14971
      Num_clavr_bytes = 15496 - 14977
   endelse

   Num_Chan = (size(Chan_Counts_Avhrr,/dimensions))(0)  ;;;5

   Valid_Therm_Cal = !sym.true

   ;;----------------------------------------------------------------
   ;; unpack scan line number
   ;;----------------------------------------------------------------
   Scan_Number(Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(1-1:2-1),!sym.UNSIGNED,Byte_Swap_1b)

   ;;----------------------------------------------------------------------
   ;; unpack Time_Temp code
   ;;----------------------------------------------------------------------
   Scan_Year(Line_Idx-1) = MAKE_I2WORD(Buffer_AVHRR(3-1:4-1),!sym.UNSIGNED,Byte_Swap_1b)
   Scan_Day(Line_Idx-1) = MAKE_I2WORD(Buffer_AVHRR(5-1:6-1),!sym.UNSIGNED,Byte_Swap_1b)
   Scan_Time(Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(9-1:12-1),!sym.UNSIGNED,Byte_Swap_1b)
   if (Scan_Time(Line_Idx-1) gt !sym.Biggest_I4) then Scan_Time(Line_Idx-1) = !sym.Missing_Value_Int4
   if (Scan_Time(Line_Idx-1) lt -1*!sym.Biggest_I4) then Scan_Time(Line_Idx) = !sym.Missing_Value_Int4

   ;;----------------------------------------------------------------------
   ;; unpack quality indicators - not done fully
   ;;----------------------------------------------------------------------
   i2word  = Buffer_AVHRR(25-1)
   if (i2word lt 0) then begin
      i2word = i2word + 256
   endif
   Fatal_AVHRR(Line_Idx-1) = ishft(i2word, -7)

   i2word  = Buffer_AVHRR(13-1)
   if (i2word lt 0) then begin
      i2word = i2word + 256
   endif
   Ascend(Line_Idx-1) = ishft(i2word, -7)

   i2word  = Buffer_AVHRR(14-1)
   if (i2word lt 0) then begin
      i2word = i2word + 256
   endif
   Ch3a_On_AVHRR(Line_Idx-1) = ishft(ishft(i2word, 6),-6)
   Ch6_On_Pixel_Mask(*,Line_Idx-1) = Ch3a_On_AVHRR(Line_Idx-1)

   i2word  = Buffer_AVHRR(28-1)
   if (i2word lt 0) then begin
      i2word = i2word + 256
   endif
   Resync_AVHRR(Line_Idx-1) = ishft(ishft(i2word, 6),-7)

;;---------------------------------------------------------------------
;; unpack space view
;;---------------------------------------------------------------------
  for j = 1,10 do begin
   Space_Count_Temp(1-1,j-1) = MAKE_I2WORD(Buffer_AVHRR(1161+10*(j-1)-1:1162+10*(j-1)-1),!sym.SIGNED,Byte_Swap_1b)
   Space_Count_Temp(2-1,j-1) = MAKE_I2WORD(Buffer_AVHRR(1163+10*(j-1)-1:1164+10*(j-1)-1),!sym.SIGNED,Byte_Swap_1b)
   Space_Count_Temp(3-1,j-1) = MAKE_I2WORD(Buffer_AVHRR(1165+10*(j-1)-1:1166+10*(j-1)-1),!sym.SIGNED,Byte_Swap_1b)
   Space_Count_Temp(4-1,j-1) = MAKE_I2WORD(Buffer_AVHRR(1167+10*(j-1)-1:1168+10*(j-1)-1),!sym.SIGNED,Byte_Swap_1b)
   Space_Count_Temp(5-1,j-1) = MAKE_I2WORD(Buffer_AVHRR(1169+10*(j-1)-1:1170+10*(j-1)-1),!sym.SIGNED,Byte_Swap_1b)
  endfor
  
;;----------------------------------------------------------------------
;; unpack calibration coefficients
;;----------------------------------------------------------------------
  for Chan_Idx = 1, 3 do begin

    vis_slope_1(Chan_Idx-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(49+(Chan_Idx-1)*60-1:52+(Chan_Idx-1)*60-1), $
                                     !sym.SIGNED,Byte_Swap_1b) / (10.0^7)

    vis_intercept_1(Chan_Idx-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(53+(Chan_Idx-1)*60-1:56+(Chan_Idx-1)*60-1), $
                                         !sym.SIGNED,Byte_Swap_1b) / (10.0^6)

    vis_slope_2(Chan_Idx-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(57+(Chan_Idx-1)*60-1:60+(Chan_Idx-1)*60-1), $
                                     !sym.SIGNED,Byte_Swap_1b) / (10.0^7)

    vis_intercept_2(Chan_Idx-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(61+(Chan_Idx-1)*60-1:64+(Chan_Idx-1)*60-1), $
                                     !sym.SIGNED,Byte_Swap_1b) / (10.0^6)

    vis_intersection(Chan_Idx-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(65+(Chan_Idx-1)*60-1:68+(Chan_Idx-1)*60-1), $
                                             !sym.SIGNED,Byte_Swap_1b)

 endfor

 for Chan_Idx = 1,3 do begin
    ir_coef_1_1b(Chan_Idx+2-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(229+(Chan_Idx-1)*24-1:232+(Chan_Idx-1)*24-1), $
                                              !sym.SIGNED,Byte_Swap_1b)/(10.0^6)
    ir_coef_2_1b(Chan_Idx+2-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(233+(Chan_Idx-1)*24-1:236+(Chan_Idx-1)*24-1), $
                                              !sym.SIGNED,Byte_Swap_1b) / (10.0^6)
    ir_coef_3_1b(Chan_Idx+2-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(237+(Chan_Idx-1)*24-1:240+(Chan_Idx-1)*24-1), $
                                              !sym.SIGNED,Byte_Swap_1b) / (10.0^6)
 endfor

;;-----------------------------------------------------------------------------
;; handling NOAA-N change to scaling of ir_coef_3_1b for ch4 and ch5 
;; the scaling changed from 10^6 to 10^7 - account for extra
;; factor of 10 in NOAA-N format here
;;------------------------------------------------------------------------------
  if (ver_1b ge 3) then begin
    ir_coef_3_1b(4-1,Line_Idx-1) = ir_coef_3_1b(4-1,Line_Idx-1) / 10.0
    ir_coef_3_1b(5-1,Line_Idx-1) = ir_coef_3_1b(5-1,Line_Idx-1) / 10.0
  endif

;;----------------------------------------------------------------------
;; unpack number of Anchor points
;;----------------------------------------------------------------------
;;   Num_loc = Buffer_Temp(53)
    Num_loc = 51

;;----------------------------------------------------------------------
;; unpack angle data
;;----------------------------------------------------------------------
      for j = 1, 51 do begin
        Byte_Idx = 329 + (j-1)*6
        Solzen_Anchor(j-1,Line_Idx-1) = MAKE_I2WORD(Buffer_AVHRR(Byte_Idx-1:Byte_Idx+1-1),!sym.SIGNED,Byte_Swap_1b)/(10.0^2)
        Satzen_Anchor(j-1,Line_Idx-1) = MAKE_I2WORD(Buffer_AVHRR(Byte_Idx+2-1:Byte_Idx+3-1),!sym.SIGNED,Byte_Swap_1b)/(10.0^2)
        Relaz_Anchor(j-1,Line_Idx-1) = MAKE_I2WORD(Buffer_AVHRR(Byte_Idx+4-1:Byte_Idx+5-1),!sym.SIGNED,Byte_Swap_1b)/(10.0^2)
      endfor

;;----------------------------------------------------------------------
;; unpack earth location
;;----------------------------------------------------------------------
      j = 1
      for j = 1,51 do begin
        Byte_Idx = 641 + (j-1)*8 
        Lat_Anchor_1b(j-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(Byte_Idx-1:Byte_Idx+3-1),!sym.SIGNED,Byte_Swap_1b)/(10.0^4)
        Lon_Anchor_1b(j-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(Byte_Idx+4-1:Byte_Idx+7-1),!sym.SIGNED,Byte_Swap_1b)/(10.0^4)
      endfor

;;-- make sure longitudes are correct ( ?? check this)
      xxx = where(Lon_Anchor_1b(*,Line_Idx-1) gt 180,cxx)
      if cxx gt 0 then Lon_Anchor_1b(xxx,Line_Idx-1) = Lon_Anchor_1b(xxx,Line_Idx-1) - 360.

;;----------------------------------------------------------------------
;; unpack channel  data
;;----------------------------------------------------------------------
   Chan_Idx = 1
   Pix_Idx = 1

   for Byte_Idx = 1265,1264+Num_data_bytes,4 do begin

      i4word = MAKE_I4WORD(Buffer_AVHRR(Byte_Idx-1:Byte_Idx+3-1),!sym.UNSIGNED,Byte_Swap_1b)

      for j = 1,3 do begin
       if (Pix_Idx gt Num_Pix) then begin
           break ;exit
       endif

;;--- old
;;      Chan_Counts_Avhrr(Chan_Idx,Pix_Idx,Line_Idx) = i4word/ (1024^(3-j))
;;      i4word = i4word - Chan_Counts_Avhrr(Chan_Idx,Pix_Idx,Line_Idx)*(1024^(3-j))
;;--- new
       Chan_Counts_Avhrr(Chan_Idx-1,Pix_Idx-1,Line_Idx-1) = ishft(ishft(i4word,2+10*(j-1)),-22)

       Chan_Idx = Chan_Idx + 1
        if (Chan_Idx gt Num_Chan) then begin
          Chan_Idx = 1
          Pix_Idx = Pix_Idx + 1
        endif
      endfor

      if (Pix_Idx gt Num_Pix) then begin
            break ;exit
      endif

   endfor

    ;;-------------------------------------------------------------------------
    ;; read in cloud mask 
    ;;-----------------------------------------------------------------------
    Pix_Idx = 0
    word_clavr_start = 1264 + Num_data_bytes + Num_digtel_bytes + Num_house_bytes + 1
 
    ;;------ clavr status flag
    onebyte = Buffer_AVHRR(word_clavr_start + 3-1)
    Clavr_Status_1b_AVHRR(Line_Idx-1) = ishft(ishft(onebyte,7),-7)

    ;;--- cloud mask
    Start_word = word_clavr_start + 8
    End_word = Start_word + Num_clavr_bytes

    for iword = Start_word, End_word do begin
      onebyte = Buffer_AVHRR(iword-1)
      Pix_Idx = 1
      Start_Pixel = (iword - Start_word)*4 + 1
      End_Pixel = Num_Pix< (Start_Pixel+3)

      for Pix_Idx = Start_Pixel, End_Pixel do begin
         if (Pix_Idx gt Num_Pix) then begin
           break ;exit
         endif
         i = (Pix_Idx - Start_Pixel) + 1
         Cld_Mask_Aux(Pix_Idx-1,Line_Idx-1) = ishft(ishft(onebyte,2*(i-1)),-6)
      endfor
    endfor

    ;;--- set flag to communicate that mask was read in
    Cloud_Mask_Aux_Read_Flag = !sym.YES

  end 


;!!--------------------------------------------------------------------------------------
 pro UNPACK_AVHRR_DATA_RECORD,Line_Idx
   
   @commonblock_1.pro  
   
   i4word = 0L  
   i2word = 0   
   Num_data_bytes = 0L 
   Num_Chan = 0 
   Chan_Idx = 0
   Pix_Idx = 0
   j = 0
   Byte_Idx = 0 
   
   data_byte = intarr(2)
   slope = fltarr(5)
   intercept = fltarr(5)

   ;;-------------------------------------------------------------------------------
   ;; set parameters for AVHRR_GAC_Flag or lac processing
   ;;-------------------------------------------------------------------------------
   if (AVHRR_GAC_Flag eq !sym.YES) then begin
      Num_data_bytes = 2728
   endif else begin
      Num_data_bytes = 6952 + 6704
   endelse
   Num_Chan = (size(Chan_Counts_Avhrr,/dimensions))(0)

   ;;----------------------------------------------------------------
   ;; unpack scan line number
   ;;----------------------------------------------------------------
   Scan_Number(Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(1-1:2-1),!sym.UNSIGNED,Byte_Swap_1b)

   ;;----------------------------------------------------------------------
   ;; unpack Time_Temp code
   ;;----------------------------------------------------------------------
   data_byte(1-1) = Buffer_AVHRR(3-1)
   data_byte(2-1) = Buffer_AVHRR(4-1)
   xxx = where(data_byte lt 0, cxx)
   if cxx gt 0 then data_byte(xxx) = data_byte(xxx)+256
   i4word = data_byte(1-1)*256L + data_byte(2-1)
   Scan_Year(Line_Idx-1) = i4word / (long(2)^9)
   Scan_Day(Line_Idx-1) = i4word - Scan_Year(Line_Idx-1) * (long(2)^9)

   i4word = MAKE_I4WORD(Buffer_AVHRR(5-1:8-1),!sym.UNSIGNED,Byte_Swap_1b)
   Scan_Time(Line_Idx-1) = i4word - ((i4word/long(2)^27)*(long(2)^27))
   if (Scan_Time(Line_Idx-1) gt !sym.Biggest_I4)  then Scan_Time(Line_Idx-1) = !sym.Missing_Value_Int4
   if (Scan_Time(Line_Idx-1) lt -1*!sym.Biggest_I4) then Scan_Time(Line_Idx-1) = !sym.Missing_Value_Int4

   ;;----------------------------------------------------------------------
   ;; unpack quality indicators - not done fully
   ;;----------------------------------------------------------------------
   i2word = Buffer_AVHRR(9-1)
   if (i2word lt 0) then i2word = i2word + 256
   Fatal_AVHRR(Line_Idx-1) = ishft(i2word, -7)

   i2word = Buffer_AVHRR(9-1)
   if (i2word lt 0) then  i2word = i2word + 256
   Ascend(Line_Idx-1) = ishft(ishft(i2word, 6),-7)

   ;;--- this needs to be verified
   i2word = Buffer_AVHRR(10-1)
   if (i2word lt 0) then  i2word = i2word + 256
   Resync_AVHRR(Line_Idx-1) = ishft(ishft(i2word, 0),-7)
   
;;----------------------------------------------------------------------
;; unpack calibration coefficients
;;----------------------------------------------------------------------
    Chan_Idx = 1
    for Byte_Idx = 13,52,8 do begin
     slope(Chan_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(Byte_Idx-1:Byte_Idx+3-1),!sym.SIGNED,Byte_Swap_1b) / (2.0^30)
     intercept(Chan_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(Byte_Idx+4-1:Byte_Idx+7-1),!sym.SIGNED,Byte_Swap_1b) / (2.0^22)
     Chan_Idx = Chan_Idx + 1
    endfor

;;----------------------------------------------
;; mimic noaa AVHRR_KLM_Flag type calibration coefficients
;;----------------------------------------------
    vis_slope_1(1-1,Line_Idx-1) = slope(1-1)   ;;high gain is same as low gain
    vis_slope_1(2-1,Line_Idx-1) = slope(2-1)   ;;high gain is same as low gain
    vis_slope_1(3-1,Line_Idx-1) = 0.0        ;;no channel 3a
    vis_slope_2(*,Line_Idx-1) = vis_slope_1(*,Line_Idx-1)   ;;high gain is same as low gain
    vis_intercept_2(*,Line_Idx-1) = vis_intercept_1(*,Line_Idx-1)   ;;high gain intercept same as low
    vis_intersection(*,Line_Idx-1) = 1024     ;;these values are ignored

    ir_linear_slope_1b(3-1,Line_Idx-1) = slope(3-1)
    ir_linear_slope_1b(4-1,Line_Idx-1) = slope(4-1)
    ir_linear_slope_1b(5-1,Line_Idx-1) = slope(5-1)

    ir_linear_intercept_1b(3-1,Line_Idx-1) = intercept(3-1)
    ir_linear_intercept_1b(4-1,Line_Idx-1) = intercept(4-1)
    ir_linear_intercept_1b(5-1,Line_Idx-1) = intercept(5-1)

    ;;----------------------------------------------------------------------
    ;; unpack geolocation coefficients
    ;;----------------------------------------------------------------------
    Num_loc = Buffer_AVHRR(53-1)

    ;;----------------------------------------------------------------------
    ;; unpack solar zenith angle data - treating as unsigned integer
    ;;----------------------------------------------------------------------
    for Byte_Idx = 54,104 do begin
        j = Byte_Idx - 53
        if (Buffer_AVHRR(Byte_Idx-1) lt 0) then begin
             Solzen_Anchor(j-1,Line_Idx-1) = (256.0 + Buffer_AVHRR(Byte_Idx-1)) / 2.0
        endif else begin
             Solzen_Anchor(j-1,Line_Idx-1) = Buffer_AVHRR(Byte_Idx-1) / 2.0
        endelse
    endfor

    ;;----------------------------------------------------------------------
    ;; unpack earth location
    ;;----------------------------------------------------------------------
    j = 1
    for Byte_Idx = 105,308,4 do begin
        Lat_Anchor_1b(j-1,Line_Idx-1) = MAKE_I2WORD(Buffer_AVHRR(Byte_Idx-1:Byte_Idx+1-1),!sym.SIGNED,Byte_Swap_1b) / 128.0
	;if Line_Idx eq 11 then begin
	;   print,Buffer_AVHRR(Byte_Idx+2-1:Byte_Idx+3-1)
	;   print,MAKE_I2WORD(Buffer_AVHRR(Byte_Idx+2-1:Byte_Idx+3-1),!sym.SIGNED,Byte_Swap_1b) / 128.0
	;endif   
        Lon_Anchor_1b(j-1,Line_Idx-1) = MAKE_I2WORD(Buffer_AVHRR(Byte_Idx+2-1:Byte_Idx+3-1),!sym.SIGNED,Byte_Swap_1b) / 128.0
        j = j + 1
    endfor
	;if Line_Idx eq 11 then stop

    ;;-- make sure longitudes are correct ( ?? check this)
    ;xxx = where(Lon_Anchor_1b(*,Line_Idx-1) gt 180,cxx)
    ;if cxx gt 0 then Lon_Anchor_1b(xxx,Line_Idx-1) = Lon_Anchor_1b(xxx,Line_Idx-1) - 360.
    
    ;;----------------------------------------------------------------------
    ;; unpack channel  data
    ;;----------------------------------------------------------------------
    Chan_Idx = 1
    Pix_Idx = 1

    for Byte_Idx = 449,448+Num_data_bytes,4 do begin

      i4word = MAKE_I4WORD(Buffer_AVHRR(Byte_Idx-1:Byte_Idx+3-1),!sym.UNSIGNED,Byte_Swap_1b)

      for j = 1,3 do begin

        if (Pix_Idx gt Num_Pix) then begin
            break  ;exit
        endif

        Chan_Counts_Avhrr(Chan_Idx-1,Pix_Idx-1,Line_Idx-1) = ishft(ishft(i4word,2+10*(j-1)),-22)

        Chan_Idx = Chan_Idx + 1

        if (Chan_Idx gt Num_Chan) then begin
          Chan_Idx = 1
          Pix_Idx = Pix_Idx + 1
        endif

      endfor

      if (Pix_Idx gt Num_Pix) then begin
            break ;exit
      endif

    endfor

  end 
  


;;----------------------------------------------------------------------------------------
;;  Read data records for AAPP
;;----------------------------------------------------------------------------------------
 pro UNPACK_AVHRR_DATA_RECORD_AAPP,Line_Idx
   
   @commonblock_1.pro  
   
   i2word = 0
   Num_Chan=0L
   Num_clavr_bytes=0L
   Num_digtel_bytes=0L
   Num_house_bytes=0L
   Chan_Idx = 0
   Pix_Idx = 0
   Anchor_Idx = 0
   j = 0
   Byte_Idx = 0
   onebyte = 0b
   first_data_byte = 0L
   
   ;;-------------------------------------------------------------------------------
   ;; set parameters for AVHRR_GAC_Flag or lac processing
   ;;-------------------------------------------------------------------------------
   if (AVHRR_GAC_Flag eq !sym.YES) then begin
      print, "ERROR: Cannot handle GAC AAPP, stopping"
      stop
   endif

   first_data_byte = 1265
   Num_digtel_bytes = 14944 - 14929
   Num_house_bytes = 14976 - 14971
   Num_clavr_bytes = 0

   Num_Chan = (size(Chan_Counts_Avhrr,/dimensions))(0)

   Valid_Therm_Cal = !sym.true

   ;;----------------------------------------------------------------
   ;; unpack scan line number
   ;;----------------------------------------------------------------
   Scan_Number(Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(1-1:2-1),!sym.UNSIGNED,Byte_Swap_1b)
 
   ;;----------------------------------------------------------------------
   ;; unpack Time_Temp code
   ;;----------------------------------------------------------------------
   Scan_Year(Line_Idx-1) = MAKE_I2WORD(Buffer_AVHRR(3-1:4-1),!sym.UNSIGNED,Byte_Swap_1b)
   Scan_Day(Line_Idx-1) = MAKE_I2WORD(Buffer_AVHRR(5-1:6-1),!sym.UNSIGNED,Byte_Swap_1b)
   Scan_Time(Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(9-1:12-1),!sym.UNSIGNED,Byte_Swap_1b)
   if (Scan_Time(Line_Idx-1) gt Biggest_I4) then Scan_Time(Line_Idx-1) = !sym.Missing_Value_Int4
   if (Scan_Time(Line_Idx-1) lt -1*Biggest_I4) then Scan_Time(Line_Idx-1) = !sym.Missing_Value_Int4

   ;;----------------------------------------------------------------------
   ;; unpack quality indicators - not done fully
   ;;----------------------------------------------------------------------
   i2word  = Buffer_AVHRR(25-1)
   if (i2word lt 0) then begin
      i2word = i2word + 256
   endif
   Fatal_AVHRR(Line_Idx-1) = ishft(i2word, -7)

   ;;--- Ch3a/b flag - note difference in position nesdis (due to byte swapping)
   i2word  = Buffer_AVHRR(14-1)
   if (i2word lt 0) then begin
      i2word = i2word + 256
   endif
   Ascend(Line_Idx-1) = ishft(i2word, -7)

   ;;--- Ch3a/b flag - note difference in position and meaning from nesdis
   i2word  = Buffer_AVHRR(13-1)
   if (i2word lt 0) then begin
      i2word = i2word + 256
   endif
   onebyte = ishft(ishft(i2word, 6),-6)

   Ch3a_On_AVHRR(Line_Idx-1) = onebyte
   if (onebyte eq 0) then begin 
      Ch3a_On_AVHRR(Line_Idx-1) = 1
   endif
   if (onebyte eq 1) then begin 
      Ch3a_On_AVHRR(Line_Idx-1) = 0
   endif
   Ch6_On_Pixel_Mask(*,Line_Idx-1) = Ch3a_On_AVHRR(Line_Idx-1)

   i2word  = Buffer_AVHRR(28-1)
   if (i2word lt 0) then begin
      i2word = i2word + 256
   endif
   Resync_AVHRR(Line_Idx-1) = ishft(ishft(i2word, 6),-7)

   ;;---------------------------------------------------------------------
   ;; unpack space view
   ;;---------------------------------------------------------------------
   for j = 1,10 do begin
    Space_Count_Temp(1-1,j-1) = MAKE_I2WORD(Buffer_AVHRR(1161+10*(j-1)-1:1162+10*(j-1)-1),!sym.SIGNED,Byte_Swap_1b)
    Space_Count_Temp(2-1,j-1) = MAKE_I2WORD(Buffer_AVHRR(1163+10*(j-1)-1:1164+10*(j-1)-1),!sym.SIGNED,Byte_Swap_1b)
    Space_Count_Temp(3-1,j-1) = MAKE_I2WORD(Buffer_AVHRR(1165+10*(j-1)-1:1166+10*(j-1)-1),!sym.SIGNED,Byte_Swap_1b)
    Space_Count_Temp(4-1,j-1) = MAKE_I2WORD(Buffer_AVHRR(1167+10*(j-1)-1:1168+10*(j-1)-1),!sym.SIGNED,Byte_Swap_1b)
    Space_Count_Temp(5-1,j-1) = MAKE_I2WORD(Buffer_AVHRR(1169+10*(j-1)-1:1170+10*(j-1)-1),!sym.SIGNED,Byte_Swap_1b)
  endfor

  ;;----------------------------------------------------------------------
  ;; unpack calibration coefficients
  ;;----------------------------------------------------------------------
  ;;--- note switch in scale factors from nesdis
  ;;--- in AAPP, the operational and test coefficients are always zero, so
  ;;--- here we read in the prelaunch

  for Chan_Idx = 1, 3 do begin
    vis_slope_1(Chan_Idx-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(89+(Chan_Idx-1)*60-1:92+(Chan_Idx-1)*60-1), $
                                           !sym.SIGNED,Byte_Swap_1b) / (10.0^10)
    vis_intercept_1(Chan_Idx-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(93+(Chan_Idx-1)*60-1:96+(Chan_Idx-1)*60-1), $ 
                                               !sym.SIGNED,Byte_Swap_1b) / (10.0^7)
    vis_slope_2(Chan_Idx-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(97+(Chan_Idx-1)*60-1:100+(Chan_Idx-1)*60-1), $
                                           !sym.SIGNED,Byte_Swap_1b) / (10.0^10)
    vis_intercept_2(Chan_Idx-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(101+(Chan_Idx-1)*60-1:104+(Chan_Idx-1)*60-1), $
                                               !sym.SIGNED,Byte_Swap_1b) / (10.0^7)
    vis_intersection(Chan_Idx-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(105+(Chan_Idx-1)*60-1:108+(Chan_Idx-1)*60-1), $
                                                !sym.SIGNED,Byte_Swap_1b)
  endfor

  ;;--- note switch from nesdis in the order
  for Chan_Idx = 1,3 do begin
    ir_coef_3_1b(Chan_Idx+2-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(229+(Chan_Idx-1)*24-1: $
                          232+(Chan_Idx-1)*24-1),!sym.SIGNED,Byte_Swap_1b)/(10.0^9)
    ir_coef_2_1b(Chan_Idx+2-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(233+(Chan_Idx-1)*24-1: $
                          236+(Chan_Idx-1)*24-1),!sym.SIGNED,Byte_Swap_1b) / (10.0^6)
    ir_coef_1_1b(Chan_Idx+2-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(237+(Chan_Idx-1)*24-1: $
                          240+(Chan_Idx-1)*24-1),!sym.SIGNED,Byte_Swap_1b) / (10.0^6)
  endfor

  ;;----------------------------------------------------------------------
  ;; unpack number of Anchor points
  ;;----------------------------------------------------------------------
  ;;Num_Loc = Buffer_AVHRR(53)
  Num_Loc = 51

  ;;----------------------------------------------------------------------
  ;; unpack angle data
  ;;----------------------------------------------------------------------
  for Anchor_Idx = 1, Num_Loc do begin
        Byte_Idx = 329 + (Anchor_Idx-1)*6
        Solzen_Anchor(Anchor_Idx-1,Line_Idx-1) = MAKE_I2WORD(Buffer_AVHRR(Byte_Idx-1:Byte_Idx+1-1),!sym.SIGNED,Byte_Swap_1b)/(10.0^2)
        Satzen_Anchor(Anchor_Idx-1,Line_Idx-1) = MAKE_I2WORD(Buffer_AVHRR(Byte_Idx+2-1:Byte_Idx+3-1),!sym.SIGNED,Byte_Swap_1b)/(10.0^2)
        Relaz_Anchor(Anchor_Idx-1,Line_Idx-1) = MAKE_I2WORD(Buffer_AVHRR(Byte_Idx+4-1:Byte_Idx+5-1),!sym.SIGNED,Byte_Swap_1b)/(10.0^2)
  endfor

  ;;----------------------------------------------------------------------
  ;; unpack earth location
  ;;----------------------------------------------------------------------
  Anchor_Idx = 1
  for Anchor_Idx = 1,Num_Loc do begin
        Byte_Idx = 641 + (Anchor_Idx-1)*8
        Lat_Anchor_1b(Anchor_Idx-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(Byte_Idx-1:Byte_Idx+3-1),!sym.SIGNED,Byte_Swap_1b)/(10.0^4)
        Lon_Anchor_1b(Anchor_Idx-1,Line_Idx-1) = MAKE_I4WORD(Buffer_AVHRR(Byte_Idx+4-1:Byte_Idx+7-1),!sym.SIGNED,Byte_Swap_1b)/(10.0^4)
  endfor

  ;;----------------------------------------------------------------------
  ;; unpack channel  data
  ;;----------------------------------------------------------------------

  ;;--- AAPP has each observation stored as two byte integers - not packed
  First_Data_Byte = 1265

  for Pix_Idx = 1, 2048 do begin
    for Chan_Idx = 1,5 do begin
       Byte_Idx = First_Data_Byte + (Pix_Idx-1)*10 + (Chan_Idx-1)*2
       Chan_Counts_Avhrr(Chan_Idx-1,Pix_Idx-1,Line_Idx-1) =  $
                         MAKE_I2WORD(Buffer_AVHRR(Byte_Idx-1:Byte_Idx+1-1),!sym.UNSIGNED,Byte_Swap_1b) 
    endfor
  endfor

end






