   ;-------------------------------------------------------------------------------------- 
   ;avhrr_caligration_subs_3.pro    2015-08-14  Yinghui Liu
   ;-------------------------------------------------------------------------------------- 
   ; AVHRR Polar Pathfinder (APP) PROCESSING SOFTWARE Version 1.1
   ; NAME: avhrr_caligration_subs_3.pro
   ; PURPOSE: subroutine to extract, and calibrate AVHRR GAC data
   ; DESCRIPTION: subroutine to calibrate AVHRR Global Area Coverage (GAC).
   ;              The calibration approach is developed by Andy Heidinger (Andrew.Heidinger@noaa.gov). 
   ;              It was in Fortran language, and converted to IDL language by the authors. 
   ;
   ; AUTHORS:
   ;  Yinghui Liu, yinghuil@ssec.wisc.edu
   ;  Richard Dworak, rdworak@ssec.wisc.edu
   ;  Jeffrey Key, jeff.key@noaa.gov
 
   ; COPYRIGHT  
   ; THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
   ; DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
   ; FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
   ; INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
   ; EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
   ; DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
   ; THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
   ; SUPPORT TO USERS.
   
   ; Software required
   ; IDL: Interactive Data Language
   
   ; Routines in this script and their purpose:
   ; IDL Subtoutines in this file
   ;   
   ;-------------------------------------------------------------------------------------- 





;;---------------------------------------------------------------------
;; gnomic navigation Anchor interpolation routine
;; note, this is only Valid for lat and lon, do not apply to angles
;;---------------------------------------------------------------------
 pro GNOMIC_ANCHOR_INTERP,Lon_Anchor_deg,Lat_Anchor_deg,longitude,latitude,num_pix

   ;real (kind=ipre), dimension(:), intent(in):: Lon_Anchor_deg,Lat_Anchor_deg
   ;real (kind=ipre), dimension(:), intent(out):: longitude,latitude
   ;real (kind=ipre), dimension(:), allocatable:: k,x_Anchor,y_Anchor, $
   ;                                              Lon_Anchor,Lat_Anchor
   ;real (kind=ipre), dimension(Num_Pix):: x,y
   ;real (kind=ipre):: yy,xx,xa1,xa2,rho,v,v_prev,c,Lat_center,Lon_center,offset
   ;integer:: ia1,ia2,Pix_Idx,Num_Anchor,Num_skip,Num_start
   
   ;Num_Anchor = size(reform(Lon_Anchor_deg),/dimensions)  ;;should be 51
   Num_Anchor = n_elements(reform(Lon_Anchor_deg))  ;;should be 51
   Num_skip = Num_Pix / Num_Anchor
   Num_start = (Num_Pix - Num_skip*(Num_Anchor-1))/2 + 1
      
   ;k = fltarr(Num_Anchor)
   ;x_Anchor = fltarr(Num_Anchor)
   ;y_Anchor = fltarr(Num_Anchor)
   ;Lon_Anchor = fltarr(Num_Anchor)
   ;Lat_Anchor = fltarr(Num_Anchor)
   
   x = fltarr(Num_Pix)  &  y = fltarr(Num_Pix)
   
   longitude = fltarr(Num_Pix) & latitude=fltarr(Num_Pix)
      
   ;;--- initialize
     ;latitude = 0.0
     ;longitude = 0.0

   ;;--- convert Anchors to radians
     Lat_Anchor = Lat_Anchor_deg * !dtor
     Lon_Anchor = Lon_Anchor_deg * !dtor

   ;;--- pick center for projection (in radians)
     Lat_center = Lat_Anchor(Num_Anchor/2-1)
     Lon_center = Lon_Anchor(Num_Anchor/2-1)

   ;;--- convert to gnomic space
     k = sin(Lat_center) * sin(Lat_Anchor) +  $
         cos(Lat_center) * cos(Lat_Anchor) * cos(Lon_Anchor - Lon_center)
     k = 1.0 / k

     x_Anchor = k * cos(Lat_Anchor) * sin(Lon_Anchor - Lon_center)
     y_Anchor = k * (cos(Lat_center) * sin(Lat_Anchor) -  $
                     sin(Lat_center) * cos(Lat_Anchor) * cos(Lon_Anchor - Lon_center))

   ;;--- linear interp in gnomic space
      for Pix_Idx = 1, Num_Pix do begin

         ;;--- determine two closest Anchor points
         ;ia1 = max(1,min(Num_Anchor-1,(Pix_Idx - Num_start)/Num_skip + 1))
         ia1 = 1>((Num_Anchor-1)<((Pix_Idx - Num_start)/Num_skip + 1))
         ia2 = ia1 + 1

         ;;--- determine weights of two closest Anchor points
         xa1 = (ia1-1)*(Num_skip) + Num_start
         xa2 = (ia2-1)*(Num_skip) + Num_start

         ;;--- determine point to interpolate to
         xx = Pix_Idx

         ;;--- interp x at xx
         yy = (xx-xa1)*(x_Anchor(ia2-1)-x_Anchor(ia1-1))/(xa2-xa1) +  $
                   x_Anchor(ia1-1)
         x(Pix_Idx-1) = yy
         
         ;;--- interp y at xx
         yy = (xx-xa1)*(y_Anchor(ia2-1)-y_Anchor(ia1-1))/(xa2-xa1) +  $
                   y_Anchor(ia1-1)
         y(Pix_Idx-1) = yy

      endfor

      ;;--- convert other quantities
      v_prev = 0.0
      offset = 0
      for Pix_Idx = 1,Num_Pix do begin

         ;;--- some needed constants
        rho = sqrt(x(Pix_Idx-1)^2 + y(Pix_Idx-1)^2)

        ;;--- handle center point where rho = 0.0
        if (rho eq 0.0) then begin 
           latitude(Pix_Idx-1) = Lat_center
           longitude(Pix_Idx-1) = Lon_center
           continue ;;cycle
        endif
        c = atan(rho)

        ;;--- v is the longitude in radians relative to the center longitude
        v = atan( x(Pix_Idx-1) * sin(c) /  $
          (rho * cos(Lat_center) * cos(c) - y(Pix_Idx-1) * sin(Lat_center) * sin(c)) )

        ;;--- look for a jump and determine offset
        if (Pix_Idx eq 1) then begin
          v_prev = v
        endif

;;       dv = v - v_prev
;;       offset = 0.0
;;       if (abs(dv) gt 0.9*pi) then begin
;;         if (dv < 0.0) then begin
;;           offset = pi
;;         endif else begin
;;           offset = -1.0*pi
;;         endelse
;;       endif

;;       if (Pix_Idx eq 205) then begin 
;;          print *, dv, 0.9*pi, offset
;;       endif
;;       v = v + offset

        latitude(Pix_Idx-1) = asin(cos(c) * sin(Lat_center) + (y(Pix_Idx-1)*sin(c)*cos(Lat_center) / rho))
        longitude(Pix_Idx-1) = Lon_center + v

        v_prev = v

      endfor

        ;;--- convert to degrees
      latitude = latitude / !dtor
      longitude = longitude / !dtor
     

        ;;----- account for discontinuity if there was one
   xxx = where(longitude gt 180., cxx)
   if cxx gt 0 then longitude(xxx) = longitude(xxx)-360.
   xxx = where(longitude lt -180., cxx)
   if cxx gt 0 then longitude(xxx) = longitude(xxx)+360.


  ;deallocate(k)
  ;deallocate(x_Anchor)
  ;deallocate(y_Anchor)
  ;deallocate(Lon_Anchor)
  ;deallocate(Lat_Anchor)

  end 





pro LAGRANGIAN_ANCHOR_INTERP,Anchor,y,num_pix

temp_y = fltarr(409)

m = 1    ;;2 means 5 point interp, 1 means 3 point interp
Num_Anchor = n_elements(reform(Anchor))  ;;should be 51

;;--- initialize
temp_Anchor = reform(Anchor)

;;-------- check for a discontinuity (ie. crossing the 180W meridian)
discon = 0
if ((min(temp_Anchor) lt -140.0) and (max(temp_Anchor) gt 140.0)) then begin
  discon = 1
  xxx = where(temp_Anchor lt 0., cxx)
  if cxx gt 0 then temp_Anchor(xxx) = temp_Anchor(xxx)+360.
endif

;;-- perform 2m+1 point Lagrangian interpolation
  
Pix_Idx = indgen(409)+1
ia = (Pix_Idx-5)/8+1
ia1 =  Num_Anchor<(1>(ia-m))
ia3 =  1>(Num_Anchor<(ia+m))
iwhere = where(ia3 lt 3,count)
if count gt 0 then ia3(iwhere) = 3
iwhere = where(ia1 gt Num_Anchor-2, count)
if count gt 0 then ia1(iwhere) = Num_Anchor - 2
ia2 = ia1 + 1

iwhere = where(Pix_Idx-5 mod 8 eq 0, count)
if count gt 0 then temp_y(Pix_Idx(iwhere)-1) = temp_Anchor(ia1(iwhere)-1)

iwhere = where(Pix_Idx-5 mod 8 ne 0, count)
if count gt 0 then begin
  x = Pix_Idx(iwhere)
  x1 = (ia1(iwhere)-1)*8.+5
  x2 = x1+8.
  x3 =x2+8.
  temp_y(x-1) = temp_Anchor(ia1(iwhere)-1)*(x-x2)*(x-x3)/((x1-x2)*(x1-x3)) +  $
    temp_Anchor(ia2(iwhere)-1)*(x-x1)*(x-x3)/((x2-x1)*(x2-x3)) +  $
    temp_Anchor(ia3(iwhere)-1)*(x-x1)*(x-x2)/((x3-x1)*(x3-x2))
end

y = temp_y
   
;;----- account for any discontinuities
xxx = where(y gt 180., cxx)
if cxx gt 0 then y(xxx) = y(xxx)-360.
xxx = where(y lt -180., cxx)
if cxx gt 0 then y(xxx) = y(xxx)+360.

end




  pro LAGRANGIAN_ANCHOR_INTERP_test,Anchor,y,num_pix

    temp_y = fltarr(409)

    m = 1    ;;2 means 5 point interp, 1 means 3 point interp
    Num_Anchor = n_elements(reform(Anchor))  ;;should be 51

 ;; I'm leaving `Num_skip' and `Num_start' here as useful constants although they
 ;; are not used in this pro.
    if (Num_Pix eq 409) then begin
       ;; It's a GAC scan line.
       Num_start = 5 ;;number of pixels between Anchors
       Num_skip = 8  ;;pixel number for first Anchor point
    endif else begin
       ;; Must be a LAC scan line.
       Num_start = 25
       Num_skip = 40
    endelse

 ;;--- initialize
    temp_Anchor = reform(Anchor)
    ;Temp_Anchor=fltarr(Num_Anchor)


 ;;-------- check for a discontinuity (ie. crossing the 180W meridian)
     discon = 0
     if ((min(temp_Anchor) lt -140.0) and (max(temp_Anchor) gt 140.0)) then begin
      discon = 1
      xxx = where(temp_Anchor lt 0., cxx)
      if cxx gt 0 then temp_Anchor(xxx) = temp_Anchor(xxx)+360.
     endif

 ;;-- perform 2m+1 point Lagrangian interpolation
       for Pix_Idx = 1, 409 do begin
          ia = (Pix_Idx-5)/8+1
          ia1 =  Num_Anchor<(1>(ia-m))
          ia3 =  1>(Num_Anchor<(ia+m))
          if ia3 lt 3 then ia3 = 3
          if ia1 gt Num_Anchor-2 then ia1 = Num_Anchor-2
          ia2 = ia1+1

          x = Pix_Idx
          if (x-5) mod 8 eq 0 then begin
             temp_y(Pix_Idx-1) = temp_Anchor(ia-1)
          endif else begin
             x1 = (ia1-1)*8.+5 & x2 = x1+8. & x3 =x2+8.
             temp_y(Pix_Idx-1) = temp_Anchor(ia1-1)*(x-x2)*(x-x3)/((x1-x2)*(x1-x3)) +  $
                temp_Anchor(ia2-1)*(x-x1)*(x-x3)/((x2-x1)*(x2-x3)) +  $
                temp_Anchor(ia3-1)*(x-x1)*(x-x2)/((x3-x1)*(x3-x2))
          endelse
       endfor

   y = temp_y
 ;;----- account for any discontinuities
    xxx = where(y gt 180., cxx)
    if cxx gt 0 then y(xxx) = y(xxx)-360.
    xxx = where(y lt -180., cxx)
    if cxx gt 0 then y(xxx) = y(xxx)+360.

  end




;;-------------------------------------------------------------------------------
;; COMPUTE_ANGLE_ANCHORS
;;
;; input
;;   jday_Temp -  Julian Day
;;   Time_Temp - Time_Temp of day_Temp
;;   Lon_Temp - vector of longitude (degrees)
;;   Lat_Temp - vector of latitude (degrees)
;;
;; output
;;   Satzen_Temp - vector of satellite view zenith angles (degrees)
;;   Solzen_Temp - vector of solar zenith angles (degrees)
;;   Relaz_Temp - vector of relative azimuth angles
;;   Solaz_Temp - vector of solar azimuth angles
;; 
;; Revision History
;;   Jan 2003 - Coded A. Heidinger (Based on CLAVR-1 code)   
;;
;; note - Relaz is the relative azimuth defined as this
;;   Relaz = 180 means the sun is behind you (backscatter)
;;   Relaz = 0 mean the sun is infront of you (forward scatter)
;;   this is convention used in noaa-AVHRR_KLM_Flag Relaz but not in clavr-1
;;-------------------------------------------------------------------------------
   pro COMPUTE_ANGLE_ANCHORS,jday_Temp,   $
                                    Time_Temp,   $
                                    Lon_Temp,    $
                                    Lat_Temp,    $
                                    Satzen_Temp, $
                                    Solzen_Temp, $
                                    Relaz_Temp,  $
                                    Solaz_Temp,  $
                                    Sataz_Temp,  $
                                    Glint_Temp,  $
                                    Scat_Temp
      
      Geox= 0.

      ;;--- convert time to fraction hours
      Frac_Time_Temp = Time_Temp / (60.0*60.0*1000.0)

      ;;--- determine size of the arrays to process
      ;n = size(Lon_Temp,/dimensions) ;;; 1 dimension
      n= n_elements(Lon_Temp)
      
      ;;--- make a local int4 julian day copy
      Jday_Temp4 = Jday_Temp

     ;;-------------------------------------------------
     ;; solar zenith angle
     ;;-------------------------------------------------
      for i = 1, n do begin
         POSSOL,Jday_Temp4, $
                   Frac_Time_Temp, $
                   Lon_Temp(i-1), $
                   Lat_Temp(i-1), $
                   Solzen_T, $
                   Solaz_T
	 Solzen_Temp(i-1) = Solzen_T
	 Solaz_Temp(i-1)  = Solaz_T		   
      endfor

      ;;-------------------------------------------------
      ;;---- relative azimuth and satellite zenith angle
      ;;-------------------------------------------------
      ;;-- sub-satellite point
      Lat_Temp_Center = Lat_Temp(26-1)
      Lon_Temp_Center = Lon_Temp(26-1)
      Solzen_Temp_Center = Solzen_Temp(26-1)
      
      ;; call viewing_geometry.pro
      for i = 1,n do begin
         Geox = great_circle_angle(Lon_Temp_Center,Lat_Temp_Center,Lon_Temp(i-1),Lat_Temp(i-1))
	 ;;tropical region only
         Satzen_Temp(i-1) = sensor_zenith_avhrr_anchor(Geox,i)   ;;; i or i-1, should be i
	 ;;
         Relaz_Temp(i-1) = relative_azimuth_avhrr_anchor(Geox,Solzen_Temp_Center,Solzen_Temp(i-1))

         Sataz_Temp(i-1) = SENSOR_AZIMUTH(Lon_Temp_Center,Lat_Temp_Center,Lon_Temp(i-1),Lat_Temp(i-1)) 

         Glint_Temp(i-1) = GLINT_ANGLE(Solzen_Temp(i-1),Satzen_Temp(i-1),Relaz_Temp(i-1)) 

         Scat_Temp(i-1) = SCATTERING_ANGLE(Solzen_Temp(i-1),Satzen_Temp(i-1),Relaz_Temp(i-1)) 
      endfor

   end 










;;---------------------------------------------------------------------
;; linear interpolation routine
;;---------------------------------------------------------------------
 pro LINEAR_ANCHOR_INTERP,Anchor,y,Num_Pix
   ;real (kind=ipre), dimension(:), intent(in):: Anchor
   ;real (kind=ipre), dimension(:), intent(out):: y
   ;real (kind=ipre), dimension(:), allocatable:: temp_Anchor
   ;real (kind=ipre):: x,xa1,xa2
   ;integer:: ia1,ia2,Pix_Idx,Num_Anchor,Num_skip,Num_start
   ;integer:: discon

   ;Num_Anchor = size(reform(Anchor),/dimensions)  ;;should be 51
   Num_Anchor = n_elements(reform(Anchor))  ;;should be 51
   Num_skip = Num_Pix / Num_Anchor
   Num_start = (Num_Pix - Num_skip*(Num_Anchor-1))/2 + 1

   Temp_Anchor = fltarr(Num_Anchor)
   ;;y = fltarr(Num_Pix) ;;; may not be needed in IDL
   ;x=0. & xa1=0. & xa2=0.
   ;ia1=0 & ia2=0 & Pix_Idx=0 & Num_Anchor=0 & Num_skip=0 & Num_start=0
   ;discon = 0
   
;;--- initialize
     y = y*0.0
     temp_Anchor = Anchor

;;-------- check for a discontinuity (ie. crossing dateline)
    discon = 0
    if ((min(temp_Anchor) lt -175.0) and (max(temp_Anchor) gt 175.0)) then begin
     discon = 1
     xxx = where(temp_Anchor lt 0., cxx)
     if cxx gt 0 then temp_Anchor(xxx) =  temp_Anchor(xxx) + 360.0

    endif

;;-----------------------------------------------------------
      for Pix_Idx = 1, Num_Pix do begin
         ;ia1 = max(1,min(Num_Anchor-1,(Pix_Idx - Num_start)/Num_skip + 1))
         ia1 = 1>((Num_Anchor-1)<((Pix_Idx - Num_start)/Num_skip + 1))
         ia2 = ia1 + 1

         xa1 = (ia1-1)*(Num_skip) + Num_start
         xa2 = (ia2-1)*(Num_skip) + Num_start

         x = Pix_Idx

         y(Pix_Idx-1) = (x-xa1)*(temp_Anchor(ia2-1)-temp_Anchor(ia1-1))/(xa2-xa1) +  $
                   temp_Anchor(ia1-1)

      endfor

;;----- account for discontinuity if there was one
  if (discon eq 1) then begin
     xxx = where(y gt 180., cxx)
     if cxx gt 0 then y(xxx) = y(xxx)-360.
  endif

  ;deallocate(Temp_Anchor)

end 


;;---------------------------------------------------------
;; compute the ascending/descending node flag 
;;---------------------------------------------------------
  pro CALCULATE_ASC_DES,Line_Idx_Min_Segment,Num_Scans_Read

        ;integer, intent(in):: Line_Idx_Min_Segment
        ;integer, intent(in):: Num_Scans_Read
        ;real:: diff
        ;real:: diff_before
        ;real:: diff_after
        ;integer:: j

        @commonblock_1.pro  
   

        for j = Line_Idx_Min_Segment+1, Num_Scans_Read - 2 do begin
        
         if (Bad_Scan_Flag(j-1) eq !sym.YES) then begin

            if (j gt Line_Idx_Min_Segment + 1) then begin
              Ascend(j-1) = Ascend(j-1-1)
            endif

            continue ;;cycle ;; Cycle if there is missing navigation

         endif

         diff = Lat_Anchor_1b(25-1,j-1) - Lat_Anchor_1b(25-1,j+1-1)

         Ascend(j-1) = 1
         if (diff le 0) then Ascend(j-1) = 0

         if (Bad_Scan_Flag(j+1-1) eq !sym.YES and  $
              Bad_Scan_Flag(j-1-1) eq !sym.NO) then begin
              Ascend(j-1) = Ascend(j-1-1)
         endif

         diff_before = Lat_Anchor_1b(25-1,j-1-1) - Lat_Anchor_1b(25-1,j-1)
         diff_after = Lat_Anchor_1b(25-1,j+1-1) - Lat_Anchor_1b(25-1,j+2-1)

         ;; check to see if current one is a fluke in diff
         if (diff_after ge 0 and Ascend(j-1-1) eq 1 and  $
              diff le 0 and Ascend(j-1) eq 0)  then begin
              Ascend(j-1) = 1
         endif
         if (Ascend(j-1-1) eq 0 and diff_after le 0 and  $
             diff ge 0 and Ascend(j-1) eq 1)  then begin
             Ascend(j-1) = 0
         endif

        endfor

        Ascend(Line_Idx_Min_Segment-1) = Ascend(Line_Idx_Min_Segment+1-1)
        Ascend(Num_Scans_Read-1-1) = Ascend(Num_Scans_Read - 2-1)
        Ascend(Num_Scans_Read-1) = Ascend(Num_Scans_Read - 2-1)

  end 



