   ;-------------------------------------------------------------------------------------- 
   ;avhrr_caligration_subs_5.pro    2015-08-14  Yinghui Liu
   ;-------------------------------------------------------------------------------------- 
   ; AVHRR Polar Pathfinder (APP) PROCESSING SOFTWARE Version 1.1
   ; NAME: avhrr_caligration_subs_5.pro
   ; PURPOSE: subroutine to extract, and calibrate AVHRR GAC data
   ; DESCRIPTION: subroutine to calibrate AVHRR Global Area Coverage (GAC).
   ;              The calibration approach is developed by Andy Heidinger (Andrew.Heidinger@noaa.gov). 
   ;              It was in Fortran language, and converted to IDL language by the authors. 
   ;
   ; AUTHORS:
   ;  Yinghui Liu, yinghuil@ssec.wisc.edu
   ;  Richard Dworak, rdworak@ssec.wisc.edu
   ;  Jeffrey Key, jeff.key@noaa.gov
 
   ; COPYRIGHT  
   ; THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
   ; DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
   ; FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
   ; INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
   ; EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
   ; DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
   ; THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
   ; SUPPORT TO USERS.
   
   ; Software required
   ; IDL: Interactive Data Language
   
   ; Routines in this script and their purpose:
   ; IDL Subtoutines in this file
   ;   
   ;-------------------------------------------------------------------------------------- 





function dtmjdn,year,month,day,hour,minute,second
    jday=julday(1,1,1970,12,0,0)
    if month ne 0 then begin
       jday1 = julday(month,day,year,hour,minute,second)    
    endif else begin
       jday1 = julday(1,1,year,hour,minute,second)-1.+day    
    endelse
    jdayd = jday1-jday
    return,jdayd
 end
 




;; $Id: reposition_module.f90 9 2014-01-31 08:19:35Z awalther $ 
;;--------------------------------------------------------------------------------------
;; Clouds from AVHRR Extended (CLAVR-x) 1b PROCESSING SOFTWARE Version 5.3
;;
;; NAME: reposition_module.f90 (src)
;;       AVHRR_REPOSITION_ROUTINES (program)
;;
;; PURPOSE: this module houses the non-Nagle routines for repositioning the
;;          AVHRR lat and lon values for time corrections
;;
;; DESCRIPTION:  
;;             Note, the mjdn numbers vary in this module.  The values used to
;;             record the clock errors are referenced to Wednesday November 17, 1858
;;
;;             The values used by Fred Nagles routines are referenced to
;;             12 Z Januar, 1970
;;
;;             The offset between the two is 40,587.5
;;
;;             In INTERPOLATE_CLOCK_ERROR, I will use the standard definition
;;             In REPOSITION_FOR_CLOCK_ERROR, I will use Nagle's definition
;;
;; AUTHORS:
;;  Andrew Heidinger, Andrew.Heidinger@noaa.gov
;;
;; COPYRIGHT
;; THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
;; DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
;; FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
;; INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
;; EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
;; DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;; THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
;; SUPPORT TO USERS.
;;
;; Public routines used in this module:
;; REPOSITION_FOR_CLOCK_ERROR - compute the time for each pixel from the scan value
;;--------------------------------------------------------------------------------------


;;--------------------------------------------------------------------------
;; Interpolate the clock error
;;--------------------------------------------------------------------------
pro INTERPOLATE_CLOCK_ERROR,start_year, start_Day, start_itime, $
                                   end_year, end_Day, end_itime, $
                                   sat_number,clock_error

   ;real(kind=real4), dimension(:), allocatable:: start_mjdn,  $
   ;                                              end_mjdn, $
   ;                                              start_clock_error, $
   ;                                              end_clock_error

   ;integer:: ibin
   ;integer:: day, year, hour, minute,  $
   ;          start_month, end_month, start_dom, end_dom,ileap
   ;integer:: num
   ;real(kind=real4):: second
   ;real(kind=real4):: temp
   ;real(kind=real4):: orbit_start_mjdn
   ;real(kind=real4):: orbit_end_mjdn
   ;real(kind=real4):: orbit_mean_mjdn
   ;real(kind=real8):: DTMJDN

   ;;-------------------------------------------------------------------------------

   @setup_clock_corrections_const.pro
    num = 0
    case sat_number of 
    7: begin
          num = num_n07
          start_mjdn = start_mjdn_n07 
          end_mjdn = end_mjdn_n07 
          start_clock_error = start_clock_error_n07 
          end_clock_error = end_clock_error_n07 
       end
    9: begin
          num = num_n07
          start_mjdn = start_mjdn_n09 
          end_mjdn = end_mjdn_n09 
          start_clock_error = start_clock_error_n09
          end_clock_error = end_clock_error_n09 
       end
    11: begin
          num = num_n11
          start_mjdn = start_mjdn_n11 
          end_mjdn = end_mjdn_n11 
          start_clock_error = start_clock_error_n11 
          end_clock_error = end_clock_error_n11 
       end
    12: begin
          num = num_n12
          start_mjdn = start_mjdn_n12 
          end_mjdn = end_mjdn_n12 
          start_clock_error = start_clock_error_n12 
          end_clock_error = end_clock_error_n12 
       end
    14: begin
          num = num_n14
          start_mjdn = start_mjdn_n14 
          end_mjdn = end_mjdn_n14 
          start_clock_error = start_clock_error_n14 
          end_clock_error = end_clock_error_n14 
       end
    15: begin
          num = num_n15
          start_mjdn = start_mjdn_n15 
          end_mjdn = end_mjdn_n15 
          start_clock_error = start_clock_error_n15 
          end_clock_error = end_clock_error_n15 
       end
    16: begin
          num = num_n16
          start_mjdn = start_mjdn_n16 
          end_mjdn = end_mjdn_n16 
          start_clock_error = start_clock_error_n16 
          end_clock_error = end_clock_error_n16 
       end
    17: begin
          num = num_n17
          start_mjdn = start_mjdn_n17 
          end_mjdn = end_mjdn_n17 
          start_clock_error = start_clock_error_n17 
          end_clock_error = end_clock_error_n17 
       end
    else: begin
            clock_error = !sym.missing_value_real4  
	    return     
	 end
    endcase

    

    ;;--- convert time and date
    ;ileap = 0
    ;ileap = leap_year_fct(int(start_year,kind=int4))
    ;start_month = COMPUTE_MONTH(int(start_day,kind=real4),ileap)
    ;end_month = COMPUTE_MONTH(int(end_day,kind=int4),ileap)
    ;start_dom = COMPUTE_DAY(int(start_day,kind=int4),ileap)
    ;end_dom = COMPUTE_DAY(int(end_day,kind=int4),ileap)
    ileap = 0
    if ((start_year mod 4 eq 0 and start_year mod 100 ne 0) or start_year mod 400 eq 0) $
    then ileap = 1
    caldat,(julday(1,1,start_year)-1+start_day),start_month,start_dom
    caldat,(julday(1,1,start_year)-1+end_day),end_month,end_dom

    ;;--- determine MJDN at start of orbit
    day = start_dom
    month = start_month
    year = start_year
    temp = start_itime / 60.0 / 60.0/ 1000.0
    hour = fix(temp)  ;; int(temp)
    temp = (temp - hour)*60.0
    minute = fix(temp)  ;;int(temp)
    second = (temp - minute)*60.0
    orbit_start_mjdn = $
         float(DTMJDN(year,month,day,hour,minute,second)) + $
         mjdn_offset

    ;;--- determine MJDN at end of orbit
    day = end_dom
    month = end_month
    year = end_year
    temp = end_itime / 60.0 / 60.0/ 1000.0
    hour = fix(temp)
    temp = (temp - hour)*60.0
    minute = fix(temp)
    second = (temp - minute)*60.0
    orbit_end_mjdn =  $
         float(DTMJDN(year,month,day,hour,minute,second)) + $
         mjdn_offset

    orbit_mean_mjdn = 0.5*(orbit_start_mjdn + orbit_end_mjdn)

    ;;---- check to see if time is within the timespan of the clock
    ;;---- correction data for this satellite
    if ( (orbit_mean_mjdn ge end_mjdn(num-1)) or $
          (orbit_mean_mjdn le start_mjdn(1-1))) then begin

        clock_error = !sym.missing_value_real4

        return

     endif

    

   ;;------ interpolate start time

   ;;--- determine which time period bin to use
   for ibin = 1, num do begin
     if ( (orbit_mean_mjdn ge start_mjdn(ibin-1)) and $
          (orbit_mean_mjdn le end_mjdn(ibin-1))) then begin
          break ;exit
     endif
   endfor

   ;;---- check for a successful bin search
   if ((ibin eq num+1)) then begin
           clock_error = !sym.missing_value_real4

   endif else begin   ;;attempt interpolaton

       clock_error = start_clock_error(ibin-1) + $
            (orbit_mean_mjdn - start_mjdn(ibin-1)) * $
            (end_clock_error(ibin-1) - start_clock_error(ibin-1))/ $
            (end_mjdn(ibin-1) - start_mjdn(ibin-1))

   endelse


 end 
 
 
;;--------------------------------------------------------------------------
;; Reposition for a clock error
;;
;; compute the time for each pixel from the scanline time
;;
;;--------------------------------------------------------------------------
pro REPOSITION_FOR_CLOCK_ERROR,AVHRR_GAC_FLAG,j1,j2,timerr,error_flag,           $
		   Num_Pix,start_year,start_day,end_year,end_day, $
		   lat_1b,lon_1b,Bad_Scan_Flag,Scan_Time,start_time, $
				  scan_number

   ;integer, intent(in):: j1,j2
   ;real (kind=real4), intent(in):: timerr
   ;integer, intent(out):: error_flag
   ;integer:: i,j
   ;real, parameter:: senzen_max = 55.3 ;;degrees
   ;real:: alpha
   ;real:: second
   ;real:: temp
   ;integer:: day, year, hour, minute,  $
   ;          start_month, end_month, start_dom, end_dom,ileap
   ;integer (kind=int4):: Pixel_offset, Pixel_spacing, ipixel

   ;real (kind=real4), dimension(num_pix,j2):: lat_temp,lon_temp
   ;real (kind=real8), dimension(num_pix,j2):: Pixel_time_mjdn
   ;integer, dimension(j2):: valid_scan_number
   ;integer, dimension(j2):: valid_scan_index
   ;integer:: num_valid_scans, jj

   ;real (kind=real8), dimension(j1:j2-j1+1):: scan_time_mjdn
   ;real (kind=real8):: time_between_scans

   ;;double precision:: DTMJDN
   
   senzen_max = 55.3 ;;degrees
   scan_time_mjdn = dblarr(j2-j1+1-j1+1)        ;;;
   lat_temp = fltarr(num_pix,j2) & lon_temp = fltarr(num_pix,j2)  
   Pixel_time_mjdn = dblarr(num_pix,j2)

   ;;--- initialize lat and lon from level-1b values
    lat = lat_1b
    lon = lon_1b

    valid_scan_index = intarr(j2)
    valid_scan_number= intarr(j2)
    scan_time_mjdn = scan_time_mjdn*0.+!sym.missing_value_real4

   ;;--- a convenient parameter
    alpha = 2.0*senzen_max / 360.0 / num_pix

   ;;--- compute date terms
    ;ileap = 0
    ;if ((start_year eq 1984) or $
    ;    (start_year eq 1988) or $
    ;    (start_year eq 1992) or $
    ;    (start_year eq 1996) or $
    ;    (start_year eq 2000) or $
    ;    (start_year eq 2004) or $
    ;    (start_year eq 2008) or $
    ;    (start_year eq 2012)) then ileap = 1

    ;start_month = COMPUTE_MONTH(int(start_day,kind=real4),ileap)
    ;end_month = COMPUTE_MONTH(int(end_day,kind=int4),ileap)
    ;start_dom = COMPUTE_DAY(int(start_day,kind=int4),ileap)
    ;end_dom = COMPUTE_DAY(int(end_day,kind=int4),ileap)
    caldat,(julday(1,1,start_year)-1+start_day),start_month,start_dom
    caldat,(julday(1,1,start_year)-1+end_day),end_month,end_dom
    
   ;;--- convert scan line times to MJDN
    jj = 1

   ;valid_scan_loop: do j = j1,j1+j2-1
   for j=j1,j1+j2-1 do begin 
     
      ;;--- check for bad scans
      if ((scan_time(j-1) eq !sym.missing_value_int4) or $
          (bad_scan_flag(j-1) eq !sym.yes))  then begin
          continue  ;;cycle 
      endif

      valid_scan_index(jj-1) = j
      valid_scan_number(jj-1) = scan_number(j-1)

      ;;--- determine date of scan
      if (scan_time(j-1) ge start_time) then begin
         day = start_dom
         month = start_month
         year = start_year
      endif else begin
         day = end_dom
         month = end_month
         year = end_year
      endelse

      ;;--- determine hour, minute and second
      temp = scan_time(j-1) / 60.0 / 60.0/ 1000.0

      hour = fix(temp)
      temp = (temp - hour)*60.0
      minute = fix(temp)
      second = (temp - minute)*60.0

      ;;--- determine MJDN of this scan
      scan_time_mjdn(jj-1) = DTMJDN(year,month,day,hour,minute,second) ;;; double precision

      jj = jj + 1

   endfor

   num_valid_scans = jj - 1

   ;;  print *, "num_valid_scans = ", num_valid_scans

   ;;-- set parameters based on data resolution
   Pixel_Offset = 0
   Pixel_Spacing = 1
   if (AVHRR_GAC_Flag eq !sym.yes) then begin
        Pixel_offset = 8
        Pixel_spacing = 5
   endif

   ;;--- compute pixel level time
   ;Pixel_time_loop:   do jj = 1, num_valid_scans
   for jj = 1, num_valid_scans do begin

      ;;--- determine mean time between lines
      if (jj eq num_valid_scans) then begin
         time_between_scans = (scan_time_mjdn(jj-1) - scan_time_mjdn(jj-1-1)) /  $
                            (valid_scan_number(jj-1) - valid_scan_number(jj-1-1))
      endif else begin
         time_between_scans = (scan_time_mjdn(jj+1-1) - scan_time_mjdn(jj-1)) /  $
                            (valid_scan_number(jj+1-1) - valid_scan_number(jj-1))
      endelse

      ;;--- interpolate time for each pixel in scan
      ipixel = Pixel_offset
      for i = 1, num_pix do begin
         Pixel_time_mjdn(i-1,jj-1) = scan_time_mjdn(jj-1) + (ipixel-1) * alpha * time_between_scans
         ipixel = ipixel + Pixel_spacing
      endfor
      lat_temp(*,jj-1) = lat(*,valid_scan_index(jj-1)-1)
      lon_temp(*,jj-1) = lon(*,valid_scan_index(jj-1)-1)

   endfor  ;Pixel_time_loop

   ;;--- do repositioning

        ;;--- assuming reposnx treats first array element as 1
	
	;;;???????????????????????????????????????????????????????
	pixel_time_mjdn_temp = Pixel_time_mjdn(*,1-1:num_valid_scans-1)
	lat_temp_temp = lat_temp(*,1-1:num_valid_scans-1)
	lon_temp_temp = lon_temp(*,1-1:num_valid_scans-1)
	;;no common block needed
	
	
	goto,skip1
	print,Pixel_time_mjdn_temp(0:1,0:1)
	print,'Pixel_time_mjdn_temp'
	;stop
	print,lat_temp_temp(0:1,0:1)
	print,'lat_temp_temp'
	;stop
	print,'lon_temp_temp'
	print,lon_temp_temp(0:1,0:1)
	;stop
	print,timerr
	print,'timerr'
	;stop
	print,num_pix
	print,'num_pix'
	;stop
	print,num_valid_scans
	print,'num_valid_scans'
	stop
	skip1:
	
	goto,skip2
	openw,uw,'lat_f77.dat',/get_lun,/F77_UNFORMATTED
	   writeu,uw,lat_temp_temp
	free_lun,uw   
	openw,uw,'lon_f77.dat',/get_lun,/F77_UNFORMATTED
	   writeu,uw,lon_temp_temp
	free_lun,uw   
	openw,uw,'mjdn_f77.dat',/get_lun,/F77_UNFORMATTED
	   writeu,uw,Pixel_time_mjdn_temp
	free_lun,uw   
	stop
	skip2:
	
	
	
        REPOSN, Pixel_time_mjdn_temp, $
                     lat_temp_temp, $
                     lon_temp_temp, $
                     timerr,num_pix,num_valid_scans
	;lat_temp(*,1-1:num_valid_scans-1) = lat_temp_temp	     
        ;lon_temp(*,1-1:num_valid_scans-1) = lon_temp_temp
        
	;;--- initialize output lat, lon to missing
        lat = lat*0.+ !sym.missing_value_real4
        lon = lon*0.+ !sym.missing_value_real4

        ;;--- populate output lat, lon with valid scans
        for jj = 1, num_valid_scans do begin
           ;lat(*,valid_scan_index(jj-1)) = lat_temp(*,jj-1)
           ;lon(*,valid_scan_index(jj-1)) = lon_temp(*,jj-1)
           lat(*,valid_scan_index(jj-1)-1)  = lat_temp_temp(*,jj-1)
           lon(*,valid_scan_index(jj-1)-1)  = lon_temp_temp(*,jj-1)
        endfor

        lat_1b = lat
	lon_1b = lon
   ;;--- assign error flag - ask Fred how to this.
     error_flag = !sym.no

end  ;subroutine REPOSITION_FOR_CLOCK_ERROR

