   ;-------------------------------------------------------------------------------------- 
   ;avhrr_caligration_subs_tool.pro    2015-08-14  Yinghui Liu
   ;-------------------------------------------------------------------------------------- 
   ; AVHRR Polar Pathfinder (APP) PROCESSING SOFTWARE Version 1.1
   ; NAME: avhrr_caligration_subs_tool.pro
   ; PURPOSE: subroutine to extract, and calibrate AVHRR GAC data
   ; DESCRIPTION: subroutine to calibrate AVHRR Global Area Coverage (GAC).
   ;              The calibration approach is developed by Andy Heidinger (Andrew.Heidinger@noaa.gov). 
   ;              It was in Fortran language, and converted to IDL language by the authors. 
   ;
   ; AUTHORS:
   ;  Yinghui Liu, yinghuil@ssec.wisc.edu
   ;  Richard Dworak, rdworak@ssec.wisc.edu
   ;  Jeffrey Key, jeff.key@noaa.gov
 
   ; COPYRIGHT  
   ; THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
   ; DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
   ; FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
   ; INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
   ; EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
   ; DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
   ; THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
   ; SUPPORT TO USERS.
   
   ; Software required
   ; IDL: Interactive Data Language
   
   ; Routines in this script and their purpose:
   ; IDL Subtoutines in this file
   ;   
   ;-------------------------------------------------------------------------------------- 





pro mreadf_int,filename,word_start,bytes_per_word, $
           Number_Of_Words,Number_Of_Words_Read,Header_Buffer_Temp
	   
    openr,ur,filename,/get_lun,ERROR=err
       if err ne 0 then begin
          print,!ERROR_STATE.MSG
       endif else begin	  
          Number_Of_Words_Read = Number_Of_Words*bytes_per_word
          Header_Buffer_Temp = bytarr(Number_Of_Words_Read)
          byte_start = word_start*bytes_per_word-1
	  ;print,byte_start,size(byte_start)
          POINT_LUN,ur,byte_start
          if not eof(ur) then begin	   
             readu,ur,Header_Buffer_Temp
          endif else begin
             print,'eof while reading the data"
          endelse	  
       endelse	  
    free_lun,ur

end


;;-----------------------------------------------------------------
;; functions to convert 1 byte integers into 2 or 4 byte integers
;; if the integeris signed (isign = 1) then begin the first byte
;; remains signed
;;+ Fortran byte: singed integer, -128 to 127
;;+ IDL byte: unsinged integer, 0 to 255
;;+ this program is to 1 byte integers in byte1(*) to one integer
;;---------------------------------------------------------------- 
 function MAKE_I2WORD,byte1,isign,iswap
  ;integer(kind=int1), intent(in), dimension(:):: byte1
  ;integer(kind=int4), intent(in):: isign
  ;integer(kind=int4), intent(in):: iswap
  ;integer(kind=int2):: i2word
  ;integer(kind=int2), dimension(:), allocatable:: byte2
  ;integer:: i,n

  n = n_elements(byte1)
  byte2 = intarr(n)

  ;;--- swap bytes if necessary ;;; 
  if (iswap eq !sym.YES) then begin
    for i = 1, n do begin 
      byte2(i-1) = byte1(n-i) 
    endfor
  endif else begin
    byte2 = byte1
  endelse

  ;;--- account for unsigned integers ;;; 
  if ((byte2(1-1) < 0) and (isign eq !sym.NO)) then begin
     byte2(1-1) = byte2(1-1) + 256
  endif
  for i = 2,n do begin
     if ((byte2(i-1) < 0)) then begin
        byte2(i-1) = byte2(i-1) + 256
     endif
  endfor

;;--- make word
   i2word = 0
   i2word = byte2(1-1)*256 + byte2(2-1)
   return,i2word

 end

;;-----
 function MAKE_I4WORD,byte1,isign,iswap
  ;integer(kind=int1), intent(in), dimension(:):: byte1
  ;integer(kind=int4), intent(in):: isign
  ;integer(kind=int4), intent(in):: iswap
  ;integer(kind=int4):: i4word
  ;integer(kind=int2), dimension(:), allocatable:: byte2
  ;integer:: i,n

   n = n_elements(byte1)
   byte2=intarr(n)

;;--- swap bytes if necessary
  if (iswap eq !sym.YES) then begin
    for i = 1, n do begin
       byte2(i-1) = byte1(n-i) 
    endfor
  endif else begin
    byte2 = byte1
  endelse

;;--- account for unsigned integers
   if ((byte2(1-1) < 0) and (isign eq !sym.NO)) then begin
     byte2(1-1) = byte2(1-1) + 256
   endif
   for i = 2,n do begin
    if ((byte2(i-1) < 0)) then begin
     byte2(i-1) = byte2(i-1) + 256
    endif
   endfor
   i4word = 0L
   for i = n,1,-1 do begin
     i4word = i4word + byte2(i-1)*(256L^(n-i))
   endfor

   return,i4word

 end 

