   ;-------------------------------------------------------------------------------------- 
   ;commonblock_1.pro    2015-08-14  Yinghui Liu
   ;-------------------------------------------------------------------------------------- 
   ; AVHRR Polar Pathfinder (APP) PROCESSING SOFTWARE Version 1.1
   ; NAME: commonblock_1.pro
   ; PURPOSE: subroutine to extract, and calibrate AVHRR GAC data
   ; DESCRIPTION: subroutine to calibrate AVHRR Global Area Coverage (GAC).
   ;              The calibration approach is developed by Andy Heidinger (Andrew.Heidinger@noaa.gov). 
   ;              It was in Fortran language, and converted to IDL language by the authors. 
   ;
   ; AUTHORS:
   ;  Yinghui Liu, yinghuil@ssec.wisc.edu
   ;  Richard Dworak, rdworak@ssec.wisc.edu
   ;  Jeffrey Key, jeff.key@noaa.gov
 
   ; COPYRIGHT  
   ; THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
   ; DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
   ; FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
   ; INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
   ; EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
   ; DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
   ; THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
   ; SUPPORT TO USERS.
   
   ; Software required
   ; IDL: Interactive Data Language
   
   ; Routines in this script and their purpose:
   ; IDL Subtoutines in this file
   ;   
   ;-------------------------------------------------------------------------------------- 





common common_avhrr_calibration,   $
          Valid_Ref_cal,           $  ; 1: "TRUE", 0:"FALSE"
          Valid_therm_cal,         $  ; 1: "TRUE", 0:"FALSE" 
          Spinup_New_therm_cal,    $  ; 1: "TRUE", 0:"FALSE" 
          Prt_Idx,                 $
          Cal_Idx,                 $
          Mean_T_Prt,              $ 
          Mean_Space_Count_1,                 $ 
          Mean_Space_Count_2,                 $ 
          Mean_Space_Count_3,                 $ 
          Mean_Space_Count_4,                 $ 
          Mean_Space_Count_5,                 $ 
          Mean_BB_Count_3,                 $ 
          Mean_BB_Count_4,                 $ 
          Mean_BB_Count_5,                 $ 
          Ch1_Gain_Low,                    $ 
          Ch1_Gain_High,                   $ 
          Ch2_Gain_Low,                    $ 
          Ch2_Gain_High,                   $ 
          Ch3a_Gain_Low,                   $ 
          Ch3a_Gain_High,                  $ 
          ;;; upack data
          avhrr_gac_flag,                  $
	  chan_counts_avhrr,               $ 
	  Scan_Number,                     $
	  Buffer_AVHRR,                    $ 
          Scan_Year,                       $
	  Scan_Day,                        $
	  Scan_Time,                       $
	  Byte_Swap_1b,                    $            
	  Fatal_AVHRR,                     $
	  Ascend,                          $
	  Ch3a_On_AVHRR,                   $ 
          Ch6_On_Pixel_Mask,               $
	  Resync_AVHRR,                    $
	  Space_Count_Temp,                $
	  vis_slope_1,                     $
	  vis_intercept_1,                 $
          vis_slope_2,                     $
	  vis_intercept_2,                 $
	  vis_intersection,                $
	  ir_coef_1_1b,                    $
	  ir_coef_2_1b,                    $     
          ir_coef_3_1b,                    $
	  ver_1b,                          $
	  Solzen_Anchor,                    $
	  Satzen_Anchor,                    $
	  Relaz_Anchor,                    $
	  Lat_Anchor_1b,                    $ 
	  Lon_Anchor_1b,                    $
	  Num_Pix,                          $
	  Clavr_Status_1b_AVHRR,           $
	  Cld_Mask_Aux,                    $
          Cloud_Mask_Aux_Read_Flag,        $
	  ir_linear_slope_1b,              $
	  ir_linear_intercept_1b,          $         
          ;;; COMPUTE_NEW_THERM_CAL_COEF
	  Ncal,                            $
	  AVHRR_KLM_Flag,                  $
	  Space_Count_Temp_1,              $
	  Space_Count_Temp_2,              $
	  Space_Count_Temp_3,              $
	  Space_Count_Temp_4,              $
	  Space_Count_Temp_5,              $
	  BB_count_3,BB_count_4,BB_count_5, $
          a1_20,a2_20,nu_20, $
          a1_31,a2_31,nu_31, $
          a1_32,a2_32,nu_32, $
	  b20,b31,b32,       $
	  T_prt,                           $
	  Prt_Coef,                        $
	  ir_linear_slope_New,              $
	  ir_linear_intercept_New,          $         
	  Space_Rad_3b,                    $ 
	  Space_Rad_4,                    $ 
	  Space_Rad_5,                    $ 
          Scan_Space_Counts_Avhrr,         $
          Scan_bb_Counts_Avhrr,           $
          ;;;REF_CAL 
	  ref_cal_1b,                    $ 
	  therm_cal_1b,                    $ 
	  Ch1_Dark_Count,                    $ 
	  Ch2_Dark_Count,                    $ 
	  Ch3a_Dark_Count,                    $ 
	  Ch1_Switch_Count,                    $
	  Ch2_Switch_Count,                    $
	  Ch3a_Switch_Count,                    $
	  Ch1_Gain_Low_0,Ch1_Degrad_Low_1, Ch1_Degrad_Low_2,   $ 
	  Ch1_Gain_High_0,Ch1_Degrad_High_1, Ch1_Degrad_High_2, $	  
	  Ch2_Gain_Low_0,Ch2_Degrad_Low_1, Ch2_Degrad_Low_2,   $
	  Ch2_Gain_High_0,Ch2_Degrad_High_1, Ch2_Degrad_High_2,   $
	  Ch3a_Gain_Low_0,Ch3a_Degrad_Low_1, Ch3a_Degrad_Low_2,   $
	  Ch3a_Gain_High_0,Ch3a_Degrad_High_1, Ch3a_Degrad_High_2,   $
	  ch,                                    $
	  ;;;CALCULATE_ASC_DES
	  ;;;THERM_CAL
	  Sc_Id_WMO,                      $
	  Bad_Scan_Flag,                  $
	  Chan_On_Flag_Default,           $
          b0_3b, b1_3b, b2_3b,            $
          b0_4, b1_4, b2_4,               $
          b0_5, b1_5, b2_5,               $
	  ;;;CONVERT_AVHRR_COUNTS_SINGLE_GAIN
	  Chan_Counts_Avhrr_Sg
	  ;;;REPOSITION_FOR_CLOCK_ERROR  
	  ;;;lat_1b,lon_1b,                    $
	  ;;;start_year,start_day,               $
	  ;;;end_year,end_day,                   $
	  ;;;start_time











