   ;-------------------------------------------------------------------------------------- 
   ;reposnx.pro    2015-08-14  Yinghui Liu
   ;-------------------------------------------------------------------------------------- 
   ; AVHRR Polar Pathfinder (APP) PROCESSING SOFTWARE Version 1.1
   ; NAME: reposnx.pro
   ; PURPOSE: subroutine to extract, and calibrate AVHRR GAC data
   ; DESCRIPTION: subroutine to calibrate AVHRR Global Area Coverage (GAC).
   ;              The calibration approach is developed by Andy Heidinger (Andrew.Heidinger@noaa.gov). 
   ;              It was in Fortran language, and converted to IDL language by the authors. 
   ;
   ; AUTHORS:
   ;  Yinghui Liu, yinghuil@ssec.wisc.edu
   ;  Richard Dworak, rdworak@ssec.wisc.edu
   ;  Jeffrey Key, jeff.key@noaa.gov
 
   ; COPYRIGHT  
   ; THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
   ; DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
   ; FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
   ; INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
   ; EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
   ; DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
   ; THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
   ; SUPPORT TO USERS.
   
   ; Software required
   ; IDL: Interactive Data Language
   
   ; Routines in this script and their purpose:
   ; IDL Subtoutines in this file
   ;   
   ;-------------------------------------------------------------------------------------- 



;;*      rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
;	Function ANGBTW(V1,V2)
;;*
;;*     To compute the Real*4 angle in degrees between two  Vector*4:
;;*
;	Implicit None
;	Double Precision dot,rq81,da1,da2,darg,d0,dtest,dsqrt,dabs
;	Double Precision d1,dsign,dc,dacos
;	Real angbtw
;;*
;	parameter(dc = 180.d+0/3.141592653589793d+0, d0=0.d+0,
;     1  d1=1.d+0)
;	double precision vv1(3),vv2(3)
;	real  v1(3),v2(3)
;;c     xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Function ANGBTW,V1,V2

        dc = 180.d+0/3.141592653589793d+0
	d0=0.d+0
	d1=1.d+0
        vv1 = double(v1) & vv2 = double(v2)
	dot=vv1(1-1)*vv2(1-1)+vv1(2-1)*vv2(2-1)+vv1(3-1)*vv2(3-1)
	da1=vv1(1-1)*vv1(1-1)+vv1(2-1)*vv1(2-1)+vv1(3-1)*vv1(3-1)
	da2=vv2(1-1)*vv2(1-1)+vv2(2-1)*vv2(2-1)+vv2(3-1)*vv2(3-1)
	darg = da1*da2

	if(darg le d0) then begin
	   print, 'ANGBTW: an argument is zero'
	   angbtw = 360.
	endif else begin
	   dtest = dot/sqrt(darg)
	   if(abs(dtest) gt d1) then begin
	      ;dtest=dsign(d1,dtest) ;;fortran function
	      if dtest ge d0 then dtest=1.d else dtest=-1.d
	   endif   
	   angbtw = dc*acos(dtest)
	endelse
	
	return,angbtw
end


;;c--------------------------------------------------------------------------
;;c Clouds from AVHRR Extended (CLAVR-x) 1b PROCESSING SOFTWARE Version 5.3
;;c
;;c NAME: reposnx.f (src)
;;c
;;c PURPOSE: 
;;c
;;c DESCRIPTION: 
;;c
;;c AUTHORS:
;;c  Andrew Heidinger, Andrew.Heidinger@noaa.gov
;;c  Andi Walther, CIMSS, andi.walther@ssec.wisc.edu
;;c  Denis Botambekov, CIMSS, denis.botambekov@ssec.wisc.edu
;;c  William Straka, CIMSS, wstraka@ssec.wisc.edu
;;c
;;c COPYRIGHT
;;c THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
;;c DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
;;c FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
;;c INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
;;c EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
;;c DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;;c THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
;;c SUPPORT TO USERS.
;;c
;;c--------------------------------------------------------------------------

;;      SUBROUTINE REPOSN (DTS,SLATS,SLONGS,TIMERR,IMAX,JMAX)
;;c
;;c     To reposition AVHRR or other data arrays to correct timing error:
;;c     ---------------------------------------------------------
;;*     Inputs:
;;*     dts(imax,jmax) - an array of FOV times in MJDN, possibly erroneous
;;*     slats(imax,jmax) - an array of FOV geodetic latitudes to be adjusted
;;*     slongs(imax,jmax) - an array of FOV longitudes to be adjusted
;;*     timerr - Timing error in seconds; a positive value means the spacecraft
;;*       clock is too fast.
;;*     imax,jmax - the cross-scan and along-scan dimensions of the above arrays
;;c
;;*     The times, latitudes, and longitudes are adjusted and returned by
;;c     this routine.
;;c     -----------------------------------------------------
;;c     This routine was originally written in Meteorological Fortran
;;c     (MeteFor), an extension of Fortran-77, in order to utilize the
;;c     vector and matrix notation available in MeteFor.  This
;;c     routine is also maintained in MeteFor.  Some of the original
;;c     MeteFor code may appear in statements which have been commented
;;c     out.  The original MeteFor source (suffix .hlf) is more
;;c     readable and self-documenting.  See the document MeteFor.doc.
;;c     --------------------------------------------------------
pro REPOSN,DTS,SLATS,SLONGS,TIMERR,IMAX,JMAX
      ;;implicit double precision(d), all variable starting with d is double precision
      d2 = 2.0d
      dtmid = dblarr(2) 
      vmid = fltarr(3,2)
      rq81 = 0.0d   &  rq82 = 0.0d
      vq41 = fltarr(3) & vq42 = fltarr(3)
      v1=fltarr(3) & v2=fltarr(3) & vam=fltarr(3) & vc=fltarr(3) & vdl=fltarr(3)

      k3 = imax/3
      k13 = k3 + 1
      k23 = imax - k3
      re = 6371.

   for j=1,2 do begin
      jj = j
      jg = 1 + (jmax - 1)*(jj - 1)
      dtmid(j-1) = (dts(k13-1,jg-1) + dts(k23-1,jg-1))/d2
      
      vec4,slats(k13-1,jg-1),slongs(k13-1,jg-1),re,vq41
      vcoord,dts(k13-1,jg-1),vq41,'DLC',v1

      vec4,slats(k23-1,jg-1),slongs(k23-1,jg-1),re,vq41
      vcoord,dts(k23-1,jg-1),vq41,'DLC',v2

      vq41(1-1) = v1(1-1)+v2(1-1)
      vq41(2-1) = v1(2-1)+v2(2-1)
      vq41(3-1) = v1(3-1)+v2(3-1)
      vq42(1-1) = vq41(1-1)*.5
      vq42(2-1) = vq41(2-1)*.5
      vq42(3-1) = vq41(3-1)*.5
      vmid(1-1,j-1) = vq42(1-1)
      vmid(2-1,j-1) = vq42(2-1)
      vmid(3-1,j-1) = vq42(3-1)
   endfor 
      
      
      
      rq81=dtmid(2-1)
      rq82=dtmid(1-1)
      vmid1_temp = vmid(*,1-1)  & vmid2_temp = vmid(*,2-1)
      ;;rq41=angbtw(vmid(1-1,1-1),vmid(1-1,2-1))  ;;; fortran use memory address
      rq41=angbtw(vmid1_temp,vmid2_temp)
      angvel=rq41/(rq81-rq82)

      vam(1-1) = vmid(2-1,1-1)*vmid(3-1,2-1) - vmid(2-1,2-1)*vmid(3-1,1-1)
      vam(2-1) = vmid(3-1,1-1)*vmid(1-1,2-1) - vmid(3-1,2-1)*vmid(1-1,1-1)
      vam(3-1) = vmid(1-1,1-1)*vmid(2-1,2-1) - vmid(1-1,2-1)*vmid(2-1,1-1)
      terr = double(timerr)/86400.
      ang = angvel*terr
      
      
   for j=1,jmax do begin
      for  i=1,imax do begin          
          slat = slats(i-1,j-1)
          re = erad84(slat)
          
          vec4,slat,slongs(i-1,j-1),re,vq41
	  
          vcoord,dts(i-1,j-1),vq41,'DLC',vc
	  
          dts(i-1,j-1)=dts(i-1,j-1)-terr
          

          vconic,vc,vam,ang,vc1
	  vc = vc1
	  	  
          vcoord,dts(i-1,j-1),vc,'CDL',vdl
	  
          slats(i-1,j-1) = vdl(1-1)
          slongs(i-1,j-1) = vdl(2-1)
	  
      endfor
   endfor
end
      
pro VEC4,x,y,z,vec
        ;To return a vector whose components are the three given
        ;values x,y,z
	
	vec = fltarr(3)
	vec(1-1) = x
	vec(2-1) = y
	vec(3-1) = z	
end

;;*      rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
;;	Double Precision Function DFRAC(ditem)
;;*
;;*     The function returns the fraction from the algebraically
;;*     lesser integer, i.e.
;;*
;;c     1.7 ===> .7   and   -1.7 ===> .3
;;*
;;	Implicit Double Precision (a-h,o-z)
;;cxr   dflt
;;c     xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Function  DFRAC,ditem
	df = ditem

	if(abs(df) gt 2147483647.9999d0) then begin
	   print,'DFRAC: argument is too large: ',df
	   df = 999999999.
	endif else begin
	   item = df
	   df = df - long(item)
	   if(df lt 0.d+0) then df = df + 1.d+0	   
	endelse
      return,df
end

;;*      rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
;;	Function VERNEQ(dt)
;;*
;;cxr   dfrac
;;*     To return the longitude (positive east) of the Vernal Equinox
;;c     at the given time dt, expressed either as JDN or MJDN:
;;*     See page 37  Montenbruck & Pfleger
;;*
;;*     Compared with old VERNEQ.for  9 Dec 91;  constant error of
;;c     .003 deg;  looks OK.
;;*
;	Implicit None
;	Double Precision dt,mjd,mjd0,dint,ut,t,gst,gstc,dfrac,ve
;	Real verneq
;;c     xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Function VERNEQ,dt
	mjd = dt - 2400000.5d0
	if(dt lt 1721060.d+0) then mjd = mjd + 2440588
	;mjd0 = dint(mjd)
	mjd0 = double(long(mjd))
	ut = 24.d0 * (mjd - mjd0)
	t = (mjd0 - 51544.5d0)/36525.d0
	gst = 6.697374558d0 + 1.0027379093d0 * ut  $
              + (8640184.812866d0+(.093104d0 - 6.2d-6*t)*t)*t/3600.
	gstc = dfrac(gst/24.d0)
	ve = 360. * (1.d0 - gstc)
	if(ve gt 180.d0) then ve = ve - 360.	
	return,ve
end


;;*      rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
;;	Function SINE(xx)
;;*     Argument in degrees
;;*
;;	Implicit Double Precision(d)
;;	Parameter( dc = 3.141592653589793d+0/180.d+0)
;;c     xxxxxxxxxxxxxxxxxxxxxxxxxx
Function SINE,xx
        dc = 3.141592653589793d+0/180.d+0
	dxx = xx
	ds = sin(dc*dxx)
	return,ds
end
;;*      rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
;	Function COSINE(xx)
;;*
;	Implicit Double Precision(d)
;	Parameter( dc = 3.141592653589793d+0/180.d+0)
;;c     xxxxxxxxxxxxxxxxxxxxxxxxx
Function COSINE,xx
        dc = 3.141592653589793d+0/180.d+0
	dxx = xx
	ds = cos(dc*dxx)
	return,ds
end



;;*      rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
;;	Function ARCTAN(xx,yy)
;;*
;;c     This routine was originally written in Meteorological Fortran
;;c     (MeteFor).  Some of the original code may appear in statements
;;c     which have been commented out.  The original MeteFor source
;;c     code may be more self-documenting and readable.
;;*
;;c     19 May 06; on laptop
;;*     SFS; output is in degrees
;;*     29 Aug 01; see Approximations for Digital Computers, Hastings.
;;*
;;	Implicit None
;;	Double Precision dpi,d1,dpio4,d0,dr2d,dc1,dc3,dc5,dc7,dc9
;;	Double Precision dc11,dc13,dc15,dx,dy,dabs,dd,dq,da,da2,dr
;;	Real xx,yy,arctan
;;*
;	Parameter(dpi=3.14159265358979d+0, d1=1.d+0,
;     1  dpio4=dpi/4.d+0, d0=0.d+0, dr2d=180.d+0/dpi,
;     1  dc1=.9999993329d+0, dc3=-.3332985605d+0,
;     2  dc5=.1994653599d+0, dc7=-.1390853351d+0, dc9=.964200441d-1,
;     3  dc11=-.559098861d-1, dc13=.218612288d-1, dc15=-.4054058d-2)
;;c     xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Function ARCTAN,xx,yy
	dpi=3.14159265358979d+0
	d1=1.d+0
        dpio4=dpi/4.d+0
	d0=0.d+0 
	dr2d=180.d+0/dpi
        dc1=.9999993329d+0 
	dc3=-.3332985605d+0
        dc5=.1994653599d+0 
	dc7=-.1390853351d+0 
	dc9=.964200441d-1
        dc11=-.559098861d-1 
	dc13=.218612288d-1 
	dc15=-.4054058d-2


	dx = xx
	dy = yy

	if(abs(dx) le d0) then begin
	   if(abs(dy) le d0) then  begin
	      print,' ARCTAN: both arguments are zero.'
	      dd = 720.d+0
	   endif
	   dd = 90.d+0
	   if(dy lt d0) then dd = -90.d+0
	endif else begin
	   dq  = abs(dy/dx)
	   da = (dq - d1)/(dq + d1)
	   da2 = da*da
	   dr = dpio4 + da*(dc1 + da2*(dc3 + da2*(dc5 + da2*(dc7 + da2*(dc9  $
               + da2*(dc11 + da2*(dc13 + da2*dc15)))))))
	   dd = dr2d*dr

	   if(dx gt d0) then begin
	      if(dy lt d0) then dd = -1.*dd
	   endif else begin
	      dd = 180.d+0 - dd
	      if(dy lt d0) then dd = -1.*dd
	   endelse
	endelse

	return,dd
	end


;;*      rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
;	Function GEOCEN(gdl)
;;*
;;*     See GEODET for explanation of this algorithm.
;;*
;	Implicit None
;	Real gdl,gd,abs,gc,tand,sine,cosine,arctan,b2,geocen,sign,
;     1  smajx,sminx,b
;;*
;	parameter (smajx=6378388., sminx=6356911.946,
;     1  b=sminx/smajx, b2 = b*b)
;;cxr   sine,arctan,cosine
;;c     xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Function GEOCEN,gdl
	smajx=6378388. & sminx=6356911.946
        b=sminx/smajx
	b2 = b*b
	gd = abs(gdl)
	if(gd ge 90.)then begin
	   gc = 90.
	endif else begin
	   tand = sine(gd)/cosine(gd)
	   gc = arctan(1., b2*tand)
	endelse
	if gdl ge 0.d+0 then begin
	   return, gc
	endif else begin
	   return,-1.*gc
	endelse   
	
end






;;*      rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
;	Function GEODET(gcl)
;;*
;;*     To convert geocentric latitude to geodetic:
;;*
;;*  Given an elliptical earth with semi-major axis = 1, and
;;*  semi-minor axis = b, then from the equation of an ellipse,
;;*  we have
;;*
;;*  b2 x2 + y2 = b2                                  (1)
;;*  y = b sqrt(1 - x2)                               (2)
;;*  sqrt(1 - x2) = y/b                               (3)
;;*  dy/dx = -bx/sqrt(1 - x2)        from (2)         (4)
;;*
;;*  But the slope dy/dx of a point on the ellipse is the negative
;;*  reciprocal of the tangent of the geodetic latitude, i.e.
;;*
;;*  tan D = sqrt(1 - x2)/bx          where D = geodetic latitude
;;*  bx tan D = sqrt(1 - x2) = y/b    using (3)
;;*  b2 tan D = y/x
;;*
;;*  On the other hand, y/x is the tangent of the geocentric
;;c  latitude.  Hence
;;*
;;c   b2 tan D = tan C,   or    tan D = (tan C)/b2    (5)
;;*
;;c   which can be used to interconvert latitudes in either direction.
;;*
;	Parameter (smajx=6378388., sminx=6356911.946,
;     1  b=sminx/smajx, b2 = b*b)
;;c     These values of semi-major and semi-minor axis were taken
;;c     from  the Encyclopedia Britannica (see Geodesy).
;;*
;;*     These values are also shown as the
;;*     "International Ellipsoid" as defined in
;;*     "American Practical Navigator" (Bowditch), Pub. No. 9,
;;*     Volume I, p. 1119, Defense Mapping Agency.
;;*
;;cxr   sine,cosine,arctan
;;c     xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Function GEODET,gcl
	smajx=6378388.d 
	sminx=6356911.946d
        b=sminx/smajx
	b2 = b*b
        
	
	gc = abs(gcl)

	if(gc ge 90.)then begin
	   gd = 90.
	endif else begin
	   tanc = sine(gc)/cosine(gc)
	   gd = arctan(b2, tanc)
	endelse
        
	if gcl ge 0.d then begin
	   return,gd
	endif else begin
	   return,-1.*gd
	endelse    
end



;;*      rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
;;c     Vector Function UNIT4(VEC)
;	Subroutine Unit4(vec,uvec)
;;*
;;*     2 Apr 03; to compute the norm of an arbitrary vector:
;;c
;	real vec(3),uvec(3)
;	equivalence(jr2,r2),(sum2,jsum2)
;;*     data j$01/z'20000000'/             dec 536870912
;;c     xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
pro Unit4,vec,uvec
    uvec = vec/norm(vec)
end


      
;;c     rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
;;      Subroutine VCOORD(dt,vecin,cmode,  vecout)
;;*
;;cxr   sine,cosine,geodet,geocen,arctan,verneq
;;! -----------------------------------------------------------------
;;! DOCUMENTATION:
;;*
;;! Cooperative Institute for Meteorological Satellite Studies
;;! University of Wisconsin
;;! 1225 W Dayton Street
;;! Madison, Wisconsin  53706      USA
;;*
;;! Name: VCOORD; a subroutine-type subprogram
;;!     This is a vector-valued function under MeteFor.
;;*
;;! Inputs: Time, real(double), in Julian Day Number
;;!         or Modified Julian Day Number (MJDN)
;;!     a real(single) 3-dimensional vector in some coordinate system
;;!     a character*3 variable stating the desired transformation
;;!     12 Jan 00: Modified Julian Day Number (from epoch of
;;!     1 Jan 70) is also accepted.
;;*
;;! Output: a 3-dimensional real(single) transformed vector
;;*
;;! Usage:
;;!     To inter-convert among various coordinate systems:
;;!      cmode = LLC         Geocentric Lat/Long to Celestial
;;!      cmode = LLT         Geocentric Lat/Long to Terrestrial
;;!      cmode = CT          Celestial to Terrestrial
;;!      cmode = DLC         Geodetic Lat/Long to Celestial
;;!      cmode = DLT         Geodetic Lat/Long to Terrestrial
;;*
;;!     and the inverses of these operations:  (CLL, TLL, TC, DLT, TDL)
;;*
;;!     Note regarding time:  An input value for time (dt) is required
;;!     only if the celestial coordinate system is involved, either as
;;!     input or output coordinates.  If the transformation is
;;!     strictly between Lat/Long/CD on the one hand, and terrestrial
;;!     on the other, a dummy value of time (e.g. zero) can be used.
;;*
;;! Modules:  verneq, sine, cosine, arctan
;;*
;;! Comment:  The three coordinate systems treated by this routine are
;;!   defined as follows:
;;*
;;!   Celestial: The x-coordinate is directed from the center of
;;!   the earth toward the vernal equinox.  The y-coordinate is
;;!   directed 90 deg east of the x-coordinate, and the z-coordinate
;;!   is directed from the earth's center toward the north pole.
;;!   The three coordinates form a right-hand orthonormal system.
;;*
;;!   Terrestrial: The x-coordinate is directed from the earth's center
;;!   toward the Greenwich meridian; the y-coordinate is directed
;;!   90 deg east of the x-coordinate; the z coordinate is directed
;;!   from the earth's center toward the north pole.  The terrestrial
;;!   coordinates are a right-hand orthonormal system.
;;*
;;!   Latitude/Longitude:  The three components are latitude in
;;!   degrees, positive north; longitude in degrees, positive east;
;;!   and radius vector in arbitrary units from the earth's center
;;!   to a given point.  These coordinates are not orthogonal
;;!   vectors.  The coordinates are vectors only in the sense that
;;!   they are three ordered real numbers.  The latitude may be either
;;!   geocentric or geodetic, depending on the conversion mode.
;;*
;;! Programmer: F W Nagle
;;!  ---------------------------------------------------------------------
pro VCOORD,dt,vecin,cmode,  vecout
      ;double precision dt,deqr2,deqrad
      ;character*3 cmodes(10),cmode
      ;dimension vecin(3),vecout(3),vecinp(3)
      ;save
      
      vecinp = fltarr(3)
      cmodes=['LLC', 'LLT', 'CT ', 'CLL', 'TLL', 'TC ',  $
              'DLC', 'DLT', 'CDL', 'TDL']
      kdetic = 0
      lm = 0
      for j=1,10 do begin
         lm = j
         if(cmodes(j-1) eq cmode) then break
      endfor
      jtest = (lm-1)*(lm-3)*(lm-4)*(lm-6)*(lm-7)*(lm-9)
      velong= 0.

      if(jtest eq 0) then begin
         if(dt gt 2.6d+6 or dt lt 0.d+0) then goto, L920
         if(dt gt 100000.d+0 and dt lt 2.4d+6) then goto, L920
         velong= verneq(dt)
      endif

      for j=1,3 do begin
        vecinp(j-1) = vecin(j-1)
      endfor

      if(lm eq 7 or lm eq 8)then begin
        lm=lm-6
        vecinp(1-1)=geocen(vecinp(1-1))
      endif

      if(lm ge 9)then begin
        lm=lm-5
        kdetic = 1
      endif
      
      
      if(lm ge 4)then lm=3-lm
      if(abs(lm) gt 1) then goto,L50
      if(lm lt 0) then goto, L20

L10:  if(abs(vecinp(1-1)) gt 90.)then begin
         print,' VCOORD: Bad input latitude: ',vecinp(1-1) 
         goto, L922
      endif

      deqrad = vecinp(3-1) * cosine(vecinp(1-1))
      clong = vecinp(2-1) - velong
      vecout(1-1) = deqrad * cosine(clong)
      vecout(2-1) = deqrad * sine(clong)
      vecout(3-1) = vecinp(3-1) * sine(vecinp(1-1))
      return

L20:  clong = arctan(vecinp(1-1), vecinp(2-1))
      deqr2 = double(vecinp(1-1))^2 + double(vecinp(2-1))^2
      eqr = sqrt(deqr2)
      vecout(1-1) = arctan(eqr, vecinp(3-1))
      vecout(2-1) = clong + velong
      if(vecout(2-1) lt -180.) then vecout(2-1) = vecout(2-1)+360.
      if(vecout(2-1) gt 180.) then vecout(2-1) = vecout(2-1)-360.
      vecout(3-1) = sqrt(deqr2 + double(vecinp(3-1))^2)
      if(kdetic gt 0) then vecout(1-1)=geodet(vecout(1-1))
      return

L50:  if(abs(lm) gt 2) then goto, L100
      if(lm lt 0) then goto, L20
      goto, L10

L100: if(lm lt 0) then goto, L110

L102:   vecout(3-1) = vecinp(3-1)
	clong = arctan(vecinp(1-1), vecinp(2-1))
	glong = clong + velong
	deqrad = sqrt(double(vecinp(1-1))^2 + double(vecinp(2-1))^2)
	vecout(1-1) = deqrad * cosine(glong)
	vecout(2-1) = deqrad * sine(glong)
	return

L110:   velong = -velong
      goto, L102

L920:   print,'VCOORD: The input time is wrong: ',dt

L922:     vecout(1-1) = 0.
	vecout(2-1) = 0.
	vecout(3-1) = 0.
	return
end













;;*      rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
;	Function ERAD84(gdlat)
;;*
;;*     28 Aug 04; added comment
;;*     9 Jan 03; uses the spheroid WGS 84
;;*     5 Jan 03; Liam Gumley reports that MODIS uses the WGS
;;c     spheroid of 1984, in which the Equatorial radius is
;;c     6378.137 km, and eccentricity .081819190843, which implies
;;c     a polar radius of 6356.75231424498 km, i.e.
;;*     b2 = a2(1 - e2)
;;*
;;c     To compute the local radius of the earth in kilometers as a
;;c     function of geodetic latitude:
;;*     See EARCOMP.hlf
;;*
;	Implicit Double Precision (d)
;;*
;	Parameter (dmajor=6378.137d+0, dminor=6356.752314245d+0,
;     1  b = dminor/dmajor, dmaj2=dmajor*dmajor,
;     2  dratio=dminor/dmajor, degrad=3.1415926535898d+0/180.d+0)
;;*     This is the WGS84 Spheroid.
;;c     xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Function ERAD84,gdlat
	
	dmajor=6378.137d+0 
	dminor=6356.752314245d+0
        b = dminor/dmajor
        dmaj2=dmajor*dmajor
        dratio=dminor/dmajor 
	degrad=3.1415926535898d+0/180.d+0
	
	if(abs(gdlat) gt 90.) then begin
	   print, 'ERAD84: Given Latitude Is Out of Range: ', gdlat
	   erad84 = -1.
	endif

	gclat = geocen(gdlat)
        ;converts to geocentric
	dxe = dmajor * cos(degrad * gclat)
	dxe2 = dxe*dxe
	dye = dratio*sqrt(dmaj2 - dxe2)
	erad84 = sqrt(dxe2 + dye*dye)
	return, erad84
end




;;*      rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
;	Subroutine VCONIC(vg,vplane,rot,vqonic)
;;*
;;cxr   sine,cosine,unit4
;;*
;;*   This routine was originally written in Meteorological Fortran
;;c   (Metefor).  Some of the original code still appears in statements
;;c   which have been commented out.  The original Metefor routine
;;c   (*.hlf) is better annotated.
;;*
;;*     21 Aug 98; shortened slightly
;;*     Imagine a cone whose axis is the vector VCONE.  Let the
;;c     vector VG be an element of the cone directed away from the
;;c     apex of the cone.  This vector-valued function rotates the
;;c     vector VG rightward (clockwise) about the axis VCONE, as
;;c     VCONE is pointed toward the viewer. If the cone is degenerate,
;;c     i.e. if it is a plane, then this function is the same as
;;c     VROT4.
;;*
;;c       More crudely, imagine looking into the interior of an empty
;;c     ice cream cone.  The vector VCONE is the axis of the cone,
;;c     directed toward the viewer.  Let the vector VG
;;c     be any element of the cone extending from the apex of
;;c     the cone toward the lip.  If the viewer now rotates the cone
;;c     'rot' degrees clockwise from his perspective, this function
;;c     computes the new vector value of VG after the rotation.
;;*     If rot<0, then the rotation is counter-clockwise.
;;*     The cone may be merely a plane, in which case VCONE and VG are
;;c     normal.  A practical application would be determining the direction
;;c     of view of a conically-scanning satellite.
;;*
;	Implicit None
;	real vq41(3),vq42(3),vq43(3),vq44(3),vq45(3)
;	real  uv(3),vplane(3),ur(3),vg(3),uh(3),sine,cosine,rot
;	real rq41,rq42,rq43,rq44,vqonic(3)
;;c     xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
pro VCONIC,vg,vplane,rot,vqonic

        ;;vg:fltarr(3) vplane:fltarr(3)
        vq41 =fltarr(3)  & vq42 =fltarr(3)  & vq43 =fltarr(3)  & vq44 =fltarr(3)  & vq45 =fltarr(3)  
	uv   =fltarr(3)  & ur   =fltarr(3)  & uh   =fltarr(3)  & vqonic =fltarr(3)  
	
	unit4,vplane,uv
	
	vq41(1-1) = vg(2-1)*uv(3-1) - uv(2-1)*vg(3-1)
	vq41(2-1) = vg(3-1)*uv(1-1) - uv(3-1)*vg(1-1)
	vq41(3-1) = vg(1-1)*uv(2-1) - uv(1-1)*vg(2-1)
	
	unit4,vq41,ur
	vq41(1-1) = uv(2-1)*ur(3-1) - ur(2-1)*uv(3-1)
	vq41(2-1) = uv(3-1)*ur(1-1) - ur(3-1)*uv(1-1)
	vq41(3-1) = uv(1-1)*ur(2-1) - ur(1-1)*uv(2-1)
	
	unit4,vq41,uh
	rq41=sine(rot)
	rq42=cosine(rot)
	rq43=uh(1-1)*vg(1-1)+uh(2-1)*vg(2-1)+uh(3-1)*vg(3-1)
	vq41(1-1) = ur(1-1)*rq41
	vq41(2-1) = ur(2-1)*rq41
	vq41(3-1) = ur(3-1)*rq41
	vq42(1-1) = uh(1-1)*rq42
	vq42(2-1) = uh(2-1)*rq42
	vq42(3-1) = uh(3-1)*rq42
	vq43(1-1) = vq41(1-1)+vq42(1-1)
	vq43(2-1) = vq41(2-1)+vq42(2-1)
	vq43(3-1) = vq41(3-1)+vq42(3-1)
	rq44 = vg(1-1)*uv(1-1)+vg(2-1)*uv(2-1)+vg(3-1)*uv(3-1)
	vq44(1-1) = vq43(1-1)*rq43
	vq44(2-1) = vq43(2-1)*rq43
	vq44(3-1) = vq43(3-1)*rq43
	vq45(1-1) = uv(1-1)*rq44
	vq45(2-1) = uv(2-1)*rq44
	vq45(3-1) = uv(3-1)*rq44
	vqonic(1-1) = vq44(1-1)+vq45(1-1)
	vqonic(2-1) = vq44(2-1)+vq45(2-1)
	vqonic(3-1) = vq44(3-1)+vq45(3-1)
	return
end

